/* $Id: Animator.java,v 1.2 2002/02/15 19:40:36 zcox Exp $
 *
 *
 */

package fcmodeler.animation;

import javax.swing.Timer;
import javax.swing.event.EventListenerList;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Arrays;
import java.util.Collections;
import java.util.EventListener;
import java.io.Reader;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import diva.graph.GraphView;

import fcmodeler.mapping.MappingUtility;
import fcmodeler.mapping.Sentence;
import fcmodeler.mapping.AtomicSentence;
import fcmodeler.mapping.CompositeSentence;
import fcmodeler.mapping.Connective;
import fcmodeler.mapping.AndConnective;
import fcmodeler.mapping.OrConnective;
import fcmodeler.mapping.NodeMapping;
import fcmodeler.mapping.EdgeMapping;
import fcmodeler.attributes.VisualAttribute;
import fcmodeler.attributes.VisualAttributeFactory;
import fcmodeler.attributes.VisualAttributeValue;
import fcmodeler.attributes.NodeVisualAttributeValue;
import fcmodeler.attributes.EdgeVisualAttributeValue;
import fcmodeler.util.NoTextNodeList;
import fcmodeler.util.ChildNodeList;

/** An animation controller.  This class uses a list of sets of node mappings and a list of sets of edge mappings, and applies
 * each set of mappings to all node and edge figures in the view one at a time to produce an
 * animation of changing visual attributes.  Each element in the lists represents one time step.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class Animator extends Object implements ActionListener {
	private List _nodeMappings = null;
	private List _edgeMappings = null;
	private GraphView _view = null;
	private int _frame = -1;
	private Timer _timer = null;
	private EventListenerList _listeners = null;
	private boolean _stopped = true;
	private boolean _paused = false;
	
	/** Creates a new <CODE>Animator</CODE> using the node and edge mappings in the XML obtained from
	 * the specified reader.
	 *
	 * @param reader the reader that provides the node and edge mappings in XML.
	 * @param view the graph view containing the node and edge figures.
	 */
	public Animator(Reader reader, GraphView view) {
		if (reader == null) { throw new NullPointerException("reader was null"); }
		if (view == null) { throw new NullPointerException("view was null"); }
		
		getMappings(reader);
		_view = view;
		_timer = new Timer(1000, this);
		_timer.setInitialDelay(0);
		_listeners = new EventListenerList();
	}
	
	/** Creates a new <CODE>Animator</CODE> using the specified node and edge mappings.
	 *
	 * @param nodeMappings the list of sets of node mappings.
	 * @param edgeMappings the list of sets of edge mappings.
	 * @param view the graph view containing the node and edge figures.
	 */
	public Animator(List nodeMappings, List edgeMappings, GraphView view) {
		if (nodeMappings == null) { throw new NullPointerException("nodeMappings was null"); }
		if (edgeMappings == null) { throw new NullPointerException("edgeMappings was null"); }
		if (nodeMappings.size() != edgeMappings.size()) { throw new IllegalArgumentException("node and edge mappings not same size"); }
		if (view == null) { throw new NullPointerException("view was null"); }
		
		_nodeMappings = nodeMappings;
		_edgeMappings = edgeMappings;
		_view = view;
		_timer = new Timer(1000, this);
		_timer.setInitialDelay(0);
		_listeners = new EventListenerList();
	}
	
	/** Invoked by the timer when the next frame of the animation should be shown.
	 *
	 * @param actionEvent totally ignored by this method.
	 */
	public void actionPerformed(ActionEvent actionEvent) {
		if (isValidFrame(_frame+1)) {
			showFrame(++_frame);
		}
		
		if (!isValidFrame(_frame+1)) {
			stop();
		}
	}
	
	/** Registers the specified animation listener to receive animation events from this animator.
	 *
	 * @param listener the animation listener to register with this animator.
	 */
	public void addAnimationListener(AnimationListener listener) {
		_listeners.add(AnimationListener.class, listener);
	}
	
	/** Private recursive method for building a sentence.
	 *
	 * @param sentenceElement either an atom or composite sentence element.
	 * @return an appropriate sentence.
	 */	
	private Sentence buildSentence(Element sentenceElement) {
		Sentence sentence = null;
		if (sentenceElement.getTagName().equals("atom")) {
			sentence = new AtomicSentence(sentenceElement.getAttribute("property"), sentenceElement.getAttribute("value"));
		}
		else if (sentenceElement.getTagName().equals("composite")) {
			NodeList children = new NoTextNodeList(sentenceElement.getChildNodes());
			Element firstElement = (Element) children.item(0);
			Sentence first = buildSentence(firstElement);
			Element connectiveElement = (Element) children.item(1);
			Connective connective = connectiveElement.getAttribute("type").equals("and") ? (Connective) AndConnective.getInstance() : (Connective) OrConnective.getInstance();
			Element secondElement = (Element) children.item(2);
			Sentence second = buildSentence(secondElement);
			sentence = new CompositeSentence(first, connective, second);
		}
		return sentence;
	}
	
	/** Notifies all animation listeners that the animation was paused.
	 *
	 */
	protected void fireAnimationPaused() {
		AnimationEvent event = new AnimationEvent(this, _frame);
		for (Iterator iterator = getAnimationListeners(); iterator.hasNext();) {
			((AnimationListener) iterator.next()).animationPaused(event);
		}
	}
	
	/** Notifies all animation listeners that the animation was started.
	 *
	 */
	protected void fireAnimationStarted() {
		AnimationEvent event = new AnimationEvent(this, _frame);
		for (Iterator iterator = getAnimationListeners(); iterator.hasNext();) {
			((AnimationListener) iterator.next()).animationStarted(event);
		}
	}
	
	/** Notifies all animation listeners that the animation was stopped.
	 *
	 */
	protected void fireAnimationStopped() {
		AnimationEvent event = new AnimationEvent(this, _frame);
		for (Iterator iterator = getAnimationListeners(); iterator.hasNext();) {
			((AnimationListener) iterator.next()).animationStopped(event);
		}
	}
	
	/** Notifies all animation listeners that a new frame was shown.
	 *
	 */
	protected void fireFrameShown() {
		AnimationEvent event = new AnimationEvent(this, _frame);
		for (Iterator iterator = getAnimationListeners(); iterator.hasNext();) {
			((AnimationListener) iterator.next()).frameShown(event);
		}
	}
	
	/** Returns an iterator over the animation listeners registered with this animator.
	 *
	 * @return an iterator over the animation listeners registered with this animator.
	 */
	public Iterator getAnimationListeners() {
		return Arrays.asList(_listeners.getListeners(AnimationListener.class)).iterator();
	}
	
	/** Returns the delay between animation steps.
	 *
	 * @return the delay between animation steps.
	 */
	public int getDelay() {
		return _timer.getDelay();
	}
	
	/** Returns the list of sets of edge mappings.
	 *
	 * @return the list of sets of edge mappings.
	 */
	public List getEdgeMappings() {
		return _edgeMappings;
	}
	
	/** Returns the current frame being shown by this animator.
	 *
	 * @return the current frame being shown by this animator.
	 */
	public int getFrame() {
		return _frame;
	}
	
	/** Returns the total number of frames to be shown by this animator.
	 *
	 * @return the total number of frames to be shown by this animator.
	 */
	public int getFrameCount() {
		return _nodeMappings.size();
	}
	
	/** Returns the graph view used by this animator.
	 *
	 * @return the graph view used by this animator.
	 */
	public GraphView getGraphView() {
		return _view;
	}
	
	/** Private method for extracting the node and edge mappings from the XML provided by the reader.
	 *
	 * @param reader the reader providing the XML.
	 */	
	private void getMappings(Reader reader) {
		try {
			DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = builderFactory.newDocumentBuilder();
			Document document = builder.parse(new InputSource(reader));
			Element root = document.getDocumentElement();
			NodeList timeList = new ChildNodeList(root, "time");
			_nodeMappings = new ArrayList(Collections.nCopies(timeList.getLength(), new Object()));
			_edgeMappings = new ArrayList(_nodeMappings);
			for (int i=0, length=timeList.getLength(); i<length; i++) {
				Element timeElement = (Element) timeList.item(i);
				Set nodeMappings = new HashSet();
				Set edgeMappings = new HashSet();
				NodeList mappingList = new ChildNodeList(timeElement, "mapping");
				for (int j=0, length2=mappingList.getLength(); j<length2; j++) {
					Element mappingElement = (Element) mappingList.item(j);
					NodeList nodeList = new NoTextNodeList(mappingElement.getChildNodes());
					Element sentenceElement = (Element) nodeList.item(0);
					Sentence sentence = buildSentence(sentenceElement);
					Element attributeValueElement = (Element) mappingElement.getElementsByTagName("attributeValue").item(0);
					VisualAttribute attribute = VisualAttributeFactory.getInstance().getAttribute(attributeValueElement.getAttribute("attribute"));
					VisualAttributeValue value = attribute.getValue(attributeValueElement.getAttribute("value"));
					if (mappingElement.getAttribute("type").equals("node")) {
						nodeMappings.add(new NodeMapping(sentence, (NodeVisualAttributeValue) value));
					}
					else if (mappingElement.getAttribute("type").equals("edge")) {
						edgeMappings.add(new EdgeMapping(sentence, (EdgeVisualAttributeValue) value));
					}
				}
				_nodeMappings.set(i, nodeMappings);
				_edgeMappings.set(i, edgeMappings);
			}
			
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/** Returns the list of sets of node mappings.
	 *
	 * @return the list of sets of node mappings.
	 */
	public List getNodeMappings() {
		return _nodeMappings;
	}
	
	/** Returns <CODE>true</CODE> if the specified frame number is valid.
	 *
	 * @param frame the frame number to test for validity.
	 * @return <CODE>true</CODE> if <CODE>frame >= 0</CODE> and <CODE>frame < getFrameCount()</CODE>.
	 */
	public boolean isValidFrame(int frame) {
		return frame >= 0 && frame < getFrameCount();
	}
	
	/** Returns <CODE>true</CODE> if the animation is paused.
	 *
	 * @return <CODE>true</CODE> if the animation is paused.
	 */
	public boolean isPaused() {
		return _paused;
	}
	
	/** Returns <CODE>true</CODE> if the animation is stopped.
	 *
	 * @return <CODE>true</CODE> if the animation is stopped.
	 */
	public boolean isStopped() {
		return _stopped;
	}
	
	/** Shows the next frame of the animation.
	 *
	 */
	public void next() {
		if (isValidFrame(_frame+1)) {
			showFrame(++_frame);
		}
	}
	
	/** Pauses the animation.
	 *
	 */
	public void pause() {
		if (!_stopped) {
			_paused = true;
			_timer.stop();
			fireAnimationPaused();
		}
	}
	
	/** Shows the previous frame of the animation.
	 *
	 */
	public void previous() {
		if (isValidFrame(_frame-1)) {
			showFrame(--_frame);
		}
	}
	
	/** Removes the specified animation listener from the list of listeners for this animator.
	 *
	 * @param listener the animation listener to remove.
	 */
	public void removeAnimationListener(AnimationListener listener) {
		_listeners.remove(AnimationListener.class, listener);
	}
	
	/** Sets the delay between animation steps to the specified value.
	 *
	 * @param delay the delay between animation steps in milliseconds.
	 */
	public void setDelay(int delay) {
		_timer.setDelay(delay);
	}
	
	/** Shows the specified frame of the animation.
	 *
	 * @param frame the frame to show.
	 */
	public void showFrame(int frame) {
		if (isValidFrame(frame)) {
			Set nodeMappings = (Set) _nodeMappings.get(frame);
			Set edgeMappings = (Set) _edgeMappings.get(frame);
			MappingUtility.usePermanents(_view);
			MappingUtility.map(_view.getGraphModel().getGraph(), nodeMappings, edgeMappings, _view, false);
			fireFrameShown();
		}
	}
	
	/** Starts the animation.
	 *
	 */
	public void start() {
		if (!_timer.isRunning()) {
			if (_paused) {
				_timer.start();
				_paused = false;
			}
			else if (_stopped) {
				_frame = -1;
				_timer.restart();
				_stopped = false;
			}
			fireAnimationStarted();
		}
	}
	
	/** Stops the animation.
	 *
	 */
	public void stop() {
		if (!_stopped) {
			_stopped = true;
			_paused = false;
			_timer.stop();
			fireAnimationStopped();
		}
	}
}

/*
 * $Log: Animator.java,v $
 * Revision 1.2  2002/02/15 19:40:36  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:41:02  zcox
 * initial commit of animation package
 *
 *
 *
 */
