/* $Id: ColorValue.java,v 1.2 2002/02/16 03:01:36 zcox Exp $
 *
 *
 * Created on March 6, 2001, 3:15 PM
 */

package fcmodeler.attributes;

import java.awt.Color;

import fcmodeler.view.EdgeFigure;
import fcmodeler.view.NodeFigure;

/** A specific value of the color attribute.  This class can represent both the color of an edge figure and the outline color
 * of a node figure.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public final class ColorValue extends Object implements VisualAttributeValue, EdgeVisualAttributeValue, NodeVisualAttributeValue {
	private final Color _color;
	private String _string = null;
	
	/** Creates a new <CODE>ColorValue</CODE> representing the color black.
	 *
	 */
	public ColorValue() {
		_color = Color.black;
		calculateString();
	}
	
	/** Creates a new <CODE>ColorValue</CODE> using the specified color.
	 *
	 * @param color the color to be used in this color value.
	 */	
	public ColorValue(Color color) {
		_color = new Color(color.getRed(), color.getGreen(), color.getBlue(), color.getAlpha());
		calculateString();
	}
	
	/** Creates a new <CODE>ColorValue</CODE> using the specified string description.  The string must
	 * either be a color name contained in <CODE>ColorFactory</CODE> or the rgb values for a color
	 * specified as "[red, green, blue]".
	 *
	 * @param color the string description of this color value.
	 */	
	public ColorValue(String color) {
		float red = 0.0f, green = 0.0f, blue = 0.0f;
		int firstComma = 0, secondComma = 0;
		
		if (color.startsWith("[")) {
			firstComma = color.indexOf(',');
			red = Float.parseFloat(color.substring(1, firstComma).trim());
			secondComma = color.indexOf(',', firstComma+1);
			green = Float.parseFloat(color.substring(firstComma+1, secondComma).trim());
			blue = Float.parseFloat(color.substring(secondComma+1, color.length()-1).trim());
			_color = new Color(red, green, blue);
		}
		else {
			_color = ColorFactory.getInstance().getColor(color);
		}
		
		if (_color == null) { throw new IllegalArgumentException(color + " is not a valid color String"); }
		
		calculateString();
	}
	
	private void calculateString() {
		float[] rgb = _color.getRGBComponents(null);
		//_string = "attribute=\"color\" value=\"[" + rgb[0] + "," + rgb[1] + "," + rgb[2] + "]\"";// <----- may be wrong!!!!
		
		//may want to store the "name" of the color, if that was used in construction instead of rgb values
		_string = "[" + rgb[0] + "," + rgb[1] + "," + rgb[2] + "]";
	}
	
	/** Compares the specified object with this color value for equality.
	 *
	 * @param o the object to be compared for equality with this color value.
	 * @return <CODE>true</CODE> if the specified object is equal to this color value.
	 */	
	public boolean equals(Object o) {
		if (o == this) 
			return true;
		
		if (!(o instanceof ColorValue))
			return false;
		
		ColorValue other = (ColorValue) o;
		return _color.equals(other.getColor());
	}
	
	public VisualAttribute getAttribute() {
		return ColorAttribute.getInstance();
	}
	
	/** Returns the color used in this color value.
	 *
	 * @return the color used in this color value.
	 */	
	public Color getColor() {
		return _color;
	}
	
	/** Returns the hash code for this color value.
	 *
	 * @return the hash code for this color value.
	 */
	public int hashCode() {
		int result = 17;
		
		result = 37*result + _color.hashCode();
		
		return result;
	}
	
	/** Sets the color of the specified edge figure to the color contained in this color value.
	 *
	 * @param figure the edge figure to update.
	 */
	public void setEdgeValue(EdgeFigure figure) {
		figure.getFigure().outlinePaint = _color;
	}
	
	/** Sets the outline color of the specified node figure to the color contained in this color value.
	 *
	 * @param figure the node figure to update.
	 */
	public void setNodeValue(NodeFigure figure) {
		figure.getFigure().setStrokePaint(_color);
	}
	
	/** Sets the permanent color of the specified edge figure to the color contained in this color value.
	 *
	 * @param figure the edge figure to update.
	 */
	public void setPermanentEdgeValue(EdgeFigure figure) {
		figure.getPermanentFigure().outlinePaint = _color;
	}
	
	/** Sets the permanent outline color of the specified node figure to the color contained in this color value.
	 *
	 * @param figure the node figure to update.
	 */
	public void setPermanentNodeValue(NodeFigure figure) {
		figure.getPermanentFigure().setStrokePaint(_color);
	}
	
	/** Returns a string representation of this color value.
	 *
	 * @return a string representation of this color value.
	 */	
	public String toString() {
		return _string;
	}
	
	/** Returns a new <CODE>ColorValue</CODE> instance, created using the specified string description.
	 *
	 * @param value the string description to be used in creating the new <CODE>ColorValue</CODE> instance.
	 * @return the new <CODE>ColorValue</CODE> instance.
	 */	
	public static ColorValue valueOf(String value) {
		//could optimize this by caching commonly used colors
		return new ColorValue(value);
	}
	
}

/* 
 * $Log: ColorValue.java,v $
 * Revision 1.2  2002/02/16 03:01:36  zcox
 * removed fcmodeler.view.ColorFactory
 *
 * Revision 1.1  2002/02/15 19:54:35  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:51:28  zcox
 * initial commit of attributes package
 *
 * Revision 1.2  2001/05/22 16:46:49  zcox
 * code clean-up
 *
 * Revision 1.1  2001/04/04 02:07:32  zcox
 * initial commit of fcmodeler.view.propviz.attributes package
 *
 *
 *
 */
