/* $Id: DashValue.java,v 1.1 2002/02/15 19:54:35 zcox Exp $
 *
 *
 * Created on April 9, 2001, 2:02 PM
 */

package fcmodeler.attributes;

import java.awt.BasicStroke;

import fcmodeler.view.EdgeFigure;

/** A specific value of the dash attribute.  This class represents a dash array, with the first
 * element being the "on" value and the second element being the "off" value.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class DashValue extends Object implements VisualAttributeValue, EdgeVisualAttributeValue {
	private final float[] _dash;
	private String _string = null;
	
	/** Creates a new <CODE>DashValue</CODE> representing the dash array [2.0,2.0].
	 *
	 */
	public DashValue() {
		_dash = new float[] {2.0f, 2.0f};
		calculateString();
	}
	
	/** Creates a new <CODE>DashValue</CODE> using the specified array.  If both elements of the 
	 * array are equal to 0.0f, the dash value represents a solid line.
	 *
	 * @param dash the dash array to be used in this dash value.
	 */		
	public DashValue(float[] dash) {
		_dash = new float[] {dash[0], dash[1]};
		calculateString();
	}
	
	/** Creates a new <CODE>DashValue</CODE> using the specified string description.  The string must
	 * be formatted as "[on,off]".
	 *
	 * @param dash the string representation of this dash value.
	 */	
	public DashValue(String dash) {
		float on = 0.0f, off = 0.0f;
		int firstComma = 0;
		
		firstComma = dash.indexOf(',');
		on = Float.parseFloat(dash.substring(1, firstComma).trim());
		off = Float.parseFloat(dash.substring(firstComma+1, dash.length()-1).trim());
		_dash = new float[] {on, off};
		calculateString();
	}
	
	private void calculateString() {
		_string = "[" + _dash[0] + "," + _dash[1] + "]";
	}
	
	/** Compares the specified object with this dash value for equality.
	 *
	 * @param o the object to be compared for equality with this dash value.
	 * @return <CODE>true</CODE> if the specified object is equal to this dash value.
	 */	
	public boolean equals(Object o) {
		if (o == this) 
			return true;
		
		if (!(o instanceof DashValue))
			return false;
		
		DashValue other = (DashValue) o;
		float[] otherDash = other.getDash();
		return _dash[0] == otherDash[0] && _dash[1] == otherDash[1];
	}
	
	public VisualAttribute getAttribute() {
		return DashAttribute.getInstance();
	}
	
	/** Returns the array used in this dash value.
	 * @return the array used in this dash value.
	 */	
	public float[] getDash() {
		//is a defensive copy necessary here?
		return _dash;
	}
	
	/** Returns the hash code for this visual attribute value.
	 *
	 * @return the hash code for this visual attribute value.
	 */	
	public int hashCode() {
		int result = 17;
		
		result = 37*result + Float.floatToIntBits(_dash[0]);
		result = 37*result + Float.floatToIntBits(_dash[1]);
		
		return result;
	}
	
	/** Sets the dash array of the specified edge figure to the array contained in this dash value.
	 *
	 * @param figure the edge figure to update.
	 */
	public void setEdgeValue(EdgeFigure figure) {
		//all zeros means use a solid line, not a dashed line
		if (_dash[0] == 0.0f && _dash[1] == 0.0f) {
			//assume it's a BasicStroke
			BasicStroke stroke = (BasicStroke) figure.getFigure().getStroke();
			figure.getFigure().setStroke(new BasicStroke(stroke.getLineWidth(), stroke.getEndCap(), stroke.getLineJoin(), 
				stroke.getMiterLimit()));
		}
		else {
			figure.getFigure().setDashArray(_dash);
		}
	}
	
	/** Sets the permanent dash array of the specified edge figure to the array contained in this dash value.
	 *
	 * @param figure the edge figure to update.
	 */
	public void setPermanentEdgeValue(EdgeFigure figure) {
		//all zeros means use a solid line, not a dashed line
		if (_dash[0] == 0.0f && _dash[1] == 0.0f) {
			//assume it's a BasicStroke
			BasicStroke stroke = (BasicStroke) figure.getPermanentFigure().getStroke();
			figure.getPermanentFigure().setStroke(new BasicStroke(stroke.getLineWidth(), stroke.getEndCap(), stroke.getLineJoin(), 
				stroke.getMiterLimit()));
		}
		else {
			figure.getPermanentFigure().setDashArray(_dash);
		}
	}
	
	/** Returns a string representation of this visual attribute value.
	 *
	 * @return a string representation of this visual attribute value.
	 */	
	public String toString() {
		return _string;
	}
	
	/** Returns a new <CODE>DashValue</CODE> instance, created using the specified string description.
	 *
	 * @param value the string description to be used in creating the new <CODE>DashValue</CODE> instance.
	 * @return the new <CODE>DashValue</CODE> instance.
	 */	
	public static DashValue valueOf(String value) {
		//could optimize this by caching commonly used dashes
		return new DashValue(value);
	}
	
}

/* 
 * $Log: DashValue.java,v $
 * Revision 1.1  2002/02/15 19:54:35  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:51:29  zcox
 * initial commit of attributes package
 *
 * Revision 1.2  2001/05/22 16:54:59  zcox
 * code clean-up
 *
 *
 *
 *
 */
