/* $Id: FillValue.java,v 1.2 2002/02/16 03:01:36 zcox Exp $
 *
 *
 * Created on April 9, 2001, 1:40 PM
 */

package fcmodeler.attributes;

import java.awt.Color;

import fcmodeler.view.NodeFigure;

/** A specific value of the fill attribute.
 *
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public final class FillValue extends Object implements VisualAttributeValue, NodeVisualAttributeValue {
	private final Color _color;
	private String _string = null;
	
	/** Creates a new <CODE>FillValue</CODE> representing the color white.
	 *
	 */
	public FillValue() {
		_color = Color.white;
		calculateString();
	}
	
	/** Creates a new <CODE>FillValue</CODE> using the specified color.
	 *
	 * @param color the color to be used in this color value.
	 */	
	public FillValue(Color color) {
		_color = new Color(color.getRed(), color.getGreen(), color.getBlue(), color.getAlpha());
		calculateString();
	}
	
	/** Creates a new <CODE>FillValue</CODE> using the specified string description.  The string must
	 * either be a color name contained in <CODE>ColorFactory</CODE> or the rgb values for a color
	 * specified as "[red, green, blue]".
	 *
	 * @param color the string description of this color value.
	 */	
	public FillValue(String color) {
		float red = 0.0f, green = 0.0f, blue = 0.0f;
		int firstComma = 0, secondComma = 0;
		
		if (color.startsWith("[")) {
			firstComma = color.indexOf(',');
			red = Float.parseFloat(color.substring(1, firstComma).trim());
			secondComma = color.indexOf(',', firstComma+1);
			green = Float.parseFloat(color.substring(firstComma+1, secondComma).trim());
			blue = Float.parseFloat(color.substring(secondComma+1, color.length()-1).trim());
			_color = new Color(red, green, blue);
		}
		else {
			_color = ColorFactory.getInstance().getColor(color);
		}
		
		if (_color == null) { throw new IllegalArgumentException(color + " is not a valid color String"); }
		
		calculateString();
	}
	
	private void calculateString() {
		float[] rgb = _color.getRGBComponents(null);
		//_string = "attribute=\"color\" value=\"[" + rgb[0] + "," + rgb[1] + "," + rgb[2] + "]\"";// <----- may be wrong!!!!
		
		//may want to store the "name" of the color, if that was used in construction instead of rgb values
		_string = "[" + rgb[0] + "," + rgb[1] + "," + rgb[2] + "]";
	}
	
	/** Compares the specified object with this fill value for equality.
	 *
	 * @param o the object to be compared for equality with this fill value.
	 * @return <CODE>true</CODE> if the specified object is equal to this fill value.
	 */	
	public boolean equals(Object o) {
		if (o == this) 
			return true;
		
		if (!(o instanceof FillValue))
			return false;
		
		FillValue other = (FillValue) o;
		return _color.equals(other.getColor());
	}
	
	public VisualAttribute getAttribute() {
		return FillAttribute.getInstance();
	}
	
	/** Returns the color used in this fill value.
	 *
	 * @return the color used in this fill value.
	 */	
	public Color getColor() {
		return _color;
	}
	
	/** Returns the hash code for this fill value.
	 *
	 * @return the hash code for this fill value.
	 */
	public int hashCode() {
		int result = 17;
		
		result = 37*result + _color.hashCode();
		
		return result;
	}
	
	/** Sets the fill color of the specified node figure to the color contained in this fill value.
	 *
	 * @param figure the node figure to update.
	 */
	public void setNodeValue(NodeFigure figure) {
		figure.getFigure().setFillPaint(_color);
	}
	
	/** Sets the permanent fill color of the specified node figure to the color contained in this fill value.
	 *
	 * @param figure the node figure to update.
	 */
	public void setPermanentNodeValue(NodeFigure figure) {
		figure.getPermanentFigure().setFillPaint(_color);
	}
	
	/** Returns a string representation of this fill value.
	 *
	 * @return a string representation of this fill value.
	 */	
	public String toString() {
		return _string;
	}
	
	/** Returns a new <CODE>FillValue</CODE> instance, created using the specified string description.
	 *
	 * @param value the string description to be used in creating the new <CODE>FillValue</CODE> instance.
	 * @return the new <CODE>FillValue</CODE> instance.
	 */	
	public static FillValue valueOf(String value) {
		//could optimize this by caching commonly used colors
		return new FillValue(value);
	}
}

/* 
 * $Log: FillValue.java,v $
 * Revision 1.2  2002/02/16 03:01:36  zcox
 * removed fcmodeler.view.ColorFactory
 *
 * Revision 1.1  2002/02/15 19:54:36  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 19:51:29  zcox
 * initial commit of attributes package
 *
 * Revision 1.2  2001/05/22 16:54:59  zcox
 * code clean-up
 *
 *
 *
 *
 */
