/* $Id: NodeShapeValue.java,v 1.2 2002/03/22 16:54:30 xein Exp $
 *
 *
 * Created on March 29, 2001, 8:51 PM
 */

package fcmodeler.attributes;

import fcmodeler.view.NodeFigure;
import fcmodeler.view.figures.NodeShape;
import fcmodeler.view.figures.RectangleNodeShape;
import fcmodeler.view.figures.RoundRectangleNodeShape;
import fcmodeler.view.figures.EllipseNodeShape;
import fcmodeler.view.figures.TinyRectangleNodeShape;
import fcmodeler.view.figures.TestNodeShape;
import fcmodeler.view.figures.DiamondNodeShape; //xein

/** A specific value of the node shape attribute.
 *
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public final class NodeShapeValue extends Object implements VisualAttributeValue, NodeVisualAttributeValue {
	private final NodeShape _nodeShape;

	/** Creates a new <CODE>NodeShapeValue</CODE> using a rectangle node shape.
	 *
	 */
	public NodeShapeValue() {
		_nodeShape = new RectangleNodeShape();
	}

	/** Creates a new <CODE>NodeShapeValue</CODE> using the specified node shape.  The supplied
	 * <CODE>NodeShape</CODE> is cloned in order to create a defensive copy.
	 *
	 * @param nodeShape the node shape to use for this node shape value.
	 */
	public NodeShapeValue(NodeShape nodeShape) {
		try {
			_nodeShape = (NodeShape) nodeShape.clone();
		} catch (CloneNotSupportedException e) {
			throw new UnsupportedOperationException(nodeShape + " does not support the clone method - BUT IT SHOULD!!!");
		}
	}

	/** Creates a new <CODE>NodeShapeValue</CODE> using the specified string description of the node shape.
	 *
	 * @param nodeShape the string description of the node shape for this node shape value.
	 */
	public NodeShapeValue(String nodeShape) {
		if (nodeShape.equals("rectangle")) {
			_nodeShape = new RectangleNodeShape();
		}
		else if (nodeShape.equals("round rectangle")) {
			_nodeShape = new RoundRectangleNodeShape();
		}
		else if (nodeShape.equals("ellipse")) {
			_nodeShape = new EllipseNodeShape();
		}
		else if (nodeShape.equals("tiny rectangle")) {
			_nodeShape = new TinyRectangleNodeShape();
		}
		else if (nodeShape.equals("test")) {
			_nodeShape = new TestNodeShape();
		}
    //xein
    else if (nodeShape.equals("diamond")) {
      _nodeShape = new DiamondNodeShape();
    }
    //xein
		else {
			throw new IllegalArgumentException(nodeShape + " is not a valid node shape");
		}
	}

	/** Compares the specified object with this node shape value for equality.
	 *
	 * @param o the object to be compared for equality with this node shape value.
	 * @return <CODE>true</CODE> if the specified object is equal to this node shape value.
	 */
	public boolean equals(Object o) {
		if (o == this)
			return true;

		if (!(o instanceof NodeShapeValue))
			return false;

		NodeShapeValue other = (NodeShapeValue) o;
		return _nodeShape.equals(other.getNodeShape());
	}

	public VisualAttribute getAttribute() {
		return NodeShapeAttribute.getInstance();
	}

	/** Returns the node shape of this node shape value.
	 * @return the node shape of this node shape value.
	 */
	public NodeShape getNodeShape() {
		return _nodeShape;
	}

	/** Returns the hash code for this node shape value.
	 *
	 * @return the hash code for this node shape value.
	 */
	public int hashCode() {
		int result = 17;

		result = 37*result + _nodeShape.hashCode();

		return result;
	}

	/** Sets the node shape of the specified node figure to the shape contained in this node shape value.
	 * The supplied <CODE>NodeFigure</CODE> receives a copy of the internal <CODE>NodeShape</CODE> each
	 * time this method is called.
	 *
	 * @param figure the node figure to update.
	 */
	public void setNodeValue(NodeFigure figure) {
		NodeShape shape = null;
		try {
			shape = (NodeShape) _nodeShape.clone();
		} catch (CloneNotSupportedException e) {
			throw new UnsupportedOperationException(shape + " does not support the clone method - BUT IT SHOULD!!!");
		}
		figure.setNodeShape(shape);
	}

	/** Sets the permanent node shape of the specified node figure to the shape contained in this node shape value.
	 * The supplied <CODE>NodeFigure</CODE> receives a copy of the internal <CODE>NodeShape</CODE> each
	 * time this method is called.
	 *
	 * @param figure the node figure to update.
	 */
	public void setPermanentNodeValue(NodeFigure figure) {
		NodeShape shape = null;
		try {
			shape = (NodeShape) _nodeShape.clone();
		} catch (CloneNotSupportedException e) {
			throw new UnsupportedOperationException(shape + " does not support the clone method - BUT IT SHOULD!!!");
		}
		figure.setPermanentNodeShape(shape);
	}

	/** Returns a string representation of this node shape value.
	 *
	 * @return a string representation of this node shape value.
	 */
	public String toString() {
		return _nodeShape.toString();
	}

	/** Returns a new <CODE>NodeShapeValue</CODE> instance, created using the specified string description.
	 *
	 * @param value the string description to be used in creating the new <CODE>NodeShapeValue</CODE> instance.
	 * @return the new <CODE>NodeShapeValue</CODE> instance.
	 */
	public static NodeShapeValue valueOf(String value) {
		return new NodeShapeValue(value);
	}

}

/*
 * $Log: NodeShapeValue.java,v $
 * Revision 1.2  2002/03/22 16:54:30  xein
 * - Changed NodeShapeEditor and NodeShapeValue to include   DiamondNodeShape
 * - BezierEdgeFigureRenderer was used to test FilledRectangleHead connector   end
 * - No change to RectangleNodeShape
 *
 * Revision 1.1  2002/02/15 19:54:36  zcox
 * big time update
 *
 * Revision 1.2  2002/01/18 19:53:30  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.1  2002/01/15 19:51:29  zcox
 * initial commit of attributes package
 *
 * Revision 1.2  2001/05/22 16:46:49  zcox
 * code clean-up
 *
 * Revision 1.1  2001/04/04 02:07:32  zcox
 * initial commit of fcmodeler.view.propviz.attributes package
 *
 *
 *
 */
