/* $Id: BezierConnectorInteractor.java,v 1.1 2002/06/28 08:06:03 adamtj Exp $
 *
 */

package fcmodeler.interaction.BezierConnector;

import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import diva.canvas.Site;
import diva.canvas.event.LayerEvent;
import diva.canvas.interactor.DragInteractor;
import diva.canvas.manipulator.GrabHandle;

import fcmodeler.interaction.GeneralConnector.GeneralConnector;
import fcmodeler.interaction.GeneralConnector.GeneralConnectorInteractor;
import fcmodeler.interaction.GeneralConnector.GeneralConnectorGrabHandle;
import fcmodeler.interaction.GeneralConnector.GeneralConnectorManipulator;
import fcmodeler.interaction.GeneralConnector.GeneralConnectorSite;

/**
 * BezierConnectorInteractor is the DragInteractor that controls the dragging of
 * BezierGrabHandles.  This links multiple grab handles together to provide better
 * useability.  Also, if you double click on a grab handle, it will lock and
 * unlock it to other related grab handles.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class BezierConnectorInteractor extends GeneralConnectorInteractor {
	
	/** the last handle that was clicked on */
	protected GrabHandle _lastHandle = null;
	/** the time of the last click in milliseconds since the epoch. */
	protected long _lastTime = (new Date()).getTime();
	/** the max delay between clicks in milliseconds */
	protected long _doubleClickDelay = 500;
	
	/** get mouse clicks and check for a double click */
	public void mousePressed(LayerEvent e) {
		//Check for a double click.
		long curTime = (new Date()).getTime();
		GrabHandle curHandle = (GrabHandle) e.getFigureSource();
		
		//Make sure the grab handle is lockable.
		if ( !(curHandle instanceof GeneralConnectorGrabHandle) ) {
			throw new UnsupportedOperationException(
			"BezierConnectorInteractor: can only lock BezierGrabHandles");
		}
		
		//if the double click was fast enough
		if (curTime-_lastTime < _doubleClickDelay) {
			//If this handle was the same as the previously click grab handle.
			if (curHandle == _lastHandle) {
				//lock or unlock the related grab handles.
				((BezierGrabHandle)curHandle).flipLockState();
				//Do this to avoid triple clicks.
				curTime = 0;
			}
		}
		
		_lastHandle = curHandle;
		_lastTime = curTime;
		
		//Do normal processing.
		super.mousePressed(e);
	}
	
	/** Translate a grab handle in response to a mouse drag event */
	public void translate(LayerEvent e, double x, double y) {
		GrabHandle g = (GrabHandle) e.getFigureSource();
		
		if ( !(g instanceof BezierGrabHandle) ) {
			throw new UnsupportedOperationException(
			"BezierConnectorInteractor: can only translate BezierGrabHandles");
		}
		
		moveHandles((BezierGrabHandle)g, x, y);
		
		// Reroute the connector.
		GeneralConnectorManipulator parent = (GeneralConnectorManipulator) g.getParent();
		((GeneralConnector)parent.getChild()).reroute();
	}
	
	/** Do all the work for translating the specified grab handle as well as other
	 *  grab handles that are affected by its motion.
	 */
	protected void moveHandles(BezierGrabHandle g, double dx, double dy) {
		//This is a long and ugly function, but I can't think of a good way to break
		//it into pieces.
		
		//Get the connector with which we are dealing.
		GeneralConnector connector = (GeneralConnector)((GeneralConnectorManipulator)g.getParent()).getChild();
		
		//We need to get grab handles that are next to this one.  We'll get the list
		//of Sites from the connector.  From the sites we can get the grab handles.
		//So, save the list, and also find the position of this handle in the list.
		List vertices = connector.getVertices();
		Iterator i = vertices.iterator();
		int pos=0;
		while (i.hasNext()) {
			pos++;
			if ( g == ((GeneralConnectorGrabHandle)((Site)i.next()).getFigure()) ) {
				pos--;
				break;
			}
		}
		
		//If we click on a CTRL_POINT, sister will be the corresponding grab handle
		//on the other side of the SEG_END.
		GrabHandle sister = null;
		//These are the coordinates of the center of rotation, the SEG_END point.
		double rotOriginX = 0;
		double rotOriginY = 0;
		
		//Get this site and handle, and the sites and handles on the right and left
		//if possible.
		GeneralConnectorSite thisSite = (GeneralConnectorSite)vertices.get(pos);
		BezierGrabHandle     thisHandle = (BezierGrabHandle) thisSite.getFigure();
		
		GeneralConnectorSite leftSite = null;
		BezierGrabHandle     leftHandle = null;
		
		GeneralConnectorSite rightSite = null;
		BezierGrabHandle     rightHandle = null;
		
		if (pos > 0) {
			leftSite = (GeneralConnectorSite)vertices.get(pos-1);
			leftHandle = (BezierGrabHandle) leftSite.getFigure();
		}
		if (pos < vertices.size()-1) {
			rightSite = (GeneralConnectorSite)vertices.get(pos+1);
			rightHandle = (BezierGrabHandle) rightSite.getFigure();
		}
		
		//If this is either the first or the last grab handle, we'll do nothing
		//and just drag it normally.
		if (leftSite != null && rightSite != null) {
			//Check on the left and right for SEG_ENDs.  If we find one,
			//then we'll have a sister CTRL_POINT that must be updated as this
			//one is dragged.
			if ( leftSite.getID() == BezierShapeFactory.SEG_END ) {
				//Save the point about which we'll rotate the sister grab handle.
				rotOriginX = leftSite.getX();
				rotOriginY = leftSite.getY();
				//Find out if we are locked to our sister.
				if ( ((BezierGrabHandle)leftSite.getFigure()).getLockState()) {
					sister = (GrabHandle)((GeneralConnectorSite)vertices.get(pos-2)).getFigure();
				}
			} else if ( rightSite.getID() == BezierShapeFactory.SEG_END ) {
				//Save the point about which we'll rotate the sister grab handle.
				rotOriginX = rightSite.getX();
				rotOriginY = rightSite.getY();
				//Find out if we are locked to our sister.
				if ( ((BezierGrabHandle)rightSite.getFigure()).getLockState()) {
					sister = (GrabHandle)((GeneralConnectorSite)vertices.get(pos+2)).getFigure();
				}
			} else if (thisSite.getID() == BezierShapeFactory.SEG_END) {
				//If we get here, this must be a SEG_END since a CTRL_POINT
				//always has a SEG_END on one side, and we aren't a HEAD or TAIL
				//since those are only on the ends for which we already checked.
				//If it is a SEG_END simply translate it.
				g.translate(dx,dy);
				//If we are locked to the other handles, move them as well.
				//They are guaranteed to not be null since this handle isn't a
				//HEAD or a TAIL.
				if (g.getLockState()) {
					leftHandle.translate(dx,dy);
					rightHandle.translate(dx,dy);
				}
				return;
			}
		}
		//If we get to this point, then the selected grab handle isn't a SEG_END
		//since we would have returned already if that were the case.
		
		//Get the coordinates of this site.
		double x = g.getSite().getX();
		double y = g.getSite().getY();
		
		//If we are not locked, sister will be null, so skip updating her.
		//If we don't have a sister, she will be null, so skip updating her.
		if (sister != null) {
			//Find the angle of the line between this grab handle and the
			//SEG_END about which we are rotating.  Also find the distance between
			//these two points.
			double base1 = x - rotOriginX;
			double height1 = y - rotOriginY;
			double angle1 = Math.atan2(base1, height1)+Math.PI;
			double length1 = Math.sqrt(base1*base1+height1*height1);
			
			//Now find the angle of the line between the SEG_END and the point to
			//which we will be translating this handle.  Also find the distance
			//between these two points.
			double base2 = (x+dx) - rotOriginX;
			double height2 = (y+dy) - rotOriginY;
			double angle2 = Math.atan2(base2, height2)+Math.PI;
			double length2 = Math.sqrt(base2*base2+height2*height2);
			//Find the difference between the previous two angles.  This will be the
			//angle through which this grab handle is being rotated.
			double theta = angle1 - angle2;
			//Find the change in the distance between the point and the SEG_END.
			double scale = (length2	- length1);
			
			//Build a rotation transformation.
			AffineTransform rot = new AffineTransform();
			//Rotate through our previously calculated angle and about the point
			//where the SEG_END resides.
			rot.setToRotation(theta, rotOriginX, rotOriginY);
			
			//Get the initial position of the sister.
			Point2D initialPos = new Point2D.Double(sister.getSite().getX(),sister.getSite().getY());
			Point2D finalPos = null;
			//transform() seems to either have a bug or be misdocumented.
			//It is supposed to set the second argument to the transformation of the
			//first point.  It doesn't and you need to get the transformation from
			//the return value.
			finalPos = rot.transform(initialPos, finalPos);
			
			//Build a vector that we will add to the translation of the sister node.
			//This vector will point along the line from the SEG_END to the sister
			//handle.  Its length will be equal to the amount by which the distance
			//from the SEG_END to the dragged handle is increasing.
			double scaleDX = 0;
			double scaleDY = 0;
			
			//There are problems with scaling in this manner.  Allow the user to
			//activate and deactivate it.
			if (g.getLockState()) {
				//Build a vector that points along the line from the SEG_END to the
				//sister's new position.  If it points along the line to the previous
				//position, then it will point in the wrong direction.
				scaleDX = finalPos.getX() - rotOriginX;
				scaleDY = finalPos.getY() - rotOriginY;
				//Find the length for normalizing.
				double len = Math.sqrt(scaleDX*scaleDX+scaleDY*scaleDY);
				//Avoid a divide by zero.
				if (len==0) {
					len = 0.0000000001;
				}
				//Normalize the vector.
				scaleDX = scaleDX/len;
				scaleDY = scaleDY/len;
				//Since the length is 1, multiply by the distance we want it to be.
				scaleDX *= scale;
				scaleDY *= scale;
			}
			
			//finalPos is the absolute position.  Because we are translating, we
			//need to find the change in position by subtracting the original position.
			//Also, add the scaling vector.
			sister.translate(finalPos.getX()-initialPos.getX()+scaleDX,finalPos.getY()-initialPos.getY()+scaleDY);
		}
		//Now translate the CTRL_POINT.  It is a CTRL_POINT since if it were a SEG_END
		//we wouldn't have reached this point.
		g.translate(dx,dy);
	}
}

/*
 * $Log: BezierConnectorInteractor.java,v $
 * Revision 1.1  2002/06/28 08:06:03  adamtj
 * Implemented grab handle locking.  Grab handles may be locked to related handles so that dragging it will cause the others to move as well.
 *
 *
 */