/* $Id: BezierShapeFactory.java,v 1.2 2002/06/28 08:02:33 adamtj Exp $
 *
 */

package fcmodeler.interaction.BezierConnector;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;

import diva.canvas.Site;
import diva.canvas.manipulator.GrabHandleFactory;
import diva.canvas.manipulator.BasicGrabHandleFactory;

import fcmodeler.interaction.GeneralConnector.GeneralConnectorSite;
import fcmodeler.interaction.GeneralConnector.GeneralConnectorShapeFactory;
import fcmodeler.view.figures.LineShape;

/**
 * BezierShapeFactory is where we build bezier curves from a list of control points.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class BezierShapeFactory implements GeneralConnectorShapeFactory {
	
	/** the ID of a site at the head of a curve */
	static public int HEAD = 101;
	/** the ID of a site at the tail of a curve */
	static public int TAIL = 102;
	/** the ID of a site at a control point of a curve */
	static public int CTRL_POINT = 103;
	/** the ID of a site at intermediate segment-end in a curve */
	static public int SEG_END = 104;
	
	/** the GrabHandleFactory used to create the grab handles of a bezier curve
	 *  connector.
	 */
	protected GrabHandleFactory _factory;
	
	
	/** Creates a new instance of BezierShapeFactory. */
	public BezierShapeFactory() {
		_factory = new BezierGrabHandle();
	}
	
	/** Return a list of GeneralConnectorSites representing the user-manipulable
	 *  vertices of this shape.
	 */
	public List setShape(Shape shape) {
		double[] coords = new double[6];
		List vertices = new Vector();
		
		PathIterator path = shape.getPathIterator(new AffineTransform());
		if (path.currentSegment(coords)==PathIterator.SEG_MOVETO) {
			GeneralConnectorSite s = new GeneralConnectorSite(coords[0], coords[1], HEAD);
			//This is so that in the while loop we don't have problems.
			coords[4] = coords[0];
			coords[5] = coords[1];
			vertices.add(new GeneralConnectorSite(coords[4], coords[5], SEG_END));
		} else {
			GeneralConnectorSite s = new GeneralConnectorSite(0, 0, HEAD);
		}
		
		while (!path.isDone()) {
			path.next();
			int segment = path.currentSegment(coords);
			switch (segment) {
				case PathIterator.SEG_CLOSE:
					((GeneralConnectorSite)vertices.get(vertices.size()-1)).setID(TAIL);
					break;
				case PathIterator.SEG_CUBICTO:
					vertices.add(new GeneralConnectorSite(coords[0], coords[1], CTRL_POINT));
					vertices.add(new GeneralConnectorSite(coords[2], coords[3], CTRL_POINT));
					vertices.add(new GeneralConnectorSite(coords[4], coords[5], SEG_END));
					break;
				case PathIterator.SEG_LINETO:
				case PathIterator.SEG_MOVETO:
					GeneralConnectorSite s = (GeneralConnectorSite)vertices.get(vertices.size()-1);
					coords[2] = coords[0];
					coords[3] = coords[1];
					coords[4] = coords[0];
					coords[5] = coords[1];
					coords[0] = (float)s.getX();
					coords[1] = (float)s.getY();
					vertices.add(new GeneralConnectorSite(coords[0], coords[1], CTRL_POINT));
					vertices.add(new GeneralConnectorSite(coords[2], coords[3], CTRL_POINT));
					vertices.add(new GeneralConnectorSite(coords[4], coords[5], SEG_END));
					break;
				case PathIterator.SEG_QUADTO:
					coords[5] = coords[3];
					coords[4] = coords[2];
					coords[3] = coords[1];
					coords[2] = coords[0];
					vertices.add(new GeneralConnectorSite(coords[0], coords[1], CTRL_POINT));
					vertices.add(new GeneralConnectorSite(coords[2], coords[3], CTRL_POINT));
					vertices.add(new GeneralConnectorSite(coords[4], coords[5], SEG_END));
					break;
			}
		}
		
		return vertices;
	}
	
	/** Given a list of user vertices, build and return a shape. */
	public Shape getShape(List vertices) {
		//Assume that this is a proper bezier curve shape as returned by setShape()
		float[] coords = new float[6];
		GeneralPath shape = new GeneralPath();
		Iterator i = vertices.iterator();
		
		//Move to the tail point.
		GeneralConnectorSite s = (GeneralConnectorSite)i.next();
		coords[0] = (float)s.getX();
		coords[1] = (float)s.getY();
		shape.moveTo(coords[0], coords[1]);
		//Loop through the rest of the control points.
		while (i.hasNext()) {
			s = (GeneralConnectorSite)i.next();
			coords[0] = (float)s.getX();
			coords[1] = (float)s.getY();
			s = (GeneralConnectorSite)i.next();
			coords[2] = (float)s.getX();
			coords[3] = (float)s.getY();
			s = (GeneralConnectorSite)i.next();
			coords[4] = (float)s.getX();
			coords[5] = (float)s.getY();
			shape.curveTo(coords[0],coords[1],coords[2],coords[3],coords[4],coords[5]);
		}
		
		return shape;
	}
	
	/** The Object data in this case is a List of Point2D objects which hold the
	 *  positions of the control points of the bezier curve.  There must be 3n+1
	 *  elements in the list with n>0;
	 * @param data the List of Point2D objects
	 */
	public List fromFile(Object data) {
		List vertices = new Vector();
		Point2D pt1 = null, pt2 = null, pt3 = null;
		Iterator cps = null;
		
		//Data must be a list of Point2Ds
		cps = ((List)data).iterator();
		pt1 = (Point2D) cps.next();
		vertices.add(new GeneralConnectorSite(pt1.getX(), pt1.getY(), HEAD));
		while (cps.hasNext()) {
			pt1 = (Point2D) cps.next();
			pt2 = (Point2D) cps.next();
			pt3 = (Point2D) cps.next();
			vertices.add(new GeneralConnectorSite(pt1.getX(), pt1.getY(), CTRL_POINT));
			vertices.add(new GeneralConnectorSite(pt2.getX(), pt2.getY(), CTRL_POINT));
			vertices.add(new GeneralConnectorSite(pt3.getX(), pt3.getY(), SEG_END));
		}
		((GeneralConnectorSite) vertices.get(vertices.size()-1)).setID(TAIL);
		return vertices;
	}
	
	/** We'll convert the List of Sites to a List of Point2Ds */
	public Object toFile(List vertices) {
		Site pt1 = null, pt2 = null, pt3 = null;
		Iterator cps = null;
		List output = new Vector();
		
		cps = vertices.iterator();
		pt1 = (Site) cps.next();
		output.add(new Point2D.Double(pt1.getX(), pt1.getY()));
		while (cps.hasNext()) {
			pt1 = (Site) cps.next();
			pt2 = (Site) cps.next();
			pt3 = (Site) cps.next();
			output.add(new Point2D.Double(pt1.getX(), pt1.getY()));
			output.add(new Point2D.Double(pt2.getX(), pt2.getY()));
			output.add(new Point2D.Double(pt3.getX(), pt3.getY()));
		}
		return output;
	}
	
	/** Given the vertices, compute the normal of the shape's head
	 *  @param headPoint the position of the head
	 */
	public double getHeadNormal(Point2D headPoint, List vertices) {
		// Figure out the normal at the line ends. NOTE that we
		// use upside-down geometry, so that pi/2 points SOUTH.
		//normal is the direction at which the line should connect to the site
		//so [x1,y1] is a vector from line beginning to tail figure center
		//and [x2,y2] is a vector from line end to head figure center
		//by line beginning/end, i mean the second/second-to-last control point
		double x = headPoint.getX() - ((GeneralConnectorSite)vertices.get(vertices.size()-2)).getX();
		double y = headPoint.getY() - ((GeneralConnectorSite)vertices.get(vertices.size()-2)).getY();
		
		//Math.atan2() converts cartestian to polar coordinates & returns phase angle
		double headAngle = Math.atan2(y, x);
		
		return headAngle;
	}
	
	/** Given the vertices, compute the normal of the shape's tail
	 *  @param tailPoint the position of the tail
	 */
	public double getTailNormal(Point2D tailPoint, List vertices) {
		// Figure out the normal at the line ends. NOTE that we
		// use upside-down geometry, so that pi/2 points SOUTH.
		//normal is the direction at which the line should connect to the site
		//so [x1,y1] is a vector from line beginning to tail figure center
		//and [x2,y2] is a vector from line end to head figure center
		//by line beginning/end, i mean the second/second-to-last control point
		double x = tailPoint.getX() - ((GeneralConnectorSite)vertices.get(1)).getX();
		double y = tailPoint.getY() - ((GeneralConnectorSite)vertices.get(1)).getY();
		
		//Math.atan2() converts cartestian to polar coordinates & returns phase angle
		double tailAngle = Math.atan2(y, x);
		
		return tailAngle;
	}
	
	/** Return the GrabHandleFactory to be used for decorating bezier curve
	 *  connectors.
	 */
	public GrabHandleFactory getCustomGrabHandleFactory() {
		return _factory;
	}
	
	/** Set the GrabHandleFactory to be used for decorating bezier curve
	 *  connectors.
	 */
	public void setCustomGrabHandleFactory(GrabHandleFactory factory) {
		_factory = factory;
	}
}

/*
 * $Log: BezierShapeFactory.java,v $
 * Revision 1.2  2002/06/28 08:02:33  adamtj
 * fixed some formatting
 *
 * Revision 1.1  2002/06/27 06:03:57  adamtj
 * Initial commit of BezierConnector files
 *
 *
 */
