/* $Id: GeneralConnector.java,v 1.2 2002/07/13 07:48:12 adamtj Exp $
 *
 */

package fcmodeler.interaction.GeneralConnector;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.geom.GeneralPath;
import java.util.Iterator;
import java.util.List;

import diva.canvas.connector.Connector;
import diva.canvas.TransformContext;
import diva.canvas.CanvasUtilities;
import diva.canvas.Figure;
import diva.canvas.AbstractFigure;
import diva.canvas.Site;
import diva.canvas.connector.AbstractConnector;
import diva.canvas.manipulator.GrabHandleFactory;
import diva.canvas.toolbox.PaintedShape;

import fcmodeler.view.AbstractEdgeFigure;
import fcmodeler.view.figures.LineShape;

/**
 * GeneralConnector is a Diva connector that is part of the GeneralConnector
 * framework.  The GeneralConnector series of classes allow connectors to have
 * user-manipulable geometries.
 *
 * GeneralConnector contains a GeneralConnectorGeometry object that stores
 * the geometry data for this connector.  A GeneralConnectorGeometry object needs
 * a ShapeFactory to translate user-manipulable vertices into shapes and vice-versa.
 *
 * To provide new connector shapes or new ways of manipulating a shape, provide a
 * new shape factory.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class GeneralConnector extends AbstractEdgeFigure {
	/** stores the geometry data and provides us with the edge Shape */
	protected GeneralConnectorGeometry _geometry;
	
	/** Creates a new instance of GeneralConnector.
	 * @param tail the Site to which the tail of this connector is attached
	 * @param head the Site to which the head of this connector is attached
	 * @param factory the ShapeFactory that maps between grab handle sites and the
	 *                shape of this GeneralConnector
	 */
	public GeneralConnector(Site tail, Site head, GeneralConnectorShapeFactory factory) {
		super(tail, head);
		
		//Make a boring shape until the geometry gets initialized.
		GeneralPath p = new GeneralPath();
		p.moveTo(0.0f, 0.0f);
		p.lineTo(0.0f, 0.0f);
		_geometry = new GeneralConnectorGeometry(this, (Shape)p, factory);
	}
	
	/** Route the connector, position the ends and make them point in the correct
	 *  direction.
	 */
	public void route() {
		TransformContext currentContext = null;
		Site headSite = null;
		Site tailSite = null;
		Point2D tailCenter = null, headCenter = null;
		Point2D headPt = null, tailPt = null;
		Point2D second = null, secondToLast = null;
		double x1 = 0;
		double y1 = 0;
		double x2 = 0;
		double y2 = 0;
		double headAngle = 0;
		double tailAngle = 0;
		
		//why do we call repaint() first???
		repaint();
		
		//get the current transform context
		currentContext = getTransformContext();
		
		//get the tail & head sites
		headSite = getHeadSite();
		tailSite = getTailSite();
		
		// Get the transformed head and tail points. Sometimes
		// people will call this before the connector is added
		// to a container, so deal with it
		if (currentContext != null) {
			tailPt = tailSite.getPoint(currentContext);
			headPt = headSite.getPoint(currentContext);
		} else {
			tailPt = tailSite.getPoint();
			headPt = headSite.getPoint();
		}
		
		//if there are figures attached, find their center points
		//if not just use the Site point
		if (tailSite.getFigure() != null) {
			tailCenter = CanvasUtilities.getCenterPoint(tailSite.getFigure());
		} else {
			tailCenter = tailPt;
		}
		if (headSite.getFigure() != null) {
			headCenter = CanvasUtilities.getCenterPoint(headSite.getFigure());
		} else {
			headCenter = headPt;
		}
		
		// Tell the sites to adjust their positions
		headSite.setNormal(_geometry.getHeadNormal(headCenter) - Math.PI);
		tailSite.setNormal(_geometry.getTailNormal(tailCenter) - Math.PI);
		
		// Recompute the head and tail points
		//these are the coordinates of the new sites on the head & tail figures
		//after the above calculations
		if (currentContext != null) {
			tailPt = tailSite.getPoint(currentContext);
			headPt = headSite.getPoint(currentContext);
		} else {
			tailPt = tailSite.getPoint();
			headPt = headSite.getPoint();
		}
		
		//adjust for decorations on the ends
		if (getTailEnd() != null) {
			getTailEnd().setNormal(_geometry.getTailNormal(tailPt) - Math.PI);
			getTailEnd().setOrigin(tailSite.getX(), tailSite.getY());
			getTailEnd().getConnection(tailPt);
		}
		if (getHeadEnd() != null) {
			//figure out the normal again
			getHeadEnd().setNormal(_geometry.getHeadNormal(headPt) - Math.PI);
			getHeadEnd().setOrigin(headSite.getX(), headSite.getY());
			getHeadEnd().getConnection(headPt);
		}
		
		//update the edge shape
		_geometry.setHeadSite(headPt);
		_geometry.setTailSite(tailPt);
		
		getPaintedShape().shape = new LineShape(_geometry.getShape());
		
		//repaint the figure
		repaint();
	}
	
	/**
	 * @return  the factory used to produce GrabHandles */
	public GrabHandleFactory getCustomGrabHandleFactory() {
		return _geometry.getCustomGrabHandleFactory();
	}
	
	/** @return a copy of the List of GeneralConnectorSites that the geometry uses
	 *          to define the shape of this Connector
	 */
	public List getVertices() {
		return _geometry.vertices();
	}
	
	/** Generate the shape from a file.
	 * @param data an Object containing the data which is to be passed to the
	 *             ShapeFactory.
	 */
	public void fromFile(Object data) {
		_geometry.fromFile(data);
	}
	
	/** Return an object with the geometry data.
	 * @returns an Object of a type determined by the ShapeFactory.
	 */
	public Object toFile() {
		return _geometry.toFile();
	}
	
	/**
	 * Translate the edge.
	 * @param x horizontal offset
	 * @param y vertical offset
	 */
	public void translate(double x, double y) {
		_geometry.translate(x,y);
	}
}

/*
 * $Log: GeneralConnector.java,v $
 * Revision 1.2  2002/07/13 07:48:12  adamtj
 * Fixed a problem with edge mappings.  GeneralConnectorEdgeFigure improperly implemented EdgeFigure.  It has been removed and now GeneralConnector extends AbstractEdgeFigure.
 *
 * Revision 1.1  2002/06/27 06:03:15  adamtj
 * Initial commit of GeneralConnector files
 *
 *
 */
