/* $Id: GeneralConnectorGeometry.java,v 1.1 2002/06/27 06:03:15 adamtj Exp $
 *
 */
package fcmodeler.interaction.GeneralConnector;

import java.awt.Shape;
import java.awt.geom.Point2D;
import java.lang.Object;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import diva.canvas.Figure;
import diva.canvas.Site;
import diva.canvas.manipulator.Geometry;
import diva.canvas.manipulator.GrabHandleFactory;

/**
 * GeneralConnectorGeometry is the object that contains the Sites that are given
 * to the grab handles that the user can drag.  When a user drags a grab handle, it
 * translates its Site, and thus the connector's geometry is updated.
 *
 * To turn the list of Sites into a Shape, we use a ShapeFactory.  The ShapeFactory
 * translates from Sites to Shapes and back.  To create a new connector type,
 * simply implement a ShapeFactory.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class GeneralConnectorGeometry implements Geometry {
	
	/** the figure associated with this geometry */
	protected Figure  _figure;
	/** turns the List of vertices into a Shape and vice-versa */
	protected GeneralConnectorShapeFactory _factory;
	/** the list of GeneralConnectorSites that define the shape.  These are
	 *  eventually given to the grab handles.
	 */
	protected List _vertices;
	
	/** Creates a new instance of GeneralConnectorGeometry
	 * @param f the figure associated with this geometry
	 * @param s the initial shape which this geometry object should describe
	 * @param factory  the ShapeFactory that does all the real work*/
	public GeneralConnectorGeometry(Figure f, Shape s, GeneralConnectorShapeFactory factory) {
		_figure = f;
		_factory = factory;
		setShape(s);
	}
	/** Generate the shape from a file.
	 * @param data an Object containing the data which is to be passed to the 
	 *             ShapeFactory.
	 */
	public Shape fromFile(Object data) {
		_vertices = _factory.fromFile(data);
		return getShape();
	}
	
	/** Get the custom GrabHandleFactory used to draw the GrabHandles that aren't
	 *  at the ends of the connector.
	 * @return the custom GrabHandleFactory
	 */
	public GrabHandleFactory getCustomGrabHandleFactory() {
		return _factory.getCustomGrabHandleFactory();
	}
	
	/** Return the figure associated with this geometry. */
	public Figure getFigure() {
		return _figure;
	}
	
	/** Compute the normal of the shape's head */
	public double getHeadNormal(Point2D headPoint) {
		return _factory.getHeadNormal(headPoint, _vertices);
	}
	
	/** Get the shape represented by this geometry object. */
	public Shape getShape() {
		return _factory.getShape(_vertices);
	}
	
	/** Compute the normal of the shape's tail */
	public double getTailNormal(Point2D tailPoint) {
		return _factory.getTailNormal(tailPoint, _vertices);
	}
	
	/** Set the custom GrabHandleFactory used to draw the GrabHandles that aren't
	 *  at the ends of the connector.
	 * @param factory  the custom GrabHandleFactory
	 */
	public void setCustomGrabHandleFactory(GrabHandleFactory factory) {
		_factory.setCustomGrabHandleFactory(factory);
	}
	
	/**  Set the head site. */
	public void setHeadSite(Point2D s) {
		((GeneralConnectorSite)_vertices.get(_vertices.size()-1)).moveTo(s.getX(), s.getY());
	}
	
	/** Set the shape represented by this geometry object. */
	public void setShape(Shape s) {
		_vertices = _factory.setShape(s);
	}
	
	/** Set the tail site.*/
	public void setTailSite(Point2D s) {
		((GeneralConnectorSite)_vertices.get(0)).moveTo(s.getX(), s.getY());
	}
	
	/** Return an object with the geometry data.
	 * @returns an Object of a type determined by the ShapeFactory.
	 */
	public Object toFile() {
		return _factory.toFile(_vertices);
	}
	
	/** Translate the shape represented by this geometry object.
	 * @param x horizontal offset
	 * @param y vertical offset
	 */
	public void translate(double x, double y) {
		int num = _vertices.size();
		for (int i=0; i<num; i++) {
			((Site)_vertices).translate(x, y);
		}
	}
	
	/** Return the requested vertex
	 * @param index the index of the Site to return
	 * @return the requested Site
	 */
	public Site vertex(int index) {
		if (index >= 0 && index < _vertices.size()) {
			return (Site)_vertices.get(index);
		} else {
			return (Site)_vertices.get(0);
		}
	}
	
	/** Get the number of vertices.
	 * @return the number of vertices
	 */
	public int vertexCount() {
		return _vertices.size();
	}
	
	/** Return the list of GeneralConnectorSites that represent the geometry of
	 *  this edge.
	 * @return the list of GeneralConnectorSites
	 */
	public List vertices() {
		return new Vector(_vertices);
	}
	/*
	 //Here lies a deceased Iterator.  May it rest in peace.
	 public Iterator vertices() {
		return new Iterator() {
			int pos = 0;
			public boolean hasNext() {
				return pos < _vertices.size();
			}
			public Object next() {
				pos++;
				return vertex(pos-1);
			}
			public void remove() {
				throw new UnsupportedOperationException(
				"GeneralConnectorGeometry: Vertex removal unimplemented");
			}
		};
	}*/
}

/*
 * $Log: GeneralConnectorGeometry.java,v $
 * Revision 1.1  2002/06/27 06:03:15  adamtj
 * Initial commit of GeneralConnector files
 *
 *
 */
