/* $Id: GeneralConnectorGrabHandle.java,v 1.3 2002/06/28 07:56:30 adamtj Exp $
 *
 */
package fcmodeler.interaction.GeneralConnector;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.RoundRectangle2D;

import diva.canvas.AbstractFigure;
import diva.canvas.Site;
import diva.canvas.manipulator.BasicGrabHandle;
import diva.canvas.manipulator.GrabHandle;
import diva.canvas.manipulator.GrabHandleFactory;
import diva.canvas.toolbox.BasicFigure;
import diva.canvas.toolbox.BasicRectangle;
import diva.canvas.toolbox.PaintedShape;

/**
 * GeneralConnectorGrabHandle is the grab handle used to manipulate GeneralConnectors.
 * It is also a GrabHandleFactory so that it can create new GrabHandles.  This class
 * can be extened to paint new styles of GrabHandles.  It is possible to subclass
 * this and easily create a GrabHandle that looks different depending on what type
 * of Site it is attached to.  See BezierGrabHandle for an example.
 *
 * It is only necessary to implement initShape() when subclassing.  
 * createGrabHandle() is capable of determining its class and returning a GrabHandle
 * of your new subclass without any additional work.
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.3 $
 * @since JDK1.3
 */

public class GeneralConnectorGrabHandle extends AbstractFigure implements GrabHandle, GrabHandleFactory {
	
	/** the shape of this grab handle */
	protected BasicFigure _shape;
	/** the site to which this handle is connected */
	protected Site _site;
	/** the visible size */
	protected float _size=8.0f;
	/** the x position of the grab handle */
	protected double _x=0;
	/** the y position of the grab handle */
	protected double _y=0;
	
	/** Creates a new instance of GeneralConnectorGrabHandle */
	public GeneralConnectorGrabHandle() {
		this(new GeneralConnectorSite());
	}
	
	/** Creates a new instance of GeneralConnectorGrabHandle and attach to the 
	 *  specified Site
	 * @param s the site to which this handle should be attached.
	 */
	public GeneralConnectorGrabHandle(Site s) {
		setSite(s);
	}
	
	/** Generates the shape that gets painted.  When subclassing, this is the only
	 *  function that needs to be overridden to create grab handles of different
	 *  shapes.
	 */
	protected void initShape() {
		//Create a diamond shape.
		GeneralPath p = new GeneralPath();
		p.moveTo((float)_x, (float)_y + _size/2);
		p.lineTo((float)_x + _size/2, (float)_y);
		p.lineTo((float)_x, (float)_y - _size/2);
		p.lineTo((float)_x - _size/2, (float)_y);
		p.closePath();
		
		_shape = new BasicFigure( p );
		_shape.setStrokePaint(Color.blue);
		_shape.setFillPaint(Color.blue);
	}
	
	/** Creates a new grab handle and attach it to the given site.  This is the 
	 *  function that makes a GeneralConnectorGrabHandle also a GrabHandleFactory.
	 *  This function does not need to be overridden when subclassing.  It
	 *  automagically determines the class of <code>this</code> and creates a
	 *  grab handle of the same class.
	 * @param s the site to which the new grab handle will be attached
	 * @return the new grab handle
	 */
	public GrabHandle createGrabHandle(Site s) {
		try {
			GrabHandle spawn = (GrabHandle) this.getClass().newInstance();
			spawn.setSite(s);
			spawn.setSize(_size);
			return spawn;
		} catch (Exception e) {
			return new BasicGrabHandle(s);
		}
	}
	
	/** Get the bounding box of the grab handle.*/
	public Shape getShape() {
		//It appears the bounding box of our shape is sometimes too small.
		//This causes grab handles to leave a mess of paint when you drag them.
		//It looks like it's probably a problem with rounding from decimals
		//to integers.  Simply increase the size of the bounding box to fix the
		//problem.
		Rectangle r = _shape.getShape().getBounds();
		Rectangle bounds = new Rectangle(r.x-1,r.y-1,r.width+2, r.height+2);
		return bounds;
	}
	
	/** Get the site to which this grab-handle is attached.
	 *  @return the site to which this grab-handle is attached
     */
    public Site getSite() {
		return _site;
	}

    /** Get the "size" of the grab-handle. The size is some dimension
     *  that approximately represents the visible size of the grab handle
     */
    public float getSize () {
		return _size;
	}

	/** Paint this grab handle */
	public void paint(Graphics2D g) {
		_shape.paint(g);
	}
	
    /** Reposition the grab-handle to its site. */
    public void relocate () {
		if (_shape != null ) {
			_shape.translate(_site.getX() - _x, _site.getY() - _y);
		}
		_x = _site.getX();
		_y = _site.getY();
	}

    /** Set the site to which this grab-handle is attached. */
    public void setSite (Site s) {
		_site = s;
		if (_site instanceof GeneralConnectorSite) {
			((GeneralConnectorSite)_site).setFigure(this);
		}
		initShape();
		relocate();
	}

    /**
     * Set the "size" of the grab-handle.  The size is some dimension
     * that approximately represents the visible size of the grab handle.
     */
    public void setSize (float size) {
		_size = size;
	}
	
	/** Apply an AffineTransform to the shape of this handle.  Caveat: as implemented
	 *  this transforms just the shape, not the site.*/
	public void transform (AffineTransform at) {
		_shape.transform(at);
	}
	
	/** Move the grab handle. */
	public void translate(double x, double y) {
		_site.translate(x,y);
	}
}

/* 
 * $Log: GeneralConnectorGrabHandle.java,v $
 * Revision 1.3  2002/06/28 07:56:30  adamtj
 * adjusted a comment
 *
 * Revision 1.2  2002/06/27 08:48:47  adamtj
 * Dragged GeneralConnectorGrabHandles no longer leave a mess of paint in their wake.
 *
 * Revision 1.1  2002/06/27 06:03:15  adamtj
 * Initial commit of GeneralConnector files
 *
 *
 */
