/* $Id: SimpleDotWriter.java,v 1.4 2002/05/26 06:10:18 adamtj Exp $
 *
 *
 */

package fcmodeler.io.dot;

import java.io.Writer;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

import diva.graph.model.Graph;
import diva.graph.model.Node;
import diva.graph.model.Edge;

import fcmodeler.FCModeler;
import fcmodeler.io.AbstractGraphWriter;

/**
 * ... is ...
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 *
 * @version $Revision: 1.4 $
 * @since JDK1.3
 */

public class SimpleDotWriter extends AbstractGraphWriter implements DotGraphWriter {
	private Map _nodeMap = null;
	private Map _edgeMap = null;
	
	/** Creates new SimpleDotWriter */
	public SimpleDotWriter() {
		
	}
	
	public Map getEdgeMap() {
		return _edgeMap;
	}
	
	public Map getNodeMap() {
		return _nodeMap;
	}
	
	/** Writes the specified graph to the specified output.
	 *
	 * @param graph the graph to write.
	 * @param writer the output where the graph in text format will go.
	 */
	public void writeGraph(FCModeler fcmodeler, Writer writer) {
		if (isWritingTopology()) {
			Graph graph = fcmodeler.getGraph();
			PrintWriter output = new PrintWriter(writer);
			_nodeMap = new HashMap();
			_edgeMap = new HashMap();

			//write beginning of graph
			output.println("digraph G {");

			//write the nodes
			_nodeMap.clear();
			for (Iterator nodes = graph.nodes(); nodes.hasNext();) {
				Node node = (Node) nodes.next();
				String nodeName = DotUtilities.getNodeName(node);

				//----------------------------------------------------------
				//According to dot, there are always 72.0 points per inch.
				//We specify the widths of nodes in inches, and dot gives us coordinates
				//in points.  FCModeler uses units of pixels.  On a given display device,
				//you need to divide pixels by the resolution in dots-per-inch to convert
				//to points, which can then be converted to inches.  It turns out it's
				//unnecessary to convert pixels to points.  dot performs only (mostly) linear
				//operations to find horizontal positions.  It is therefore sufficient to know 
				//that scale factor to convert input width units to ouput coordinate units is 72.0
				//Caveats:
				// 1: Actually it's not completely linear.  The default node separation (nodesep)
				//    is 0.5 inches.  That would convert to 36 output units (pixels).  The problem
				//    is that 36 pixels could actually be a very tiny distance, depending on the
				//    display device.  As long as the node widths in pixels are on the order
				//    of 100 pixels or so, it should work.  Otherwise, you need to change the
				//    node separation.
				// 2: I think there are some interesting non-linear effects when node widths get
				//    tiny.  dot seems to think that 0.04 inches is too small or something.
				//    As long as node widths are >72 pixels (or 1 inch to dot), it should
				//    be fine.
				//
				// Conclusion: If node widths are on the order of 100 pixels, divide widths in 
				//             pixels by 72, and dot will spit out coordinates in pixels.
				//----------------------------------------------------------
				double nodeWidth = fcmodeler.getGraphView().getNodeFigure(node).getBounds().getWidth()/(double)72.0;
				String nodeAttributes = "[";
				nodeAttributes += "width=" + String.valueOf(nodeWidth);
				//This seems to be the easiest way to fix the problem of arrows being inside
				//nodes and pointing outwards.  The problem is because the edge goes within
				//the NodeFigure and turns around to point outward towards the edge.  If 
				//we tell dot that the node is taller, then it adjusts the edge control
				//points accordingly and they don't end up within the NodeFigure.
				//Oddly though, calculating the height in the same way we calculate the
				//width doesn't work even though I would think that it should. -- atj
				nodeAttributes += ", height=0.6";
				nodeAttributes += "]";
				
				//-----------------------

				_nodeMap.put(nodeName, node);
				
				output.println("node "+ nodeAttributes+";");
				output.println(DotUtilities.addQuotes(nodeName) + ";");
			}

			//write the edges
			_edgeMap.clear();
			for (Iterator nodes = graph.nodes(); nodes.hasNext();) {
				for (Iterator outEdges = ((Node) nodes.next()).outEdges(); outEdges.hasNext();) {
					Edge edge = (Edge) outEdges.next();
					String edgeName = DotUtilities.getEdgeName(edge);
					_edgeMap.put(edgeName, edge);
					output.println(DotUtilities.getQuotedEdgeName(edge) + ";");
				}
			}

			//write end of graph
			output.println("}");

			//finish the output
			output.flush();
			output.close();
		}
	}
	
}

/* 
 * $Log: SimpleDotWriter.java,v $
 * Revision 1.4  2002/05/26 06:10:18  adamtj
 * Fixes the problem of the arrows being inside the nodes and pointing outward.
 *
 * Revision 1.3  2002/05/20 19:26:50  adamtj
 * Stops nodes from overlapping when displayed by passing node widths to dot.
 *
 * Revision 1.2  2002/02/15 20:06:46  zcox
 * big time update
 *
 * Revision 1.1  2002/01/15 20:29:28  zcox
 * initial commit of fcmodeler.io.dot package
 *
 * 
 */
