/* $Id: XMLGraphReader.java,v 1.2 2002/05/29 22:01:10 adamtj Exp $
 *
 *
 */

package fcmodeler.io.xml;

import java.awt.geom.Point2D;
import java.io.Reader;
import java.io.IOException;
import java.util.List;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

import diva.graph.model.GraphModel;
import diva.graph.model.Edge;

import fcmodeler.FCModeler;
import fcmodeler.io.AbstractGraphReader;
import fcmodeler.util.QuotedStringIterator;
import fcmodeler.util.NoTextNodeList;
import fcmodeler.mapping.Sentence;
import fcmodeler.mapping.AtomicSentence;
import fcmodeler.mapping.CompositeSentence;
import fcmodeler.mapping.Connective;
import fcmodeler.mapping.AndConnective;
import fcmodeler.mapping.OrConnective;
import fcmodeler.mapping.NodeMapping;
import fcmodeler.mapping.EdgeMapping;
import fcmodeler.layout.SpecifiedLayout;
import fcmodeler.attributes.VisualAttribute;
import fcmodeler.attributes.VisualAttributeFactory;
import fcmodeler.attributes.VisualAttributeValue;
import fcmodeler.attributes.NodeVisualAttributeValue;
import fcmodeler.attributes.EdgeVisualAttributeValue;

/** 
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class XMLGraphReader extends AbstractGraphReader {
	private GraphModel _model = null;
	private Set _nodeMappings = null;
	private Set _edgeMappings = null;
	private Map _nodeCoordinates = null;
	private Map _edgeCoordinates = null;
	
	/** Creates new XMLGraphReader */
	public XMLGraphReader() {
		
	}
	
	public XMLGraphReader(boolean readCoordinates, boolean readMappings, boolean readTopology) {
		readCoordinates(readCoordinates);
		readMappings(readMappings);
		readTopology(readTopology);
	}
	
	private Sentence buildSentence(Element sentenceElement) {
		Sentence sentence = null;
		if (sentenceElement.getTagName().equals("atom")) {
			sentence = new AtomicSentence(sentenceElement.getAttribute("property"), sentenceElement.getAttribute("value"));
		}
		else if (sentenceElement.getTagName().equals("composite")) {
			NodeList children = new NoTextNodeList(sentenceElement.getChildNodes());
			Element firstElement = (Element) children.item(0);
			Sentence first = buildSentence(firstElement);
			Element connectiveElement = (Element) children.item(1);
			Connective connective = connectiveElement.getAttribute("type").equals("and") ? (Connective) AndConnective.getInstance() : (Connective) OrConnective.getInstance();
			Element secondElement = (Element) children.item(2);
			Sentence second = buildSentence(secondElement);
			sentence = new CompositeSentence(first, connective, second);
		}
		return sentence;
	}
	
	public void configure(FCModeler fcmodeler) {
               
		if (isReadingTopology()) {
			fcmodeler.setGraphModel(_model);
		}
		
		if (isReadingMappings()) {
			Set nodeMappings = fcmodeler.getNodeMappings();
			nodeMappings.clear();
			nodeMappings.addAll(_nodeMappings);
			Set edgeMappings = fcmodeler.getEdgeMappings();
			edgeMappings.clear();
			edgeMappings.addAll(_edgeMappings);
		}
		
		//The coordinates must be read after the topology.
		//The topology may be loaded, and then the coordinates at a later time,
		//even from a different file.
		if (isReadingCoordinates()) {
			SpecifiedLayout layout = new SpecifiedLayout(fcmodeler, _nodeCoordinates, _edgeCoordinates);
			layout.layout();
		}
	}
	
	/** Reads the graph from the specified input source.
	 *
	 * @param reader the input source containing the graph in text format.
	 */
	public void readGraph(Reader reader) {
		if (isReadingCoordinates() || isReadingMappings() || isReadingTopology()) {
			try {
				DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
				DocumentBuilder builder = builderFactory.newDocumentBuilder();
				Document document = builder.parse(new InputSource(reader));
				
				if (isReadingTopology()) {
					NodeList nodeList = document.getElementsByTagName("graph");
					if (nodeList == null || nodeList.getLength() != 1) { throw new IllegalStateException("must be exactly one graph element in order to read topology"); }
					Element graphElement = (Element) nodeList.item(0);
					readTopology(graphElement);
				}
				
				if (isReadingMappings()) {
					NodeList nodeList = document.getElementsByTagName("mappings");
					if (nodeList == null || nodeList.getLength() != 1) { throw new IllegalStateException("must be exactly one mappings element in order to read mappings"); }
					Element mappingsElement = (Element) nodeList.item(0);
					readMappings(mappingsElement);
				}
                
				if (isReadingCoordinates()) {
					NodeList nodeList = document.getElementsByTagName("coordinates");
					if (nodeList == null || nodeList.getLength() != 1) { throw new IllegalStateException("must be exactly one coordinates element in order to read mappings"); }
					Element coordinatesElement = (Element) nodeList.item(0);
					readCoordinates(coordinatesElement);
				}
			} catch (ParserConfigurationException e) {
				e.printStackTrace();
			} catch (SAXException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	private void readMappings(Element mappingsElement) {
		_nodeMappings = new HashSet();
		_edgeMappings = new HashSet();
		NodeList mappingElements = mappingsElement.getElementsByTagName("mapping");
		for (int i=0, length=mappingElements.getLength(); i<length; i++) {
			Element mappingElement = (Element) mappingElements.item(i);
			NodeList nodeList = new NoTextNodeList(mappingElement.getChildNodes());
			Element sentenceElement = (Element) nodeList.item(0);
			Sentence sentence = buildSentence(sentenceElement);
			Element attributeValueElement = (Element) mappingElement.getElementsByTagName("attributeValue").item(0);
			VisualAttribute attribute = VisualAttributeFactory.getInstance().getAttribute(attributeValueElement.getAttribute("attribute"));
			VisualAttributeValue value = attribute.getValue(attributeValueElement.getAttribute("value"));
			if (mappingElement.getAttribute("type").equals("node")) {
				_nodeMappings.add(new NodeMapping(sentence, (NodeVisualAttributeValue) value));
			}
			else if (mappingElement.getAttribute("type").equals("edge")) {
				_edgeMappings.add(new EdgeMapping(sentence, (EdgeVisualAttributeValue) value));
			}
		}
	}
	
	private void readTopology(Element graphElement) {
		_model = new GraphModel();
		
		Element nodePropertiesElement = (Element) graphElement.getElementsByTagName("nodeProperties").item(0);
		List nodeProperties = new ArrayList();
		for (Iterator iterator = new QuotedStringIterator(nodePropertiesElement.getFirstChild().getNodeValue().trim()); iterator.hasNext();) {
			nodeProperties.add(iterator.next());
		}
		_model.getGraph().setProperty("nodeProperties", nodeProperties);
		
		Element edgePropertiesElement = (Element) graphElement.getElementsByTagName("edgeProperties").item(0);
		List edgeProperties = new ArrayList();
		for (Iterator iterator = new QuotedStringIterator(edgePropertiesElement.getFirstChild().getNodeValue().trim()); iterator.hasNext();) {
			edgeProperties.add(iterator.next());
		}
		_model.getGraph().setProperty("edgeProperties", edgeProperties);
		
		NodeList nodeElements = graphElement.getElementsByTagName("node");
		Map nodeMap = new HashMap();
		for (int i=0, length=nodeElements.getLength(); i<length; i++) {
			Element nodeElement = (Element) nodeElements.item(i);
			diva.graph.model.Node node = _model.createNode(nodeElement.getAttribute("id"));
			for (Iterator iterator = new QuotedStringIterator(nodeElement.getFirstChild().getNodeValue().trim()), iterator2 = nodeProperties.iterator(); iterator.hasNext();) {
				node.setProperty(iterator2.next().toString(), iterator.next());
			}
			node.setProperty("label", node.getSemanticObject().toString());
			_model.addNode(node);
			nodeMap.put(node.getSemanticObject(), node);
		}
		
		NodeList edgeElements = graphElement.getElementsByTagName("edge");
		for (int i=0, length=edgeElements.getLength(); i<length; i++) {
			Element edgeElement = (Element) edgeElements.item(i);
			Edge edge = _model.createEdge(edgeElement.getAttribute("id"));
			diva.graph.model.Node tail = (diva.graph.model.Node) nodeMap.get(edgeElement.getAttribute("tail"));
			diva.graph.model.Node head = (diva.graph.model.Node) nodeMap.get(edgeElement.getAttribute("head"));
			edge.setDirected(edgeElement.getAttribute("directed").equals("true") ? true : false);
			for (Iterator iterator = new QuotedStringIterator(edgeElement.getFirstChild().getNodeValue().trim()), iterator2 = edgeProperties.iterator(); iterator.hasNext();) {
				edge.setProperty(iterator2.next().toString(), iterator.next());
			}
			edge.setProperty("label", edge.getSemanticObject().toString());
			_model.connectEdge(edge, tail, head);
		}
	}
	private void readCoordinates(Element coordinatesElement) {
		_nodeCoordinates = new HashMap();
		_edgeCoordinates = new HashMap();
		if (_model==null) {
			_model = new GraphModel();
		}
        
		NodeList nodeFigureElements = coordinatesElement.getElementsByTagName("nodeFigure");
		for (int i=0, length=nodeFigureElements.getLength(); i<length; i++) {
			Element nodeFigureElement = (Element) nodeFigureElements.item(i);
			String nodeName = nodeFigureElement.getAttribute("node");
			String nodeCoordinates = nodeFigureElement.getAttribute("coord");

			//Parse the coordinates
			StringTokenizer st = new StringTokenizer(nodeCoordinates, ",");
			Point2D coords = new Point2D.Double(Double.parseDouble(st.nextToken()),Double.parseDouble(st.nextToken()));
            
			//Store the data.
			//It was non-obvious to me that a Node is a SemanticObjectContainer
			//And the correct way to test for node equality is to test for
			//equality of the semantic object -- atj
			diva.graph.model.Node node = _model.createNode(nodeName);
			_nodeCoordinates.put(node.getSemanticObject(), coords);
		}
        
		NodeList edgeFigureElements = coordinatesElement.getElementsByTagName("edgeFigure");
		for (int i=0, length=edgeFigureElements.getLength(); i<length; i++) {
			Element edgeFigureElement = (Element) edgeFigureElements.item(i);
			String edgeName = edgeFigureElement.getAttribute("edge");
			String edgeCoordinates = edgeFigureElement.getAttribute("coords");

			//Parse the ordered pairs
			StringTokenizer orderedPairs = new StringTokenizer(edgeCoordinates, " ");
			List coords = new Vector();
			//Add pairs of coordinates to the list.
			try {
				for(; orderedPairs.hasMoreTokens(); ) {
					//Break the pair into components
					StringTokenizer st = new StringTokenizer(orderedPairs.nextToken(), ",");
					Point2D pt = new Point2D.Double(Double.parseDouble(st.nextToken()), Double.parseDouble(st.nextToken()));
					coords.add(pt);
				}
			} catch (NoSuchElementException e) {
				throw new IllegalStateException("Invalid coordinates");                
			}
            
			//Store the data.
			diva.graph.model.Edge edge = _model.createEdge(edgeName);
			_edgeCoordinates.put(edge.getSemanticObject(), coords);
		}
	}
}

/* 
 * $Log: XMLGraphReader.java,v $
 * Revision 1.2  2002/05/29 22:01:10  adamtj
 * Added the ability to read coordinates from a file.
 *
 * Revision 1.1  2002/02/15 20:08:54  zcox
 * big time update
 *
 *
 *
 */
