/*
 * WeightedSet.java
 *
 * Created on April 11, 2002, 12:06 PM
 */

package fcmodeler.jsomap;

import java.util.AbstractSet;
import java.util.Collection;
import java.util.Map;
import java.util.HashMap;
import java.util.TreeMap; // <----------------------------- this is for testing - switch back to HashMap for performance
import java.util.Iterator;

/** A set in which each element has a corresponding real-valued weight.  This class implements the
 * <CODE>java.util.Set</CODE> interface and supports all of the normal set operations.  Additionally,
 * each element in this set has a corresponding weight, of type <CODE>double</CODE>.  The weights are
 * obtained using the <CODE>get</CODE> method, and altered using the <CODE>set</CODE> method.  Adding
 * an element(s) to this set using the <CODE>add</CODE> or <CODE>addAll</CODE> methods adds the
 * element(s) and assignes a weight of <CODE>1.0d</CODE> to it/them.  Additionally, if the collection passed
 * to the <CODE>addAll</CODE> method is of type <CODE>WeightedSet</CODE>, that set is effectively copied
 * into this set, including its weights.
 *
 * @author default
 * @version
 */
public class WeightedSet extends AbstractSet {
	private Map _map = null;
	private boolean _iterating = false;
	
	/** Creates a new empty <CODE>WeightedSet</CODE>.
	 *
	 */
	public WeightedSet() {
		super();
		//_map = new TreeMap(); // <----------------------------- this is for testing - switch back to HashMap for performance
		_map = new HashMap();
	}
	
	/** Creates a new <CODE>WeightedSet</CODE> containing the elements in the specified collection.  If the
	 * collection is of type <CODE>WeightedSet</CODE>, all elements are assigned to their corresponding
	 * weights; otherwise, all elements are assigned a weight of <CODE>1.0d</CODE>.
	 *
	 * @param collection the collection whose elements are to be placed in this set.
	 */	
	public WeightedSet(Collection collection) {
		this();
		if (collection == null) { throw new NullPointerException("collection was null"); }
		
		//add all of the elements in the collection to this set
		addAll(collection);
	}
	
	/** Adds the specified object to this set if it is not already present and assigns it a weight
	 * of <CODE>1.0d</CODE>.
	 *
	 * @param obj the object to add to this set.
	 * @return <CODE>true</CODE> if this set did not already contain the object.
	 */	
	public boolean add(Object obj) {
		if (obj == null) { throw new NullPointerException("obj was null"); }
		
		if (!contains(obj)) {
			set(obj, 1.0d);
			return true;
		}
		else { return false; }
	}
	
	/** Adds all of the elements in the specified collection to this set if they're not already present.
	 * If the collection is of type <CODE>WeightedSet</CODE> the elements are assigned their corresponding
	 * weights; if not, they are assigned the weight <CODE>1.0d</CODE>.
	 *
	 * @param collection the collection whose elements are to be added to this set.
	 * @return <CODE>true</CODE> if this set changed as a result of this call.
	 */	
	public boolean addAll(Collection collection) {
		if (collection == null) { throw new NullPointerException("collection was null"); }
		
		if (collection instanceof WeightedSet) {
			WeightedSet set = (WeightedSet) collection;
			for (Iterator iterator = set.iterator(); iterator.hasNext();) {
				Object obj = iterator.next();
				set(obj, set.get(obj));
			}
			return true;
		}
		else {
			return super.addAll(collection);
		}
	}
	
	/** Removes all elements with weight of 0.0 from this weighted set.
	 *
	 */	
	private void cleanup() {
		for (Iterator iterator = _map.keySet().iterator(); iterator.hasNext();) {
			if (get(iterator.next()) == 0.0d) { iterator.remove(); }
		}
	}
	
	/** Removes all elements from this set.
	 *
	 */	
	public void clear() {
		_map.clear();
	}
	
	/** Returns <CODE>true</CODE> if this set contains the specified object.
	 *
	 * @param obj the object to test for containment.
	 * @return <CODE>true</CODE> if this set contains the specified object.
	 */	
	public boolean contains(Object obj) {
		return _map.containsKey(obj);
	}
	
	/** Compares the specified object with this set for equality.  Returns <CODE>true</CODE> if the specified
	 * object is also a weighted set, the two sets have the same size, and every member of the specified set
	 * is contained in this set and has the same weight.
	 *
	 * @param obj the object to be compared for equality with this set.
	 * @return <CODE>true</CODE> if the specified object is equal to this set.
	 */	
	public boolean equals(Object obj) {
		if (obj == this) { return true; }
		if (!(obj instanceof WeightedSet)) { return false; }
		
		WeightedSet other = (WeightedSet) obj;
		if (size() != other.size()) { return false; }
		for (Iterator iterator = other.iterator(); iterator.hasNext();) {
			Object o = iterator.next();
			if (!contains(o)) { return false; }
			if (get(o) != other.get(o)) { return false; }
		}
		
		return true;
	}
	
	/** Returns the weight associated with the specified object.  Returns the value <CODE>0.0d</CODE> if the
	 * specified object is not contained in this set.
	 *
	 * @param obj the object whose weight is returned.
	 * @return the weight of the specified object.
	 */	
	public double get(Object obj) {
		if (obj == null) { throw new NullPointerException("obj was null"); }
		
		Weight w = (Weight) _map.get(obj);
		return w == null ? 0.0d : w.weight;
	}
	
	/** <p>Returns the hash code value for this set.  The hash code of a weighted set is defined to be the sum
	 * of the hash codes of each element and it's associated weight.  Specifically, the hash code of each
	 * element and its weight are combined as follows:</p>
	 *
	 * <p><PRE>
	 * long bits = Double.doubleToLongBits(get(obj));
	 * int temp = (int) (bits ^ (bits >>> 32));
	 * result += obj.hashCode() ^ temp;
	 * </PRE>
	 *
	 * <p>This is similar to the hash code used by <CODE>java.util.Map.Entry</CODE> and uses ideas from
	 * Bloch's book entitled Effective Java.  It ensures that two weighted sets have the same hash value
	 * if they contain the same elements with the same weights.</p>
	 *
	 * @return the hash code for this set.
	 */	
	public int hashCode() {
		int result = 17;
		for (Iterator iterator = _map.keySet().iterator(); iterator.hasNext();) {
			Object obj = iterator.next();
			//result += obj.hashCode() ^ (int) get(obj);
			long bits = Double.doubleToLongBits(get(obj));
			int temp = (int) (bits ^ (bits >>> 32));
			result += obj.hashCode() ^ temp;
		}
		
		return result;
	}
	
	/** Returns an iterator over the elements in this set.
	 *
	 * @return an iterator over the elements in this set.
	 */	
	public Iterator iterator() {
		//return _map.keySet().iterator();
		_iterating = true;
		return new Iterator() {
			private Iterator _iterator = _map.keySet().iterator();
			public boolean hasNext() {
				boolean hasNext = _iterator.hasNext();
				if (!hasNext) { 
					cleanup();
					_iterating = false;
				}
				return hasNext;
			}
			public Object next() { return _iterator.next(); }
			public void remove() { _iterator.remove(); }
		};
	}
	
	/** Removes the specified element from this set if it is present.
	 *
	 * @param obj the object to be removed from this set.
	 * @return <CODE>true</CODE> if this set contained the specified object.
	 */	
	public boolean remove(Object obj) {
		return _map.remove(obj) == null ? false : true;
	}
	
	/** Sets the weight of the specified object to the specified value.  If this set does not contain the
	 * specified object, it is added to this set with the specified weight.  If the weight is equal to
	 * <CODE>0.0d</CODE>, the specified object is removed from this set unless this set is being iterated
	 * over (i.e. the <CODE>iterator</CODE> method was called). If this set is being iterated over, the
	 * object's weight is immediately set to 0.0 and the object is then removed from this set once the
	 * iteration is finished (i.e. the <CODE>Iterator.hasNext</CODE> method returns <CODE>false</CODE>).
	 *
	 * @param obj the object whose weight is to be adjusted.
	 * @param weight the new weight of the specified object.
	 */	
	public void set(Object obj, double weight) {
		if (obj == null) { throw new NullPointerException("obj was null"); }
		
		Weight w = (Weight) _map.get(obj);
		if (w == null) {
			if (weight != 0.0d) { _map.put(obj, new Weight(weight)); }
		} 
		else {
			if (weight != 0.0d) { w.weight = weight; }
			else {
				if (_iterating) { w.weight = 0.0d; }
				else { _map.remove(obj); }
			}
		}
	}
	
	/** Returns the number of elements in this set.
	 *
	 * @return the number of elements in this set.
	 */	
	public int size() {
		return _map.size();
	}
	
	/** Returns an array containing all of the elements in this set.
	 *
	 * @return an array containing all of the elements in this set.
	 */	
	public Object[] toArray() {
		return _map.keySet().toArray();
	}
	
	/** Returns an array containing all of the elements in this set whose runtime type is that of the
	 * specified array.
	 *
	 * @param obj the array into which the elements of this set are to be stored, if it is big enough; otherwise,
	 * a new array of the same runtime type is allocated for this purpose.
	 * @return an array containing the elements of this set.
	 */	
	public Object[] toArray(Object[] obj) {
		return _map.keySet().toArray(obj);
	}
	
	/** Returns the string representation of this set.
	 *
	 * @return the string representation of this set.
	 */	
	public String toString() {
		return _map.toString();
	}
	
}

/** A convenience class for holding the weights of a weighted set.
 *
 */
class Weight {
	public double weight = 0.0d;
	
	/** Creates a new <CODE>Weight</CODE> with the value <CODE>0.0d</CODE>.
	 *
	 */	
	public Weight() {
		
	}
	
	/** Creates a new <CODE>Weight</CODE> with the specified value.
	 *
	 * @param weight the value of this weight.
	 */	
	public Weight(double weight) {
		this.weight = weight;
	}
	
	/** Returns the string representation of this weight.
	 *
	 * @return the string representation of this weight.
	 */	
	public String toString() {
		return Double.toString(weight);
	}
}
