/* $Id: GEMLayout.java,v 1.3 2002/07/31 08:39:51 adamtj Exp $
 *
 *
 */
package fcmodeler.layout;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.Map;
import java.util.HashMap;
import java.util.Random;
import java.util.Date;
import java.awt.geom.Point2D;
import java.awt.Point; // Integral coordinates
import java.lang.Math;
import java.lang.Integer;

import diva.canvas.Figure;
import diva.canvas.Site;
import diva.canvas.connector.AbstractConnector;
import diva.canvas.connector.StraightConnector;
import diva.canvas.connector.AutonomousSite;
import diva.graph.BasicEdgeRenderer;
import diva.graph.model.Node;
import diva.graph.model.Edge;

import fcmodeler.FCModeler;
import fcmodeler.interaction.GeneralConnector.GeneralConnector;
import fcmodeler.view.EdgeFigure;

/**
 *  GEM (Graph EMbedder) is a spring embedder style layout algorithm by
 *  A. Frick.  See www.frick-consulting.de/publications.html
 *
 * @author Adam Tomjack <adamtj@iastate.edu>
 *
 * @version $Revision: 1.3 $
 * @since JDK1.3
 */
public class GEMLayout extends AbstractLayout {//implements Runnable {
	
	//Debug statments
	protected boolean _verbose = true;
	//Output temps and completion percentange.
	protected boolean _statusReport = false;
	
	// 'i' is for Insert
	// 'a' is for Arrange
	// 'o' is for Optimize
	protected double i_maxTemp = 1.0;
	protected double i_startTemp = 0.3;
	protected double i_finalTemp = 0.05;
	protected double i_maxIter = 10;
	protected double i_gravity = 0.05;
	protected double i_oscillation = 0.4;
	protected double i_rotation = 0.5;
	protected double i_shake = 0.2;
	protected double a_maxTemp = 1.5;
	protected double a_startTemp = 1.0;
	protected double a_finalTemp = 0.02;
	protected double a_maxIter = 3;
	protected double a_gravity = 0.1;
	protected double a_oscillation = 0.4;
	protected double a_rotation = 0.9;
	protected double a_shake = 0.3;
	protected double o_maxTemp = 0.25;
	protected double o_startTemp = 0.05;
	protected double o_finalTemp = 0.02;
	protected double o_maxIter = 3;
	protected double o_gravity = 0.1;
	protected double o_oscillation = 0.4;
	protected double o_rotation = 0.9;
	protected double o_shake = 0.3;
	
	/** I think this is some sort of scaling constant related to the fact that
	 *  the algorithm is integer based.
	 */
	protected int _elen = 128;
	protected int _MAXATTRACT = 1048576;
	
	protected int _temperature = 0;
	protected int _maxTemp = 0;
	/** The barycenter of the graph multiplied by the number of nodes.  In other
	 *  words, _center represents the vector sum of the positions of all the nodes
	 *  To find the center of mass, simply divide by the number of nodes.
	 */
	protected Point _center = new Point(0,0);
	
	protected double _oscillation = 0.0;
	protected double _rotation = 0.0;
	protected int _iteration;
	
	protected int _numNodes = 0;
	protected int _numEdges = 0;
	
	/** Node state information. */
	protected NodeInfo[] _nodeData;
	
	//miscellaneous structures.
	protected int[] _nodesPermutation;
	protected Map _nodesToIndices;
	
	protected int _layoutType = 0;
	
	//A random number generator.
	protected Random _rand = new Random();
	
	protected class NodeInfo {
		int arrayIndex; //the index into the array of Nodes
		Point pos; // current position
		int mass; // mass/weight
		int heat; // current temperature
		Point impulse; // last impulse
		int dir; // direction gauge, for detection of rotations
		int degree; // number of incident edges
		int depth; // used as mark or counter in breadthFirstSearch
		int x, y;	// scaled coordinates
		int weight;
		public NodeInfo() {
			pos = new Point(0,0);
			mass = 0;
			heat = 0;
			impulse = new Point(0,0);
			dir = 0;
			degree = 0;
			depth = 0;
			x = y = 0;
		}
	}
	///////////////////////////////////////////////////////////////////////////////////////////////////////
	public GEMLayout(FCModeler fcmodeler, int layoutType) {
		super(fcmodeler);
		/* Type 0 is mostly true to A. Frick's original algorithm.  It does not do
		 * the optimization step.  It may or may not use his insertion routine.
		 * I find that it doesn't significantly speed up the algorithm.
		 *
		 * Type 1 uses my modified impulse function.  It is a fairly radical 
		 * departure from Frick's algorithm.  The results are visually similar,
		 * but an order of N has been removed from the run time.
		 *
		 * Type 2 uses the fast GEM (type 1) to do the initial layout, then uses
		 * type 0 to touch it up.
		 */
		
		_layoutType = layoutType;
	}
	
	/** Loop through all nodes and displace() each once. */
	public void a_round(int impulseFunction) {
		int v;
		permuteNodes();
		for (int i = 0; i < _numNodes; i++) {
			v = _nodesPermutation[i];
			if (impulseFunction == 0) {
				displace(v, impulse(v, null, a_shake, a_gravity));
			}
			if (impulseFunction == 1) {
				displace(v, a_impulse(v, null, a_shake, a_gravity));
			}
			
			_iteration ++;
		}
	}
	
	/** Move the nodes.  This is the main computation of this layout algorithm.
	 */
	public void arrange(int impulseFunction) {
		//impulseFunction = 0 selects the standard GEM impulse fn.
		//impulseFunction = 1 selects a faster impulse fn.
		
		int stopTemperature;
		long stopIteration;
		
		initVertexData(a_startTemp);
		_oscillation = a_oscillation;
		_rotation = a_rotation;
		_maxTemp = (int)a_maxTemp * _elen;
		stopTemperature = (int)(a_finalTemp * a_finalTemp * (double)_elen*(double)_elen * (double) _numNodes);
		stopIteration = (int)(a_maxIter * (double)_numNodes * (double)_numNodes);
		_iteration = 0;
		/*
		 int it = 0;
		int stopit = 999999;
		if (_layoutType == 2) {
			stopit = 30;
		} else {
			stopit = 200;
		}*/
		while (_temperature > stopTemperature && _iteration < stopIteration) {
			if(_statusReport) {
				
				System.out.print("Temp: ");
				System.out.print(_temperature);
				System.out.print(", Stop: ");
				System.out.print(stopTemperature);
				System.out.print(", Iter: ");
				String percent = Double.toString((double)_iteration/(double)stopIteration);
				try {
					percent = percent.substring(0,4);
				} catch (Exception e) {
				}
				System.out.println(percent);
				
			}
			a_round(impulseFunction);
		}
	}
	
	/** This isn't a breadth first search.  It uses one to calculate a number which
	 *  represents the centralness of the given node.  It finds the minimum distance
	 *  between every node and the specified node, adds those distances, and returns
	 *  that number.
	 */
	public int breadthFirstSearch(int root) {
		int weight = 0;
		int depth = 0;
		List q = new Vector();
		//_nodeData[root].depth = depth;
		q.add(new Integer(root));
		while (q.size()>0) {
			depth++;
			List l = getAdjacentNodes(((Integer)q.remove(0)).intValue(), getAllActive());
			Iterator i = l.iterator();
			while (i.hasNext()) {
				Integer index =  (Integer) i.next();
				if (_nodeData[index.intValue()].depth ==  0) {
					q.add(index);
				}
				_nodeData[index.intValue()].depth = depth;
				weight += depth;
			}
		}
		
		return weight;
	}
	
	/** Find the center of mass of a List of nodes.  This does not return a node.
	 *  graphCenter() returns a node which at the center.  Given a layout, this
	 *  computes the center of mass of the layout.  This center of mass may or
	 *  may not be located at a node's position.
	 */
	public Point centerOfMass(List nodes) {
		Iterator i = nodes.iterator();
		int x =  0;
		int y = 0;
		while (i.hasNext()) {
			int index = ((Integer)i.next()).intValue();
			x += _nodeData[index].pos.x;
			y += _nodeData[index].pos.y;
		}
		x /=  _nodes.size();
		y /=  _nodes.size();
		
		return new Point(x,y);
	}
	
	/** This is the main function which drives the layout.
	 */
	public void computeLayout() {

		init();
		
		long startTime = (new Date()).getTime();
		long insertTime = startTime;
		if (_layoutType == 0) {
			//circleInsert();
			insert();
			insertTime = (new Date()).getTime();
			arrange(0);
		}
		if (_layoutType == 1) {
			circleInsert();
			insertTime = (new Date()).getTime();
			arrange(1);
		}
		if (_layoutType == 2) {
			circleInsert();
			arrange(1);
			insertTime = (new Date()).getTime();
			arrange(0);
		}
		long doneTime = (new Date()).getTime();
		
		if (_verbose || _statusReport) {
			System.out.print("Time to insert: ");
			System.out.print((double)(insertTime-startTime)/1000.0);
			System.out.println(" (s)");
			System.out.print("Time to arrange: ");
			System.out.print((double)(doneTime-insertTime)/1000.0);
			System.out.println(" (s)");
			System.out.print("Total time: ");
			System.out.print((double)(doneTime-startTime)/1000.0);
			System.out.println(" (s)");
		}
		
		/*
		
		if(_statusReport) {
			System.out.println("Inserting Nodes");
		}
		//randomInsert();
		circleInsert();
		//insert();
		arrange(1);
		
		//if (a_finalTemp < a_startTemp)
		if(_statusReport) {
			now = new Date();
			long insertTime = now.getTime();
			System.out.print("Time to insert: ");
			System.out.print((double)(insertTime-startTime)/1000.0);
			
			
			System.out.println("Arranging Nodes");
		}
		long arrangeStart = (new Date()).getTime();
		arrange(0);
		//if (o_finaltemp < o_starttemp)
		if(_statusReport) {
			now = new Date();
			long arrangeTime = now.getTime();
			System.out.print("Time to Arrange: ");
			System.out.print((double)(arrangeTime-arrangeStart)/1000.0);
			System.out.println("(s)");
			//System.out.println("Optimizing layout");
		}
		//optimize();
		
		if(_statusReport) {
			now = new Date();
			long doneTime = now.getTime();
			System.out.print("Total time: ");
			System.out.print((double)(doneTime-startTime)/1000.0);
			System.out.println("(s)");
		}*/
		//Positions the Diva Nodes.
		moveNodes();
	}
	
	/** Given the previously computed impulse, move the node, update the necessary
	 *  variables, check for oscillation and rotation, and update the
	 *  temperatures.
	 */
	public void displace(int nodeIndex, Point impulse) {
		int t, n;
		Point imp;
		
		if (impulse.x!=0 || impulse.y!=0) {
			
			//I'm not quite sure what this does.  It looks like it's roughly
			//putting an upper bound on the size of the impulse.  This is probably
			//So that the original 16 bit integer implementation didn't overflow when
			//squaring.
			n = Math.max(Math.abs(impulse.x), Math.abs(impulse.y)) / 16384;
			if (n > 1) {
				impulse.x /= n;
				impulse.y /= n;
			}
			t = _nodeData[nodeIndex].heat;
			//The magnitude of the impulse.
			n = (int)Math.sqrt(impulse.x*impulse.x+impulse.y*impulse.y);
			//Normalize the impulse?!?! Then multiply by the heat.
			impulse.x = impulse.x * t / n;
			impulse.y = impulse.y * t / n;
			//Add the impulse to the position.
			_nodeData[nodeIndex].pos.x += impulse.x;
			_nodeData[nodeIndex].pos.y += impulse.y;
			//Add the impulse to the center coordinates.  Remember, center isn't
			//actually a point.  It's the vector sum of the positions of all the
			//nodes.  To calculate the center, you have to divide by the number
			//of nodes.  Adding the impulse to this quantity is a fast way to
			//reflect the change in position of the current node.
			_center.x += impulse.x;
			_center.y += impulse.y;
			
			//Check for rotations or oscillations and decrease the global
			//temperature.
			imp = _nodeData[nodeIndex].impulse;
			n = t * (int)Math.sqrt(imp.x*imp.x + imp.y*imp.y);
			if (n!=0) {
				//Decrease the global temperature.
				_temperature -= t * t;
				//Increase the local temperature in some odd way.
				t += t * _oscillation * (impulse.x * imp.x + impulse.y * imp.y) / n;
				//Don't let the local temp get too big.
				t = Math.min(t, _maxTemp);
				//Find the direction the node moved
				_nodeData[nodeIndex].dir += _rotation * (impulse.x * imp.y - impulse.y * imp.x) / n;
				//Decrease the local temp in some odd way.
				t -= t * Math.abs(_nodeData[nodeIndex].dir) / _numNodes;
				//Don't let it get too small.
				t = Math.max(t, 2);
				//Increase the global temperature, presumably by less than we
				//decreased it above.
				_temperature += t * t;
				//Store the new local temperature.
				_nodeData[nodeIndex].heat = t;
			}
			
			//Save the previous impulse to make sure we aren't rotating or
			//oscillating.
			_nodeData[nodeIndex].impulse = new Point(impulse);
		}
	}
	
	/** Build a list of nodes adjacent to the indicated node.
	 */
	public List getAdjacentNodes(int node, boolean[] activeNodes) {
		
		List adjacentNodes = new Vector();
		Node n = (Node)_nodes.get(node);
		Iterator i = n.inEdges();
		while (i.hasNext()) {
			Edge e = (Edge) i.next();
			Node tail = (Node) e.getTail();
			int index = ((Integer)_nodesToIndices.get(tail)).intValue();
			if (activeNodes[index]) {
				adjacentNodes.add(new Integer(index));
			}
		}
		i = n.outEdges();
		while (i.hasNext()) {
			Edge e = (Edge) i.next();
			Node head = (Node) e.getHead();
			int index = ((Integer)_nodesToIndices.get(head)).intValue();
			if (activeNodes[index]) {
				adjacentNodes.add(new Integer(index));
			}
		}
		return adjacentNodes;
	}
	
	/** Some functions require a list of nodes that should be considered during
	 *  calculations.  This returns a such a list where every node is turned
	 *  on by default.
	 */
	public boolean[] getAllActive() {
		boolean[] a = new boolean[_numNodes];
		for(int i=0; i<a.length; i++) {
			a[i] = true;
		}
		return a;
	}
	/** like getAllActive() except all nodes are off by default.
	 */
	public boolean[] getNoneActive() {
		boolean[] a = new boolean[_numNodes];
		for(int i=0; i<a.length; i++) {
			a[i] = false;
		}
		return a;
	}
	
	/** Find the index of the node that is at the center of the graph.
	 *  What constitutes the center is debatable.  The GEM algorithm as presented
	 *  in Frick's implementation finds the node where the maximum distance between
	 *  that node and any other is less than the max distance with any other node
	 *  as the reference point.  I think a more appropriate center point could be
	 *  computed in the following manner:  Select a reference node.  Find the
	 *  minimum distance between that node and every other node.  Sum the distances.
	 *  The center is the reference node for which this sum is minimized.
	 *  Consider a complete graph with a long tail attached to it.  Since every
	 *  node in the complete subgraph is separated by a distance of one, the center
	 *  point would be one edge away from the center of the tail subgraph by Frick's
	 *  algorithm.  Mine would select the node to which the tail is attached.
	 *  In summary: Frick chooses the center of area where I choose the center of
	 *  mass.
	 *
	 *  For now however, I'll use the first node as the center.  I'll mess with this
	 *  later.
	 */
	public int graphCenter() {
		int minWeight = -1;
		int index = -1;
		for (int w=0; w<_numNodes; w++) {
			_nodeData[w].weight = breadthFirstSearch(w);
			if (_nodeData[w].weight < minWeight || minWeight == -1) {
				minWeight = _nodeData[w].weight;
				index = w;
			}
		}
		return index;
	}
	
	/** Given a list of nodes to consider, compute the sum of the forces on a node.
	 */
	public Point impulse(int nodeIndex, boolean[] activeNodes, double shake, double gravity) {
		if (activeNodes == null) {
			activeNodes = getAllActive();
		}
		int u;
		int e;
		Point i = new Point();
		Point d = new Point();
		int	n;
		Point p = new Point(_nodeData[nodeIndex].pos);
		
		//The max distance that the node can be shake
		n = (int) shake * _elen;
		//Pick a random number in the range [-n,n].  Start by picking a random
		//number in the range [0,2n] and translating.
		i.x = _rand.nextInt() % (2 * n + 1) - n;
		i.y = _rand.nextInt() % (2 * n + 1) - n;
		//Compute the gravity:
		//Center isn't actually the center.  It's coordinates are the sums of the
		//corresponding coordinates of all the nodes.  The coordinates need to
		//be divided by the number of nodes to get the actual center coordinates.
		//So, find the delta x between this node and the center, then multiply
		//by the mass and the gravity.  It isn't true gravity.  The farther the
		//node is from the center, the stronger the gravity is.  It's actually like
		//a streched spring instead of gravity.
		i.x += (_center.x / _numNodes - p.x) * _nodeData[nodeIndex].mass * gravity;
		i.y += (_center.y / _numNodes - p.y) * _nodeData[nodeIndex].mass * gravity;
		//Calculate the attraction to all other nodes.
		//Notice that 'n' is the square of the distance between the nodes.
		//dividing the separation vector by n has the effect of first normalizing
		//the separation vector, then setting its magnitute to the inverse of
		//the distance.  Notice the sign of the impulse.  This pushes nodes
		//farther apart.
		//  So, the gravity calculation above is actually like a stretched spring
		//which connects every node to the center of mass.  This is like a
		//compressed spring which pushes all nodes away from each other.
		//There may be a net contraction or expansion of the layout depending on the
		//relative strengths of the forces.
		//  On a large scale, the gravitational force of a system of particles such
		//as we have here is equivalent to all the particles being located at their
		//center of mass.  On the scale of individual particles, the force is more
		//strongly directed towards it's neighbors.  A collapsing system of particles
		//will first lump up.  That process could be used for clustering data.
		//  I think this function is using similar concepts.  There is the large
		//scale force that attracts all nodes to the center of mass.  The attractive
		//force is not like gravity, but like a spring.  In addition, there is a
		//repulsive force between every two nodes that pushes them apart.
		for (u=0; u<_numNodes; u++) {
			//Only work on nodes that have already been inserted.
			if ( ! activeNodes[u] ) {
				continue;
			}
			//Note the sign.  This is a repulsive force.
			d.x = p.x - _nodeData[u].pos.x;
			d.y = p.y - _nodeData[u].pos.y;
			n = d.x * d.x + d.y * d.y;
			if (n!=0) {
				i.x += d.x * _elen*_elen / n;
				i.y += d.y * _elen*_elen / n;
			}
		}
		//Here we connect a stretched spring between this node and each of its
		//neighbors.  This tends to clump connected nodes together.  The previous code
		//will push the clumps apart.  The "gravity" will keep the layout from
		//exploding.
		//  Actually, they aren't connected with a spring.  They are connected by
		//a quadratic spring.  It's like the inverse of true gravity.  Double
		//the distance and you'll quadruple the attractive force.
		Iterator iter= getAdjacentNodes(nodeIndex, activeNodes).iterator();
		while (iter.hasNext()) {
			u = ((Integer)iter.next()).intValue();
			//Only work on nodes that have already been inserted.
			//Hmm, getAdjacentNodes takes care of this.
			//if ( ! activeNodes[u] ) {
			//	continue;
			//}
			d.x = p.x - _nodeData[u].pos.x;
			d.y = p.y - _nodeData[u].pos.y;
			n = (d.x * d.x + d.y * d.y) / _nodeData[nodeIndex].mass;
			n = Math.min(n, _MAXATTRACT);
			//Note the minus.  This force is attractive.
			i.x -= d.x * n / (_elen*_elen);
			i.y -= d.y * n / (_elen*_elen);
		}
		return i;
	}
	
	/** Given a list of nodes to consider, compute the sum of the forces on a node.
	 */
	public Point a_impulse(int nodeIndex, boolean[] activeNodes, double shake, double gravity) {
		if (activeNodes == null) {
			activeNodes = getAllActive();
		}
		int u;
		int e;
		Point i = new Point();
		Point d = new Point();
		int	n;
		Point p = new Point(_nodeData[nodeIndex].pos);
		
		n = (int) shake * _elen;
		i.x = _rand.nextInt() % (2 * n + 1) - n;
		i.y = _rand.nextInt() % (2 * n + 1) - n;
		
		Iterator iter= getAdjacentNodes(nodeIndex, activeNodes).iterator();
		
		if (iter.hasNext()) {
			//int power = _numNodes*_numNodes*10000;
			int power = _numNodes*1000;
			//int power = 1000;
			double dx = (_center.x / _numNodes - p.x);
			double dy = (_center.y / _numNodes - p.y);
			double distance= Math.sqrt(dx*dx+dy*dy);
			distance = (distance==0)?0.01:distance;
			i.x -= (int)(power*dx)/(distance*distance*distance);
			i.y -= (int)(power*dy)/(distance*distance*distance);
			
			//Linear separation.  This makes sure that it doesn't collapse.
			//This is a rough approximation of the inverse of the net inward force 
			//due to the spring tension.  For singletons, however, it reverses
			//directions to pull them in.
			//FIXME: it is not appropriate to force all singletons to the barycenter.
			double sepFact = 1.0;
			if (_nodeData[nodeIndex].degree != 0) {
				i.x -= (int)(sepFact*_numEdges*dx)/(distance);
				i.y -= (int)(sepFact*_numEdges*dy)/(distance);
			} else {
				i.x += (int)(sepFact*_numEdges*dx)/(distance);
				i.y += (int)(sepFact*_numEdges*dy)/(distance);				
			}
		}
		//Iterator iter= getAdjacentNodes(nodeIndex, activeNodes).iterator();
		while (iter.hasNext()) {
			u = ((Integer)iter.next()).intValue();
			d.x = p.x - _nodeData[u].pos.x;
			d.y = p.y - _nodeData[u].pos.y;
			n = (d.x * d.x + d.y * d.y) / _nodeData[nodeIndex].mass;
			n = Math.min(n, _MAXATTRACT);
			//Note the minus.  This force is attractive.
			i.x -= d.x * n / (_elen*_elen);
			i.y -= d.y * n / (_elen*_elen);
		}
		return i;
	}
	
	/** Set up some global variables.  This only needs to be run once, unlike
	 *  initVertexData which may be run more than once.
	 */
	public void init() {
		_numNodes = _nodes.size();
		_numEdges = _edgesToEdgeFigures.keySet().size();
		
		_nodeData = new NodeInfo[_numNodes];
		_nodesPermutation = new int[_numNodes];
		for(int c=0; c<_numNodes; c++) {
			_nodesPermutation[c] = c;
		}
		
		_nodesToIndices = new HashMap();
		//Calculate the degree of each Node.
		//Also set up the _nodesToIndices Map
		for (int c=0; c<_numNodes; c++) {
			//Iterators can be so dumb sometimes.
			Iterator i = ((Node)_nodes.get(c)).inEdges();
			int degree = 0;
			while (i.hasNext()) {
				i.next();
				degree++;
			}
			i = ((Node)_nodes.get(c)).outEdges();
			while (i.hasNext()) {
				i.next();
				degree++;
			}
			_nodeData[c] = new NodeInfo();
			_nodeData[c].degree = degree;
			
			_nodesToIndices.put(_nodes.get(c), new Integer(c));
		}
	}
	
	/** Set the initial temperature of each node, find the total temperature
	 *  of the graph, and compute the center point.  This does not affect the
	 *  positions, so it can be used without affecting previous layout work.
	 */
	public void	initVertexData(double startTemp) {
		_temperature = 0;
		_center = new Point(0,0);
		for(int i=0; i<_numNodes; i++) {
			_nodeData[i].heat = (int)startTemp * _elen;
			_temperature += _nodeData[i].heat * _nodeData[i].heat;
			_nodeData[i].impulse = new Point(0,0);
			_nodeData[i].dir = 0;
			_nodeData[i].mass = 1 + _nodeData[i].degree / 3;
			if (_nodeData[i].pos !=null ) {
				_center.x += _nodeData[i].pos.x;
				_center.y += _nodeData[i].pos.y;
			}
		}
	}
	
	/** Insert the nodes.  This could be random, but instead we do some extra work
	 *  to get close to the final layout to speed things up.
	 *  It works like this:  Place the first node at (0,0).
	 *  Loop {
	 *    Pick a node and place it at the center of mass of the adjacent nodes that
	 *    have already been placed.
	 *    Then do the spring embedder displacing thing with all nodes already
	 *    placed to get it close to its final position.
	 *  }
	 *  This doesn't appear to work very well or to provide any speed enhancement
	 *  over a random insertion.  Perhaps I mistranslated it.
	 */
	public void insert() {
		
		boolean[] activeNodes = getNoneActive();
		List insertedNodes = new Vector();
		List tempNodes = new Vector();
		
		boolean in[] =  getNoneActive();
		
		int centerNode = graphCenter();
		_nodeData[centerNode].pos.setLocation(0,0);
		tempNodes.add(new Integer(centerNode));
		
		int stopTemperature;
		long stopIteration;
		
		initVertexData(i_startTemp);
		randomInsert();
		initVertexData(i_startTemp);
		_oscillation = i_oscillation;
		_rotation = i_rotation;
		_maxTemp = (int)i_maxTemp * _elen;
		stopTemperature = (int)i_finalTemp * (int)i_finalTemp * _elen*_elen * _numNodes;
		stopIteration = (int)i_maxIter * _numNodes * _numNodes;
		_iteration = 0;
		
		for (int c=0; c<_numNodes; c++) {
			if (in[c] == false) {
				tempNodes.add(new Integer(c));
			} else {
				continue;
			}
			if(c==0){
				tempNodes.remove(1);
			}
			while (tempNodes.size()>0) {
				Integer node = (Integer)tempNodes.remove(0);
				
				int workingNode = node.intValue();
				if ( in[node.intValue()] )  {
					continue;
				}
				
				//if (_verbose) {
					//System.out.print("Inserting ");
					//System.out.println(((Node)_nodes.get(node.intValue())).getProperty("label"));
				//}
				
				List neighbors = getAdjacentNodes(workingNode, getAllActive());
				Iterator i = neighbors.iterator();
				while (i.hasNext()) {
					node = (Integer) i.next();
					if ( in[node.intValue()] == false )  {
						//System.out.print("Inserting ");
						//System.out.println(((Node)_nodes.get(node.intValue())).getProperty("label"));
						//System.out.println(node.intValue());
						
						tempNodes.add(node);
					}
				}
				
				Point com = centerOfMass(insertedNodes);
				/*
				if(_verbose) {
					System.out.print("  insertion point: ");
					System.out.print(com.x);
					System.out.print(",");
					System.out.println(com.y);
				}
				 */
				
				if (com.x==0 && com.y==0) {
					//com.setLocation(200,200);
				}
				
				com.x += _rand.nextInt(20);
				com.y += _rand.nextInt(20);
				_nodeData[workingNode].pos = new Point(com);
				
				int d=0;
				//System.out.print("displacing ");
				//System.out.println(workingNode);
				while ((d++ < i_maxIter) && (_nodeData[workingNode].heat > i_finalTemp * _elen)) {
					displace(workingNode, impulse(workingNode, in, i_shake, i_gravity));
				}
				com  = _nodeData[workingNode].pos;
				/*
				if(_verbose) {
					System.out.print("  final point: ");
					System.out.print(com.x);
					System.out.print(",");
					System.out.println(com.y);
					System.out.print("  max iter:  ");
					System.out.println(d);
				}
				 */
				
				in[workingNode] = true;
				insertedNodes.add(new Integer(workingNode));
			}
		}
	}
	
	public void optimize() {
	}
	public void o_round() {
	}
	
	/** Insert the nodes along the perimeter of a circle.
	 */
	public void circleInsert() {
		double radius = 175;
		double dTheta = 2*Math.PI/_numNodes;
		double theta = 0;
		permuteNodes();
		for (int i=0; i<_numNodes-1; i++) {
			int n = _nodesPermutation[i];
			int x = (int)(radius * Math.cos(theta));
			int y = (int)(radius * Math.sin(theta));
			_nodeData[n].pos.setLocation(x, y);
			theta += dTheta;
		}
	}
	
	/** Insert the nodes randomly.
	 */
	public void randomInsert() {
		for (int i=0; i<_numNodes-1; i++) {
			_nodeData[i].pos.setLocation(_rand.nextInt(401), _rand.nextInt(401));
		}
	}
	
	/** Translate the nodes so that they are all in the correct quadrant
	 *  and there is no extra padding.  Then place each Diva Node in it's correct
	 *  position.
	 */
	public void moveNodes() {
		//Find minx and miny
		int minx = _nodeData[0].pos.x;
		int miny = _nodeData[0].pos.y;
		int padx = 30;
		int pady = 20;
		double xscale = 1;
		double yscale = xscale;
		
		//FIXME: What a gimpy hack...
		if (_layoutType == 1 && _numNodes < 100) {
			xscale = 2;
			yscale = xscale;			
		}
		
		for (int c=0; c<_numNodes; c++) {
			if (_nodeData[c].pos.x < minx) {
				minx = _nodeData[c].pos.x;
			}
			if (_nodeData[c].pos.y < miny) {
				miny = _nodeData[c].pos.y;
			}
		}
		
		//Adjust the coordinates
		for (int c=0; c<_numNodes; c++) {
			_nodeData[c].pos.x -= minx;
			_nodeData[c].pos.y -= miny;
			
			_nodeData[c].pos.x *= xscale;
			_nodeData[c].pos.y *= yscale;
			
			_nodeData[c].pos.x += padx;
			_nodeData[c].pos.y += pady;
		}
		
		Iterator i = _nodes.iterator();
		while (i.hasNext()) {
			Node n = (Node) i.next();
			int index = ((Integer)_nodesToIndices.get(n)).intValue();
			placeNode(n, _nodeData[index].pos.x, _nodeData[index].pos.y);
		}
		
		i = _edgesToEdgeFigures.keySet().iterator();
		while(i.hasNext()) {
			Edge e = (Edge)  i.next();
			EdgeFigure edgeFigure = (EdgeFigure)_edgesToEdgeFigures.get(e);
			//EdgeFigure edgeFigure = (EdgeFigure) i.next();
			int hi = ((Integer)_nodesToIndices.get(e.getHead())).intValue();
			int ti = ((Integer)_nodesToIndices.get(e.getTail())).intValue();
			Point hp = _nodeData[hi].pos;
			Point tp = _nodeData[ti].pos;
			List coords = new Vector();
			coords.add(new Point2D.Double(hp.x,hp.y));
			coords.add(new Point2D.Double((hp.x+tp.x)/2,(hp.y+tp.y)/2));
			coords.add(new Point2D.Double((hp.x+tp.x)/2,(hp.y+tp.y)/2));
			coords.add(new Point2D.Double(tp.x,tp.y));
			if (edgeFigure instanceof GeneralConnector){
				((GeneralConnector) edgeFigure).fromFile(coords);
			} else {
				throw new UnsupportedOperationException("SpecifiedLayout.layout(): EdgeFigure not supported!");
			}
		}
	}
	
	/** Permute the nodes using the Fischer Yates shuffle.
	 *  Frick has a function called select() that returned the index of a node.
	 *  It did the shuffling a step at a time as you called select().  That seems
	 *  unnecessary to me, so I chose to do the entire permutation at once.  It
	 *  makes things a little simpler.
	 *
	 *  I just noticed that java.util.Collections.shuffle(List) could work,
	 *  but this is already done.  Using java.util.Arrays.asList() would
	 *  make using the shuffle() function easier.
	 */
	public void permuteNodes() {
		//FIXME: Check this for validity.
		for (int i=_numNodes-1; i>0; i--) {
			int x = _rand.nextInt(i);
			int temp = _nodesPermutation[i];
			_nodesPermutation[i] = _nodesPermutation[x];
			_nodesPermutation[x] = temp;
		}
	}
}

/*
 * $Log: GEMLayout.java,v $
 * Revision 1.3  2002/07/31 08:39:51  adamtj
 * GEM now outputs correct run times.
 *
 * Revision 1.2  2002/07/31 08:31:59  adamtj
 * added variations on the GEM layout.
 *
 * Revision 1.1  2002/07/26 23:09:05  adamtj
 * Added the files necessary for the GEM layout algorithm.
 *
 *
 */
