/* $Id: AtomicSentence.java,v 1.1 2002/02/15 20:23:57 zcox Exp $
 *
 *
 */

package fcmodeler.mapping;

import diva.util.PropertyContainer;

/** <p>An atomic sentence in first-order logic.  This sentence is of the form </p>
 *
 * <p><PRE>ValueOf(PropertyContainer, Property) = PropertyValue</PRE></p>
 *
 * <p>The Property is represented by a String, and the PropertyValue by an Object.  The
 * PropertyContainer is a variable and is supplied to the <CODE>evaluate</CODE> method.</p>
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class AtomicSentence extends Object implements Sentence {
	private final String _property;
	private final Object _value;
	private final String _string;
	
	/** Creates a new <CODE>AtomicSentence</CODE> that uses the specified Property and PropertyValue.
	 *
	 * @param property the Property of this atomic sentence.
	 * @param value the PropertyValue of this atomic sentence.
	 */
	public AtomicSentence(String property, Object value) {
		_property = property;
		_value = value; // NOTE: this does not guarantee immutability; the Object could change state!
		_string = "(" + _property + " = " + _value.toString() + ")";
	}
	
	/** Evaluates this sentence using the specified <CODE>PropertyContainer</CODE>.
	 *
	 * @param container the <CODE>PropertyContainer</CODE> used to evaluate this sentence.
	 * @return <CODE>true</CODE> if the Property of the specified <CODE>PropertyContainer</CODE> equals the PropertyValue; <CODE>false</CODE> otherwise.
	 */	
	public boolean evaluate(PropertyContainer container) {
		Object value = container.getProperty(_property);
		return value == null ? false : value.equals(_value);
	}
	
	/** Returns the Property of this atomic sentence.
	 * @return the Property of this atomic sentence.
	 */	
	public String getProperty() {
		return _property;
	}
	
	/** Returns the PropertyValue of this atomic sentence.
	 * @return the PropertyValue of this atomic sentence.
	 */	
	public Object getPropertyValue() {
		return _value; // NOTE: returning the internal reference allows the Object to be mofified in client code!
	}
	
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: AtomicSentence.java,v $
 * Revision 1.1  2002/02/15 20:23:57  zcox
 * big time update
 *
 *
 *
 */
