/* $Id: CompositeSentence.java,v 1.1 2002/02/15 20:23:57 zcox Exp $
 *
 *
 */

package fcmodeler.mapping;

import diva.util.PropertyContainer;

/** <p>A composite sentence in first-order logic.  This sentence is of the form</p>
 *
 * <p><PRE>Sentence Connective Sentence</PRE></p>
 *
 * <p>This sentence can be used to build up complex sentences using other sentences and
 * connectives.</p>
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class CompositeSentence extends Object implements Sentence {
	private final Sentence _first;
	private final Sentence _second;
	private final Connective _connective;
	private final String _string;
	
	/** Creates a new <CODE>CompositeSentence</CODE> that uses the specified <CODE>Sentence</CODE>s
	 * and <CODE>Connective</CODE>.
	 *
	 * @param first the first sentence of this composite sentence.
	 * @param connective the connective used to join the first and second sentences.
	 * @param second the second sentence of this composite sentence.
	 */
	public CompositeSentence(Sentence first, Connective connective, Sentence second) {
		_first = first;
		_connective = connective;
		_second = second;
		_string = "[" + _first.toString() + " " + _connective.toString() + " " + _second.toString() + "]";
	}
	
	/** Evaluates this sentence using the specified <CODE>PropertyContainer</CODE>.  This class
	 * just sends the request to its two <CODE>Sentence</CODE> objects, then returns the result
	 * of the <CODE>Connective.evaluate</CODE> method.
	 *
	 * @param container the <CODE>PropertyContainer</CODE> used to evaluate this sentence.
	 * @return <CODE>true</CODE> if this sentence evaluates to true using the specified <CODE>PropertyContainer</CODE>; <CODE>false</CODE> otherwise.
	 */	
	public boolean evaluate(PropertyContainer container) {
		return _connective.evaluate(_first.evaluate(container), _second.evaluate(container));
	}
	
	/** Returns the connective used in this composite sentence.
	 *
	 * @return the connective used in this sentence.
	 */	
	public Connective getConnective() {
		return _connective;
	}
	
	/** Returns the first sentence used in this composite sentence.
	 *
	 * @return the first sentence used in this composite sentence.
	 */	
	public Sentence getFirstSentence() {
		return _first;
	}
	
	/** Returns the second sentence used in this composite sentence.
	 *
	 * @return the second sentence used in this composite sentence.
	 */	
	public Sentence getSecondSentence() {
		return _second;
	}
	
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: CompositeSentence.java,v $
 * Revision 1.1  2002/02/15 20:23:57  zcox
 * big time update
 *
 *
 *
 */
