/* $Id: MappingUtility.java,v 1.1 2002/02/15 20:23:57 zcox Exp $
 *
 *
 */

package fcmodeler.mapping;

import java.util.Iterator;
import java.util.Set;

import diva.graph.GraphView;
import diva.graph.model.Graph;
import diva.graph.model.Node;
import diva.graph.model.Edge;
import diva.graph.model.GraphUtilities.EdgeIterator;

import fcmodeler.view.NodeFigure;
import fcmodeler.view.EdgeFigure;
import fcmodeler.attributes.NodeVisualAttributeValue;
import fcmodeler.attributes.EdgeVisualAttributeValue;

/** A utility class containing methods used to update the appearance of node and edge figures based
 * on properties of thier corresponding nodes and edges.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class MappingUtility {
	
	/** This constructor is private so that this class is never instantiated.
	 *
	 */
	private MappingUtility() { }
	
	/** Maps the property values of the nodes and edges in the graph to visual attribute values of the
	 * corresponding node and edge figures in the view using the specified node and edge mapping rules.
	 *
	 * @param graph the graph containing the nodes and edges.
	 * @param nodeMappings the set of node mappings.
	 * @param edgeMappings the set of edge mappings.
	 * @param view the graph view.
	 * @param permanent if set to <CODE>true</CODE>, the permanent visual attributes will be updated.
	 */
	public static void map(Graph graph, Set nodeMappings, Set edgeMappings, GraphView view, boolean permanent) {
		mapNodes(graph.nodes(), nodeMappings, view, permanent);
		
		//not sure how efficient EdgeIterator is, but it's already written...
		mapEdges(new EdgeIterator(graph), edgeMappings, view, permanent);
	}
	
	/** Maps the property values of the edges to visual attributes of the corresponding
	 * edge figures in the view using the specified edge mapping rules.
	 *
	 * @param edges the edges.
	 * @param edgeMappings the set of edge mappings.
	 * @param view the graph view.
	 * @param permanent if set to <CODE>true</CODE>, the permanent visual attriubtes will be update.
	 */
	public static void mapEdges(Iterator edges, Set edgeMappings, GraphView view, boolean permanent) {
		while (edges.hasNext()) {
			Edge edge = (Edge) edges.next();
			for (Iterator mappings = edgeMappings.iterator(); mappings.hasNext();) {
				EdgeMapping mapping = (EdgeMapping) mappings.next();
				if (mapping.getSentence().evaluate(edge)) {
					EdgeVisualAttributeValue value = mapping.getValue();
					EdgeFigure figure = (EdgeFigure) view.getEdgeFigure(edge);
					if (permanent) { value.setPermanentEdgeValue(figure); figure.usePermanents(); } //NOTE: this is a bad work-around for the usePermanents() issue!
					else { value.setEdgeValue(figure); }
				}
			}
		}
		
		view.getGraphicsPane().repaint();
	}
	
	/** Maps the property values of the nodes in the graph to visual attributes of the corresponding
	 * node figures in the view using the specified node mapping rules.
	 *
	 * @param nodes the nodes.
	 * @param nodeMappings the set of node mappings.
	 * @param view the graph view.
	 * @param permanent if set to <CODE>true</CODE>, the permanent visual attriubtes will be update.
	 */
	public static void mapNodes(Iterator nodes, Set nodeMappings, GraphView view, boolean permanent) {
		while (nodes.hasNext()) {
			Node node = (Node) nodes.next();
			for (Iterator mappings = nodeMappings.iterator(); mappings.hasNext();) {
				NodeMapping mapping = (NodeMapping) mappings.next();
				if (mapping.getSentence().evaluate(node)) {
					NodeVisualAttributeValue value = mapping.getValue();
					NodeFigure figure = (NodeFigure) view.getNodeFigure(node);
					if (permanent) { value.setPermanentNodeValue(figure); figure.usePermanents(); } //NOTE: this is a bad work-around for the usePermanents() issue!
					else { value.setNodeValue(figure); }
				}
			}
		}
		
		view.getGraphicsPane().repaint();
	}
	
	/** Calls the <CODE>usePermanents</CODE> method on all node and edge figures in the graph view.
	 *
	 * @param view the graph view with the node and edge figures.
	 */	
	public static void usePermanents(GraphView view) {
		Graph graph = view.getGraphModel().getGraph();
		for (Iterator nodes = graph.nodes(); nodes.hasNext();) {
			Node node = (Node) nodes.next();
			NodeFigure nodeFigure = (NodeFigure) view.getNodeFigure(node);
			nodeFigure.usePermanents();
			for (Iterator edges = node.outEdges(); edges.hasNext();) {
				Edge edge = (Edge) edges.next();
				EdgeFigure edgeFigure = (EdgeFigure) view.getEdgeFigure(edge);
				edgeFigure.usePermanents();
			}
		}
	}
	
}

/*
 * $Log: MappingUtility.java,v $
 * Revision 1.1  2002/02/15 20:23:57  zcox
 * big time update
 *
 *
 *
 */
