/* $Id: ObservableSet.java,v 1.1 2002/02/15 20:20:36 zcox Exp $
 *
 *
 */

package fcmodeler.util;

import java.util.Set;
import java.util.HashSet;
import java.util.Collection;
import java.util.Iterator;
import java.util.Arrays;
import javax.swing.event.EventListenerList;

/** A set that notifies listeners of any modications.  Other classes can implement the
 * <CODE>SetChangeListener</CODE> interface, and register with this class to receive notifiactions
 * of any changes to the wrapped set.  Note that listeners are notified after the set has already been
 * modified.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public class ObservableSet extends Object implements Set {
	private Set _set = null;
	private EventListenerList _listeners = null;
	
	/** Creates a new <CODE>ObservableSet</CODE> using a <CODE>java.util.HashSet</CODE>.
	 *
	 */	
	public ObservableSet() {
		this(new HashSet());
	}
	
	/** Creates a new <CODE>ObservableSet</CODE> that wraps the specified set.
	 *
	 * @param set the set to notify listeners of changes to.
	 */
	public ObservableSet(Set set) {
		if (set == null) { throw new NullPointerException("set was null"); }
		
		_set = set;
		_listeners = new EventListenerList();
	}
	
	/** Adds the specified element to this set if it is not already present and notifies listeners of
	 * the addition.
	 *
	 * @param obj the element to add.
	 * @return <CODE>true</CODE> if the element was added.
	 */	
	public boolean add(Object obj) {
		boolean added = _set.add(obj);
		if (added) { fireElementAdded(obj); }
		
		return added;
	}
	
	/** Adds all of the elements in the specified collection to this set if they're not already present
	 * and notifies listeners of any elements that are added.
	 *
	 * @param collection the collection whose elements are to be added to this set.
	 * @return <CODE>true</CODE> if any of the elements were added.
	 */	
	public boolean addAll(Collection collection) {
		Set copy = new HashSet(_set);
		boolean changed = _set.addAll(collection);
		
		if (changed) {
			copy.retainAll(collection);
			Collection newCollection = new HashSet(collection);
			newCollection.removeAll(copy);
			for (Iterator iterator = newCollection.iterator(); iterator.hasNext();) {
				fireElementAdded(iterator.next());
			}
		}
		
		return changed;
	}
	
	/** Registers the specified listener with this set.
	 *
	 * @param listener the listener to register with this set.
	 */	
	public void addSetChangeListener(SetChangeListener listener) {
		_listeners.add(SetChangeListener.class, listener);
	}
	
	/** Removes all of the elements from this set and notifies listeners of the removals.
	 *
	 */	
	public void clear() {
		Set copy = new HashSet(_set);
		_set.clear();
		for (Iterator iterator = copy.iterator(); iterator.hasNext();) {
			fireElementRemoved(iterator.next());
		}
	}
	
	/** Returns <CODE>true</CODE> if this set contains the specified element.
	 *
	 * @param obj element to test for containment.
	 * @return <CODE>true</CODE> if this set contains the specified element.
	 */	
	public boolean contains(Object obj) {
		return _set.contains(obj);
	}
	
	/** Returns <CODE>true</CODE> if this set contains all of the elements of the specified collection.
	 *
	 * @param collection the collection to test for containment.
	 * @return <CODE>true</CODE> if this set contains all of the elements of the specified collection.
	 */	
	public boolean containsAll(Collection collection) {
		return _set.containsAll(collection);
	}
	
	/** Compares the specified object with this set for equality.
	 *
	 * @param obj the object to test for equality.
	 * @return <CODE>true</CODE> if the specified object is equal to this set.
	 */	
	public boolean equals(Object obj) {
		return _set.equals(obj);
	}
	
	/** Notifies listeners that the specified element has been added to this set.
	 *
	 * @param element the element added to this set.
	 */	
	protected void fireElementAdded(Object element) {
		SetChangeEvent event = new SetChangeEvent(this, element, SetChangeEvent.ELEMENT_ADDED);
		for (Iterator iterator = getSetChangeListeners(); iterator.hasNext();) {
			SetChangeListener listener = (SetChangeListener) iterator.next();
			listener.elementAdded(event);
		}
	}
	
	/** Notifies listeners that the specified element has been removed from this set.
	 *
	 * @param element the element that was removed.
	 */	
	protected void fireElementRemoved(Object element) {
		SetChangeEvent event = new SetChangeEvent(this, element, SetChangeEvent.ELEMENT_REMOVED);
		for (Iterator iterator = getSetChangeListeners(); iterator.hasNext();) {
			SetChangeListener listener = (SetChangeListener) iterator.next();
			listener.elementRemoved(event);
		}
	}
	
	/** Returns the set that this set wraps.
	 *
	 * @return the set that this set wraps.
	 */	
	public Set getSet() {
		return _set;
	}
	
	/** Returns an iterator over all listeners for this set.
	 *
	 * @return an iterator over all listeners for this set.
	 */	
	public Iterator getSetChangeListeners() {
		return Arrays.asList(_listeners.getListeners(SetChangeListener.class)).iterator();
	}
	
	/** Returns the hash code value for this set.
	 *
	 * @return the hash code for this set.
	 */	
	public int hashCode() {
		return _set.hashCode();
	}
	
	/** Returns true if this set contains no elements.
	 *
	 * @return <CODE>true</CODE> if this set contains no elements.
	 */	
	public boolean isEmpty() {
		return _set.isEmpty();
	}
	
	/** Returns an iterator over the elements in this set
	 * @return an iterator over the elements in this set
	 */	
	public Iterator iterator() {
		return _set.iterator();
	}
	
	/** Removes the specified element from this set if it is present and notifies listeners of the removal.
	 *
	 * @param obj the element to remove from this set.
	 * @return <CODE>true</CODE> if the element was removed.
	 */	
	public boolean remove(Object obj) {
		boolean removed = _set.remove(obj);
		if (removed) { fireElementRemoved(obj); }
		
		return removed;
	}
	
	/** Removes from this set all of its elements that are contained in the specified collection and
	 * notifies listeners of the removals.
	 *
	 * @param collection the collection whose elements are to be removed from this set.
	 * @return <CODE>true</CODE> if any of the collection's elements were removed.
	 */	
	public boolean removeAll(Collection collection) {
		Set copy = new HashSet(_set);
		boolean changed = _set.removeAll(collection);
		
		if (changed) {
			copy.retainAll(collection);
			for (Iterator iterator = copy.iterator(); iterator.hasNext();) {
				fireElementRemoved(iterator.next());
			}
		}
		
		return changed;
	}
	
	/** Stops the specified listener from receiving notifications from this set.
	 *
	 * @param listener the listener to stop receiving notifications from this set.
	 */	
	public void removeSetChangeListener(SetChangeListener listener) {
		_listeners.remove(SetChangeListener.class, listener);
	}
	
	/** Retains only the elements in this set that are contained in the specified collection and
	 * notifies listeners of the removals.
	 *
	 * @param collection collection that defines which elements this set will retain.
	 * @return <CODE>true</CODE> if any elements were removed from this set.
	 */	
	public boolean retainAll(Collection collection) {
		Set copy = new HashSet(_set);
		Set copy2 = new HashSet(_set);
		boolean changed = _set.retainAll(collection);
		
		if (changed) {
			copy.retainAll(collection);
			copy2.removeAll(copy);
			for (Iterator iterator = copy2.iterator(); iterator.hasNext();) {
				fireElementRemoved(iterator.next());
			}
		}
		
		return changed;
	}
	
	/** Returns the number of elements in this set.
	 *
	 * @return the number of elements in this set.
	 */	
	public int size() {
		return _set.size();
	}
	
	/** Returns an array containing all of the elements in this set.
	 *
	 * @return an array containing all of the elements in this set.
	 */	
	public Object[] toArray() {
		return _set.toArray();
	}
	
	/** Returns an array containing all of the elements in this set whose runtime type is that of
	 * the specified array.
	 *
	 * @param obj the array into which the elements of this set are to be stored, if it is big enough;
	 * otherwise, a new array of the same runtime type is allocated for this purpose.
	 * @return an array containing the elements of this set.
	 */	
	public Object[] toArray(Object[] obj) {
		return _set.toArray(obj);
	}
	
	/** Returns a string representation of this set.
	 *
	 * @return a string representation of this set.
	 */	
	public String toString() {
		return _set.toString();
	}
	
}

/* 
 * $Log: ObservableSet.java,v $
 * Revision 1.1  2002/02/15 20:20:36  zcox
 * big time update
 *
 *
 *
 */
