/* $Id: QuotedStringIterator.java,v 1.1 2002/02/15 20:20:36 zcox Exp $
 *
 *
 */

package fcmodeler.util;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;
import java.util.StringTokenizer;

/** <p>An iterator over the quoted words in a string.  This iterator returns the strings surrounded by
 * double-quotes in the specified string.  For example, if the string is</p>
 *
 * <PRE>"this" "is" "a" "quoted" "string"</PRE>
 *
 * <p>then the strings returned by the <CODE>next</CODE> method are</p>
 *
 * <PRE>this
 * is
 * a
 * quoted
 * string</PRE>
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class QuotedStringIterator extends Object implements Iterator {
	private final Iterator _iterator;
	
	/** Creates a new <CODE>QuotedStringIterator</CODE> using the specified string.
	 *
	 * @param quotedString the string containing the quoted strings to iterate over.
	 */
	public QuotedStringIterator(String quotedString) {
		if (quotedString == null) { throw new NullPointerException("quotedString was null"); }
		
		StringTokenizer st = new StringTokenizer(quotedString, "\"");
		int count = st.countTokens()+1;
		if (count%2 != 0) { throw new IllegalArgumentException("string must have an even number of quotes"); }
		
		List words = new ArrayList(count/2);
		while (st.hasMoreTokens()) {
			String word = st.nextToken();
			words.add(word);
			if (st.hasMoreTokens()) { st.nextToken(); }
		}
		_iterator = words.iterator();
	}
	
	/** Returns <CODE>true</CODE> if the iteration has more elements.
	 *
	 * @return <CODE>true</CODE> if the iteration has more elements.
	 */	
	public boolean hasNext() {
		return _iterator.hasNext();
	}
	
	/** Returns the next element in the iteration.
	 *
	 * @return the next element in the iteration.
	 */	
	public Object next() {
		return _iterator.next();
	}
	
	/** Throws an <CODE>UnsupportedOperationException</CODE> since this is an unsupported operation.
	 *
	 */	
	public void remove() {
		throw new UnsupportedOperationException("remove operation is not supported");
	}
	
}

/* 
 * $Log: QuotedStringIterator.java,v $
 * Revision 1.1  2002/02/15 20:20:36  zcox
 * big time update
 *
 *
 *
 */
