/* $Id: SetChangeEvent.java,v 1.1 2002/02/15 20:20:36 zcox Exp $
 *
 *
 */

package fcmodeler.util;

import java.util.EventObject;
import java.util.Set;

/** An event representing a change in a set.  Instances of this class represent either
 * an element being added to a set or an element being removed from a set.  The exact type can be
 * accessed through the <CODE>getChangeType</CODE> method.  The set that was modified can be
 * accessed through the <CODE>getSource</CODE> method, and the element that was added to or removed
 * from the set can be accessed through the <CODE>getElement</CODE> method.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.1 $
 * @since JDK1.3
 */

public final class SetChangeEvent extends EventObject {
	/** The constant for an element addition event.
	 *
	 */	
	public static final int ELEMENT_ADDED = 1;
	/** The constant for an element removal event.
	 */	
	public static final int ELEMENT_REMOVED = 2;
	private final Object _element;
	private final int _type;
	private final String _string;
	
	/** Creates a new <CODE>SetChangeEvent</CODE>.  The specified element was either added to or removed
	 * from the source set, as idicated by the specified type.
	 *
	 * @param source the set that was modified.
	 * @param element the element that was added to or removed from the source set.
	 * @param type either ELEMENT_ADDED or ELEMENT_REMOVED.
	 */
	public SetChangeEvent(Set source, Object element, int type) {
		super(source);
		
		if (source == null) { throw new NullPointerException("source was null"); }
		if (element == null) { throw new NullPointerException("element was null"); }
		if (!(type == ELEMENT_ADDED || type == ELEMENT_REMOVED)) { throw new IllegalArgumentException("type not valid: " + type); }
		
		_element = element;
		_type = type;
		
		if (_type == ELEMENT_ADDED) {
			_string = _element.toString() + " added to " + getSource().toString();
		}
		else {
			_string = _element.toString() + " removed from " + getSource().toString();
		}
	}
	
	/** Returns the element that was either added to or removed from the set.
	 *
	 * @return the element that was either added to or removed from the set.
	 */	
	public Object getElement() {
		return _element;
	}
	
	/** Returns the type of change.  The returned <CODE>int</CODE> is equal to either
	 * <CODE>ELEMENT_ADDED</CODE> or <CODE>ELEMENT_REMOVED</CODE>.
	 *
	 * @return the type of change.
	 */	
	public int getChangeType() {
		return _type;
	}
	
	/** Returns the string representation of this set change event.
	 *
	 * @return the string representation of this set change event.
	 */	
	public String toString() {
		return _string;
	}
	
}

/* 
 * $Log: SetChangeEvent.java,v $
 * Revision 1.1  2002/02/15 20:20:36  zcox
 * big time update
 *
 *
 *
 */
