/* $Id: BezierEdgeFigure.java,v 1.3 2002/01/23 21:23:20 zcox Exp $
 *
 *
 */

package fcmodeler.view;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import diva.canvas.Site;
import diva.canvas.TransformContext;
import diva.canvas.CanvasUtilities;

import fcmodeler.view.figures.LineShape;

/** An <CODE>EdgeFigure</CODE> implementation providing a Bezier curve line type.  Clients should
 * first create an instance of <CODE>BezierEdgeFigure</CODE> and then supply the Bezier control
 * points via the <CODE>setControlPoints</CODE> method.  This class uses the <CODE>LineShape</CODE>
 * wrapper class to provide an appropriate line-based <CODE>Shape</CODE> object.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.3 $
 * @since JDK1.3
 */

public class BezierEdgeFigure extends AbstractEdgeFigure {
	private List _points = null;
	
	/** <p>Creates a new <CODE>BezierEdgeFigure</CODE> using the given sites.  The control points are initialized to:</p>
	 * <PRE>pt1 = tail point
	 * pt2 = (0,0)
	 * pt3 = (0,0)
	 * pt4 = head point</PRE>
	 *
	 * @param tail the site on the tail node to connect to.
	 * @param head the site on the head node to connect to.
	 */
	public BezierEdgeFigure(Site tail, Site head) {
		super(tail, head);
		
		_points = new ArrayList(4);
		_points.add(new Point2D.Double(tail.getX(), tail.getY()));
		_points.add(new Point2D.Double(0.0d, 0.0d));
		_points.add(new Point2D.Double(0.0d, 0.0d));
		_points.add(new Point2D.Double(head.getX(), head.getY()));
		createShape();
	}
	
	private void createShape() {
		Point2D pt1 = null, pt2 = null, pt3 = null;
		GeneralPath line = new GeneralPath();
		Iterator cps = null;
		
		cps = _points.iterator();
		pt1 = (Point2D) cps.next();
		line.moveTo((float) pt1.getX(), (float) pt1.getY());
		while (cps.hasNext()) {
			pt1 = (Point2D) cps.next();
			pt2 = (Point2D) cps.next();
			pt3 = (Point2D) cps.next();
			line.curveTo((float) pt1.getX(), (float) pt1.getY(), 
					(float) pt2.getX(), (float) pt2.getY(), 
					(float) pt3.getX(), (float) pt3.getY());
		}
		
		getPaintedShape().shape = new LineShape(line);
	}
	
	/** Returns the control points used for this <CODE>BezierEdgeFigure</CODE>.  The returned <CODE>List</CODE>
	 * is a copy of the original used in this object.
	 *
	 * @return the control points used for this <CODE>BezierEdgeFigure</CODE>.
	 */	
	public List getControlPoints() {
		return new ArrayList(_points);
	}
	
	/** Routes this edge between the tail and head nodes.
	 *
	 */	
	public void route() {
		TransformContext currentContext = null;
		Site headSite = null;
		Site tailSite = null;
		Point2D tailCenter = null, headCenter = null;
		Point2D headPt = null, tailPt = null;
		Point2D second = null, secondToLast = null;
		double x1 = 0;
		double y1 = 0;
		double x2 = 0;
		double y2 = 0;
		double headAngle = 0;
		double tailAngle = 0;
		
		//why do we call repaint() first???
		repaint();
		
		//get the current transform context
		currentContext = getTransformContext();
		
		//get the tail & head sites
		headSite = getHeadSite();
		tailSite = getTailSite();
		
		// Get the transformed head and tail points. Sometimes
		// people will call this before the connector is added
		// to a container, so deal with it
		if (currentContext != null) {
			tailPt = tailSite.getPoint(currentContext);
			headPt = headSite.getPoint(currentContext);
		} else {
			tailPt = tailSite.getPoint();
			headPt = headSite.getPoint();
		}
		
		//if there are figures attached, find their center points
		//if not just use the Site point
		if (tailSite.getFigure() != null) {
			tailCenter = CanvasUtilities.getCenterPoint(tailSite.getFigure());
		} else {
			tailCenter = tailPt;
		}
		if (headSite.getFigure() != null) {
			headCenter = CanvasUtilities.getCenterPoint(headSite.getFigure());
		} else {
			headCenter = headPt;
		}
		
		// Figure out the normal at the line ends. NOTE that we
		// use upside-down geometry, so that pi/2 points SOUTH.
		//normal is the direction at which the line should connect to the site
		//so [x1,y1] is a vector from line beginning to tail figure center
		//and [x2,y2] is a vector from line end to head figure center
		//by line beginning/end, i mean the second/second-to-last control point
		second = (Point2D) _points.get(1);
		secondToLast = (Point2D) _points.get(_points.size()-2);
		x1 = tailCenter.getX() - second.getX();
		y1 = tailCenter.getY() - second.getY();
		x2 = headCenter.getX() - secondToLast.getX();
		y2 = headCenter.getY() - secondToLast.getY();
		
		//Math.atan2() converts cartestian to polar coordinates & returns phase angle
		tailAngle = Math.atan2(y1, x1);
		headAngle = Math.atan2(y2, x2);
		
		// Tell the sites to adjust their positions
		tailSite.setNormal(tailAngle - Math.PI);
		headSite.setNormal(headAngle - Math.PI);
		
		// Recompute the head and tail points
		//these are the coordinates of the new sites on the head & tail figures
		//after the above calculations
		if (currentContext != null) {
			tailPt = tailSite.getPoint(currentContext);
			headPt = headSite.getPoint(currentContext);
		} else {
			tailPt = tailSite.getPoint();
			headPt = headSite.getPoint();
		}
		
		//adjust for decorations on the ends
		if (getTailEnd() != null) {
			//figure out the normal again
			x1 = tailPt.getX() - second.getX();
			y1 = tailPt.getY() - second.getY();
			tailAngle = Math.atan2(y1, x1);
			getTailEnd().setNormal(tailAngle+Math.PI);
			getTailEnd().setOrigin(tailPt.getX(), tailPt.getY());
			getTailEnd().getConnection(tailPt);
		}
		if (getHeadEnd() != null) {
			//figure out the normal again
			x2 = headPt.getX() - secondToLast.getX();
			y2 = headPt.getY() - secondToLast.getY();
			headAngle = Math.atan2(y2, x2);
			getHeadEnd().setNormal(headAngle+Math.PI);
			getHeadEnd().setOrigin(headPt.getX(), headPt.getY());
			getHeadEnd().getConnection(headPt);
		}
		
		//update the edge shape
		_points.set(0, tailPt);
		_points.set(_points.size()-1, headPt);
		createShape();
		
		//repaint the figure
		repaint();
	}
	
	/** Sets the control points of this edge to the points in the specified <CODE>List</CODE>.  The
	 * <CODE>List</CODE> should contain 3n+1 <CODE>Point2D</CODE> objects, where n is the number of
	 * Bezier curve sections.
	 *
	 * @param points the new control points for this <CODE>BezierEdgeFigure</CODE>.
	 * @throws IllegalArgumentException if <PRE>points.size() < 4</PRE> or <PRE>points.size() % 3 != 1</PRE>.
	 */	
	public void setControlPoints(List points) {
		List temp = new ArrayList(points);
		
		if (temp.size() < 4 || temp.size() % 3 != 1) { throw new IllegalArgumentException("invalid number of control points: " + temp.size()); }
		for (Iterator iterator = temp.iterator(); iterator.hasNext();) {
			if (!(iterator.next() instanceof Point2D)) { throw new IllegalArgumentException("all elements of List must be of type java.awt.geom.Point2D"); }
		}
		
		_points = temp;
		createShape();
	}
	
	/** <p>Translates this edge by the specified x and y coordinates.</p>
	 *
	 * <p>Currently just throws an <CODE>UnsupportedOperationException</CODE>, as this method
	 * never even seems to get called and edge translation is not yet well-defined.</p>
	 *
	 * @param dx the x-distance to translate.
	 * @param dy the y-distance to translate.
	 */	
	public void translate(double dx, double dy) {
		throw new UnsupportedOperationException("BezierEdgeFigure.translate() not supported");
	}
	
}

/* 
 * $Log: BezierEdgeFigure.java,v $
 * Revision 1.3  2002/01/23 21:23:20  zcox
 * added javadoc comments
 *
 * Revision 1.2  2002/01/22 22:59:30  zcox
 * added cvs tags
 *
 * Revision 1.1  2002/01/22 22:53:38  zcox
 * modifications to EdgeFigure classes
 *
 *
 *
 */
