/* $Id: NodeFigure.java,v 1.3 2002/02/15 20:56:38 zcox Exp $
 *
 *
 */

package fcmodeler.view;

import java.awt.Shape;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;

import diva.canvas.AbstractFigure;
import diva.canvas.toolbox.LabelFigure;
import diva.canvas.toolbox.BasicFigure;
import diva.canvas.CanvasUtilities;

import fcmodeler.view.figures.NodeShape;
import fcmodeler.view.figures.RectangleNodeShape;

/** A <CODE>Figure</CODE> representing a node.  <CODE>NodeFigure</CODE> consists of
 * an arbitrary shape with a string label.  The shape is backed by a <CODE>BasicFigure</CODE>
 * and the label is backed by a <CODE>LabelFigure</CODE>.
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.3 $
 * @since JDK1.3
 */

public class NodeFigure extends AbstractFigure implements PermanentFigure {
	private LabelFigure _label = null;
	private BasicFigure _figure = null;
	private BasicFigure _permanent = null;
	private NodeShape _shape = null;
	private NodeShape _permanentShape = null;
	
	/** <p>Creates a new <CODE>NodeFigure</CODE> using an empty label.  Equivalent to calling </p>
	 *
	 * <p><PRE>new NodeFigure(new LabelFigure(" "));</PRE></p>
	 */
	public NodeFigure() {
		this(new LabelFigure(" "));
	}
	
	/** Creates a new <CODE>NodeFigure</CODE> using the specified <CODE>LabelFigure</CODE>.
	 *
	 * @param label the label to use for this <CODE>NodeFigure</CODE>.
	 */	
	public NodeFigure(LabelFigure label) {
		_label = label;
		
		_shape = new RectangleNodeShape();
		_shape.enclose(label.getBounds());
		_figure = new BasicFigure(_shape.getShape());
		
		_permanentShape = new RectangleNodeShape();
		_permanentShape.enclose(label.getBounds());
		_permanent = new BasicFigure(_permanentShape.getShape());
		
		_figure.setShape(_shape.getShape());
		_permanent.setShape(_permanentShape.getShape());
		
		_figure.setFillPaint(Color.white);
		_permanent.setFillPaint(Color.white);
	}
	
	/** Returns the <CODE>BasicFigure</CODE> used for rendering this node figure.
	 *
	 * @return the <CODE>BasicFigure</CODE> used for rendering this node figure.
	 */	
	public BasicFigure getFigure() {
		return _figure;
	}
	
	/** Returns the <CODE>LabelFigure</CODE> used for the label of this node.
	 *
	 * @return the <CODE>LabelFigure</CODE> used for the label of this node.
	 */	
	public LabelFigure getLabelFigure() {
		return _label;
	}
	
	/** Returns the <CODE>NodeShape</CODE> used to determine the shape of this node.
	 *
	 * @return the <CODE>NodeShape</CODE> used to determine the shape of this node.
	 */	
	public NodeShape getNodeShape() {
		return _shape;
	}
	
	/** Returns the permanent <CODE>BasicFigure</CODE> for this node.
	 *
	 * @return the permanent <CODE>BasicFigure</CODE> for this node.
	 */	
	public BasicFigure getPermanentFigure() {
		return _permanent;
	}
	
	/** Returns the permanent <CODE>NodeShape</CODE> for this node.
	 * @return the permanent <CODE>NodeShape</CODE> for this node.
	 */	
	public NodeShape getPermanentNodeShape() {
		return _permanentShape;
	}
	
	/** Returns the outline <CODE>Shape</CODE> of this node.
	 *
	 * @return the outline <CODE>Shape</CODE> of this node.
	 */	
	public Shape getShape() {
		return _shape.getShape();
	}
	
	/** Paints this node figure.  The <CODE>BasicFigure</CODE> is painted first, followed
	 * by the <CODE>LabelFigure</CODE>.
	 *
	 * @param g the graphics context to paint to.
	 */	
	public void paint(Graphics2D g) {
		_figure.paint(g);
		
		_label.paint(g);
	}
	
	/** Sets the label for this node to the specified <CODE>String</CODE>.
	 *
	 * @param label the new label for this node.
	 */	
	public void setLabel(String label) {
		_label.setString(label);
		_shape.enclose(_label.getBounds());
		_permanentShape.enclose(_label.getBounds());
		_figure.setShape(_shape.getShape());
	}
	
	/** Sets the <CODE>NodeShape</CODE> for this node to the specified <CODE>NodeShape</CODE>.
	 *
	 * @param nodeShape the new <CODE>NodeShape</CODE> for this node.
	 */	
	public void setNodeShape(NodeShape nodeShape) {
		_shape = nodeShape;
		_shape.enclose(_label.getBounds());
		_figure.setShape(_shape.getShape());
	}
	
	/** Sets the permanent <CODE>NodeShape</CODE> for this node to the specified
	 * <CODE>NodeShape</CODE>.
	 *
	 * @param nodeShape the new permanent <CODE>NodeShape</CODE> for this node.
	 */	
	public void setPermanentNodeShape(NodeShape nodeShape) {
		_permanentShape = nodeShape;
		_permanentShape.enclose(_label.getBounds());
		_figure.setShape(_permanentShape.getShape());
	}
	
	/** Transforms this node using the specifed <CODE>AffineTransform</CODE>.
	 *
	 * @param affineTransform the transform to apply to this node.
	 */	
	public void transform(AffineTransform affineTransform) {
		throw new UnsupportedOperationException("NodeFigure.transform() not suported");
	}
	
	/** Translates this node by the specified x and y distances.
	 *
	 * @param x the x-distance to translate.
	 * @param y the y-distance to translate.
	 */	
	public void translate(double x, double y) {
		repaint();
		
		_label.translate(x, y);
		
		_shape.translate(x, y);
		_permanentShape.translate(x, y);
		_figure.setShape(_shape.getShape());
		
		repaint();
	}
	
	/** Sets the visual attributes for this node to those stored in the permanent
	 * <CODE>BasicFigure</CODE>.  This can be used to revert back to a permanent visual
	 * state after modifying the appearance of this node.
	 *
	 */	
	public void usePermanents() {
		_figure.setComposite(_permanent.getComposite());
		_figure.setFillPaint(_permanent.getFillPaint());
		_figure.setLineWidth(_permanent.getLineWidth());
		try {
			_shape = (NodeShape) _permanentShape.clone();
		} catch (CloneNotSupportedException e) {
			throw new UnsupportedOperationException(_shape + " does not support the clone method - BUT IT SHOULD!!!");
		}
		_figure.setShape(_shape.getShape());
		_figure.setStroke(_permanent.getStroke());
		_figure.setStrokePaint(_permanent.getStrokePaint());
	}
}

/* 
 * $Log: NodeFigure.java,v $
 * Revision 1.3  2002/02/15 20:56:38  zcox
 * big time update
 *
 * Revision 1.2  2002/01/18 19:52:30  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.1  2002/01/15 20:46:33  zcox
 * name changes & addition of some view-related classes
 *
 * 
 */
