package fcmodeler.view.figures;

import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;

/** A <CODE>NodeShape</CODE> implementation providing a diamond node shape.
 * <CODE>DiamondNodeShape</CODE> uses a <CODE>GeneralPath</CODE> as its
 * <CODE>Shape</CODE> object.
 *
 * @author xein <xein@iastate.edu>
 * @version
 * @since JDK1.3
 */
public class DiamondNodeShape extends Object implements NodeShape {

    private final static float _space = 10.0f;
    private final static String _string = "diamond";

    private GeneralPath _shape = new GeneralPath();


    /**
     *  Creates a new DiamondNodeShape inside a rectangle (0, 0, 4, 4)
     */
    public DiamondNodeShape() {
        _shape.moveTo(2.0f, 0.0f);
        _shape.lineTo(4.0f, 2.0f);
        _shape.lineTo(2.0f, 4.0f);
        _shape.lineTo(0.0f, 2.0f);
        _shape.closePath();
    }


    /**
     *  Creates a new DiamondNodeShape inside a rectangle (x, y, w, h)
     *
     *@param  x  x-coordinate of this <CODE>DiamondNodeShape</CODE>.
     *@param  y  y-coordinate of this <CODE>DiamondNodeShape</CODE>.
     *@param  w  width of this <CODE>DiamondNodeShape</CODE>.
     *@param  h  height of this <CODE>DiamondNodeShape</CODE>.
     */
    public DiamondNodeShape(double x, double y, double w, double h) {
        _shape.moveTo((float)(x + w / 2), (float)y);
        _shape.lineTo((float)(x + w), (float)(y + h / 2));
        _shape.lineTo((float)(x + w / 2), (float)(y + h));
        _shape.lineTo((float)x, (float)(y + h / 2));
        _shape.closePath();
    }


    /** Returns the <CODE>Shape</CODE> object represented by this <CODE>DiamondNodeShape</CODE>.
	   *
	   * @return the <CODE>Shape</CODE> object represented by this <CODE>DiamondNodeShape</CODE>.
	   */
    public Shape getShape() {
        return _shape;
    }


    /** Creates and returns a copy of this <CODE>DiamondNodeShape</CODE>.  The returned
     * <CODE>Object</CODE> is guaranteed to be of type <CODE>DiamondNodeShape</CODE>
     * and is also guaranteed to have copies of any mutable fields of this object.
     *
     * @throws CloneNotSupportedException should never be thrown, as this class extends <CODE>Object</CODE> and implements
     * <CODE>Cloneable</CODE> by implementing <CODE>NodeShape</CODE>.
     * @return a clone of this <CODE>DiamondNodeShape</CODE> object.
     */
    public Object clone() throws CloneNotSupportedException {
        //create the clone
        DiamondNodeShape clone = (DiamondNodeShape) super.clone();

        //create new copies of any mutable fields
        clone._shape = new GeneralPath(_shape);

        return clone;
    }


    /** Translates this <CODE>DiamondNodeShape</CODE> by the specified x and y values.  Note that calling
     * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
     * to <CODE>getShape</CODE> may be necessary after calling this method.
     *
     * @param dx the amount to translate in the x direction.
     * @param dy the amount to translate in the y direction.
     */
    public void translate(double dx, double dy) {
        _shape.transform(AffineTransform.getTranslateInstance(dx, dy));
    }


    /** <p>Transforms this <CODE>DiamondNodeShape</CODE> using the specified <CODE>AffineTransform</CODE>.
     * Note that calling this method may create a new <CODE>Shape</CODE> instance.  Therefore,
     * a new call to <CODE>getShape</CODE> may be necessary after calling this method.</p>
     *
     * <p>This method currently throws an <CODE>UnsupportedOperationException</CODE>
     * until a further research is done on whether <CODE>AffineTransform</CODE> will work with
     * <CODE>GeneralPath</CODE>.</p>
     *
     * @param transform the <CODE>AffineTransform</CODE> used to transform this <CODE>DiamondNodeShape</CODE> object.
     */
    public void transform(AffineTransform at) {
        //_shape = transform.createTransformedShape(_shape);
        throw new UnsupportedOperationException("transform not supported yet");
    }


    /** Ensures that this <CODE>DiamondNodeShape</CODE> completely encloses the specified
     * rectangle.
     *
     * @param rectangle the rectangle to enclose.
     */
    public void enclose(Rectangle2D rectangle) {
        float x = 0.0f;
        float y = 0.0f;
        float w = 0.0f;
        float h = 0.0f;

        x = (float) rectangle.getX() - _space;
        y = (float) rectangle.getY() - _space;
        w = (float) rectangle.getWidth() + 2 * _space;
        h = (float) rectangle.getHeight() + 2 * _space;

        _shape.reset();
        _shape.moveTo(x + w / 2, y);
        _shape.lineTo(x + w, y + h / 2);
        _shape.lineTo(x + w / 2, y + h);
        _shape.lineTo(x, y + h / 2);
        _shape.closePath();
    }


    /** Positions this <CODE>DiamondNodeShape</CODE> at the specified coordinates.  Note that calling
     * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
     * to <CODE>getShape</CODE> may be necessary after calling this method.
     *
     * @param x the x-coordinate of the new position.
     * @param y the y-coordinate of the new position.
     */
    public void position(double x, double y) {
        Rectangle2D rec = _shape.getBounds2D();
        translate(x - rec.getX(), y - rec.getY());
    }


    /**
     *  The String representation of this <CODE>DiamondNodeShape</CODE> object
     *
     *@return    the String representation
     */
    public String toString() {
        return _string;
    }
}
