/* $Id: EllipseNodeShape.java,v 1.4 2002/02/15 20:21:18 zcox Exp $
 *
 *
 * Created on October 20, 2000, 12:53 AM
 */

package fcmodeler.view.figures;

import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;

/** A <CODE>NodeShape</CODE> implementation providing an ellipse node shape.
 * <CODE>EllipseNodeShape</CODE> uses an <CODE>Ellipse2D.Double</CODE> as its
 * <CODE>Shape</CODE> object.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.4 $
 * @since JDK1.3
 */

public class EllipseNodeShape extends Object implements NodeShape {
	private Ellipse2D.Double _shape = null;
	private double _space = 12.0d;
	private static final String _string = "ellipse";

	/** Creates a new EllipseNodeShape with coordinates (0,0), zero width, and zero height.
	 *
	 */
	public EllipseNodeShape() {
		_shape = new Ellipse2D.Double();
	}
	
	/** Creates a new EllipseNodeShape with the specified coordinates, width, and height.
	 *
	 * @param x the x-coordinate of this <CODE>EllipseNodeShape</CODE>.
	 * @param y the y-coordinate of this <CODE>EllipseNodeShape</CODE>.
	 * @param w the width of this <CODE>EllipseNodeShape</CODE>.
	 * @param h the height of this <CODE>EllipseNodeShape</CODE>.
	 */	
	public EllipseNodeShape(double x, double y, double w, double h) {
		_shape = new Ellipse2D.Double(x, y, w, h);
	}
	
	/** Creates and returns a copy of this <CODE>EllipseNodeShape</CODE>.  The returned
	 * <CODE>Object</CODE> is guaranteed to be of type <CODE>EllipseNodeShape</CODE>
	 * and is also guaranteed to have copies of any mutable fields of this object.
	 *
	 * @throws CloneNotSupportedException should never be thrown, as this class extends <CODE>Object</CODE> and implements
	 * <CODE>Cloneable</CODE> by implementing <CODE>NodeShape</CODE>.
	 * @return a clone of this <CODE>EllipseNodeShape</CODE>.
	 */	
	public Object clone() throws CloneNotSupportedException {
		//create the clone
		EllipseNodeShape clone = (EllipseNodeShape) super.clone();
		
		//create new copies of any mutable fields
		clone._shape = new Ellipse2D.Double(_shape.getX(), _shape.getY(), _shape.getWidth(), _shape.getHeight());
		
		return clone;
	}
	
	/** Ensures that this <CODE>EllipseNodeShape</CODE> completely encloses the specified
	 * rectangle.
	 *
	 * @param rectangle the rectangle to enclose.
	 */
	public void enclose(Rectangle2D rectangle) {
		_shape.setFrame(rectangle.getX()-_space, rectangle.getY()-_space,
			rectangle.getWidth()+2*_space, rectangle.getHeight()+2*_space);
	}
	
	/** Returns the <CODE>Shape</CODE> object represented by this <CODE>EllipseNodeShape</CODE>.
	 *
	 * @return the <CODE>Shape</CODE> object represented by this <CODE>EllipseNodeShape</CODE>.
	 */
	public Shape getShape() {
		return _shape;
	}
	
	/** Positions this <CODE>EllipseNodeShape</CODE> at the specified coordinates.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param x the x-coordinate of the new position.
	 * @param y the y-coordinate of the new position.
	 */
	public void position(double x, double y) {
		_shape.setFrame(x, y, _shape.getWidth(), _shape.getHeight());
	}
	
	public String toString() {
		return _string;
	}
	
	/** Transforms this <CODE>EllipseNodeShape</CODE> using the specified <CODE>AffineTransform</CODE>.
	 * Note that calling this method may create a new <CODE>Shape</CODE> instance.  Therefore,
	 * a new call to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param transform the <CODE>AffineTransform</CODE> used to transform the <CODE>Shape</CODE> object.
	 */
	public void transform(AffineTransform transform) {
		throw new UnsupportedOperationException("transform not supported yet");
	}
	
	/** Translates this <CODE>NodeShEllipseNodeShapeape</CODE> by the specified x and y values.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param dx the amount to translate the x direction.
	 * @param dy the amount to translate in the y direction.
	 */
	public void translate(double dx, double dy) {
		_shape.setFrame(_shape.getX()+dx, _shape.getY()+dy, _shape.getWidth(), _shape.getHeight());
	}
	
}

/* 
 * $Log: EllipseNodeShape.java,v $
 * Revision 1.4  2002/02/15 20:21:18  zcox
 * big time update
 *
 * Revision 1.3  2002/01/18 19:52:52  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.2  2002/01/15 20:40:06  zcox
 * some changes
 *
 * Revision 1.1  2001/05/22 16:45:02  zcox
 * code clean-up
 *
 * Revision 1.2  2001/04/05 03:01:28  zcox
 * made FCMNodeShape a real flyweight
 *
 * Revision 1.1  2001/04/04 01:33:45  zcox
 * initial commit of some fcmodeler.view.figures classes
 *
 *
 *
 */
