package fcmodeler.view.figures;

import diva.util.java2d.Polygon2D;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;

/**
 *  This is a class to draw a filled rectangle shape at the end of a connector.
 *  The way the class works is almost similar with <CODE>Arrowhead</CODE>. The
 *  only different part is in its reshape() method. The _polygonValid variable
 *  is removed here because it does not serve any purpose
 *
 *@author     xein <xein@iastate.edu>
 *@created    March 22, 2002
 */
public class FilledRectangleHead extends Object implements ConnectorEndExt {

    // the filled rectangle length
    private final static double _length = 16.0d;

    // the x and y coordinate
    private double _originX;
    private double _originY;

    // the normal to the line
    private double _normal;

    // a flag that says to slip the direction in which the arrowhead is drawn.
    private boolean _flipped = false;

    // the shape to draw
    private Polygon2D _polygon = null;

    // the string representation for this ConnectorEnd
    private String _string = "filled rectangle head";

    /** Creates and returns a copy of this <CODE>FilledRectangleHead</CODE>.  The returned
     * <CODE>Object</CODE> is guaranteed to be of type <CODE>FilledRectangleHead</CODE>
     * and is also guaranteed to have copies of any mutable fields of this object.
     *
     * @throws CloneNotSupportedException should never be thrown, as this class extends <CODE>Object</CODE> and implements
     * <CODE>Cloneable</CODE> by implementing <CODE>ConnectorEndExt</CODE>.
     * @return a clone of this <CODE>FilledRectangleHead</CODE> object.
     */
    public Object clone() throws CloneNotSupportedException {
        FilledRectangleHead clone = (FilledRectangleHead)super.clone();
        clone = new FilledRectangleHead();
        return clone;
    }

    /**
     *  Creates the <CODE>FilledRectangleHead</CODE> at position (0,0) with
     *  the normal angle (n = 0)
     *
     */
    public FilledRectangleHead() {
        this(0.0d, 0.0d, 0.0d);
    }


    /**
     *  Creates the <CODE>FilledRectangleHead</CODE> at position (x,y) with
     *  the angle at normal rad
     *
     *@param  x       x-coordinate of this <CODE>FilledRectangleHead</CODE> object
     *@param  y       y-coordinate of this <CODE>FilledRectangleHead</CODE> object
     *@param  normal  the angle in radian
     */
    public FilledRectangleHead(double x, double y, double normal) {
        _originX = x;
        _originY = y;
        _normal = normal;
        reshape();
    }


    /**
     *  Sets the normal attribute of the <CODE>FilledRectangleHead</CODE> object
     *
     *@param  angle  the new angle in radian
     */
    public void setNormal(double angle) {
        _normal = angle;
    }


    /**
     *  Sets the flipped attribute of the FilledRectangleHead object
     *
     *@param  flag  The new flipped value
     */
    public void setFlipped(boolean flag) {
        _flipped = flag;
    }


    /**
     *  Sets the origin attribute of the FilledRectangleHead object
     *
     *@param  x  The new x-coordinate
     *@param  y  The new y-coordinate
     */
    public void setOrigin(double x, double y) {
        translate(x - _originX, y - _originY);
    }


    /** Get the bounding box of the shape used to draw
     * this connector end.
     *
     *@return    The bounds value
     */
    public Rectangle2D getBounds() {
        return _polygon.getBounds2D();
    }


    /**
     *  Gets the connection point into the given point
     *
     *@param  p  the point of the connector end
     */
    public void getConnection(Point2D p) {
        reshape();

        // Set the point
        p.setLocation(_originX, _originY);
        AffineTransform at = new AffineTransform();
        at.setToRotation(_normal, _originX, _originY);
        at.transform(p, p);
    }


    /**
     *  Gets the origin into the given point
     *
     *@param  p  point of origin
     */
    public void getOrigin(Point2D p) {
        p.setLocation(_originX, _originY);
    }


    /**
     *  Recalculate the shape of the decoration
     */
    public void reshape() {
        AffineTransform at = new AffineTransform();
        at.setToRotation(_normal, _originX, _originY);

        double l = _length * 0.5;
        double w = _length * 0.2;

        if (_flipped) {
            l = -l;
            at.translate(_length, 0.0);
        }

        _polygon = new Polygon2D.Double();
        _polygon.moveTo(_originX, _originY);
        _polygon.lineTo(_originX, _originY + l);
        _polygon.lineTo(_originX + w, _originY + l);
        _polygon.lineTo(_originX + w, _originY - l);
        _polygon.lineTo(_originX, _originY - l);
        _polygon.closePath();
        _polygon.transform(at);
    }


    /** Paint the filled-rectangle-head.  This method assumes that
     * the graphics context is already set up with the correct
     * paint and stroke.
     *
     * @param g2D the reference to a <CODE>Graphics2D</CODE> object
     */
    public void paint(Graphics2D g2D) {
        reshape();
        g2D.fill(_polygon);
    }


    /**
     *  Translate the origin by the given x and y coordinates
     *
     *@param  x  x-coordinate of this <CODE>FilledRectangleHead</CODE> object
     *@param  y  y-coordinate of this <CODE>FilledRectangleHead</CODE> object
     */
    public void translate(double x, double y) {
        _originX += x;
        _originY += y;

        _polygon.translate(x, y);
    }

    /**
     * Get the String representation for <CODE>FilledRectangleHead</CODE>
     *
     * @return the String representation
     */
    public String toString() {
      return _string;
    }
}
