/* $Id: LineShape.java,v 1.2 2002/01/22 23:01:21 zcox Exp $
 *
 *
 */

package fcmodeler.view.figures;

import java.awt.Shape;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;

/** <p>A <CODE>Shape</CODE> implementation that contains methods appropriate for lines.  <CODE>LineShape</CODE>
 * wraps another <CODE>Shape</CODE> object, and provides <CODE>contains</CODE> and
 * <CODE>intersects</CODE> methods appropriate for arbitrary line shapes.  These methods break up the
 * line into straight pieces before testing.  This is in contrast to the approach most other
 * <CODE>Shape</CODE> implementations take, which assume that the shape is an enclosed region.</p>
 *
 * <p>The wrapped <CODE>Shape</CODE> object provides the geometry of the line.  <CODE>LineShape</CODE>
 * simply intercepts method calls to the wrapped <CODE>Shape</CODE>.  Clients will typically
 * setup the <CODE>Shape</CODE> object, then provide it to <CODE>LineShape</CODE> and use
 * the <CODE>LineShape</CODE> instance in place of the original <CODE>Shape</CODE>.</p>
 *
 * @author Julie Dickerson <julied@iastate.edu>
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.2 $
 * @since JDK1.3
 */

public class LineShape extends Object implements Shape {
	private Shape _shape = null;
	private double _flatness = 0.0d;
	private float _margin = 0.0f;
	private static float[] B = new float[6];
	private static float[] A = new float[6];
	private static float[] temp = null;
	private static Rectangle2D.Double _rectangle = new Rectangle2D.Double();
	private static final AffineTransform _identity = new AffineTransform();
	
	/** Creates a new <CODE>LineShape</CODE> using the specified <CODE>Shape</CODE>.  The margin is
	 * set to <CODE>5.0f</CODE> and the flatness is set to <CODE>12.0d</CODE>.
	 *
	 * @param shape the <CODE>Shape</CODE> providing the geomoetry for this <CODE>LineShape</CODE>.
	 */
	public LineShape(Shape shape) {
		this(shape, 5.0f, 12.0d);
	}
	
	/** Creates a new <CODE>LineShape</CODE> using the specified <CODE>Shape</CODE>, margin, and
	 * flatness.
	 *
	 * @param shape the <CODE>Shape</CODE> providing the geometry for this <CODE>LineShape</CODE>.
	 * @param margin the maximum distance away from this <CODE>LineShape</CODE> for point containment.
	 * @param flatness the flatness parameter provided to all <CODE>getPathIterator(AffineTransform, double)</CODE>
	 * calls.
	 */	
	public LineShape(Shape shape, float margin, double flatness) {
		if (shape == null) { throw new NullPointerException("shape was null"); }
		if (margin < 0.0f) { throw new IllegalArgumentException("margin must be greater than or equal to zero, not " + margin); }
		
		_shape = shape;
		_margin = margin;
		_flatness = flatness;
	}
	
	/** Tests if the specified point is within the margin of this line.
	 *
	 * @param x the x-coordinate of the point.
	 * @param y the y-coordinate of the point.
	 * @return <CODE>true</CODE> if the specified point is within the margin of this line; <CODE>false otherwise</CODE>.
	 */	
	public boolean contains(double x, double y) {
		boolean contains = false;
		PathIterator pathIterator = null;
		float c = 0.0f, d = 0.0f;
		float x1 = (float) x, y1 = (float) y;
		
		pathIterator = _shape.getPathIterator(_identity, _flatness);
		while (!pathIterator.isDone() && !contains) {
			switch (pathIterator.currentSegment(B)) {
				case PathIterator.SEG_MOVETO:
					//switch the arrays, to avoid having to copy B into A
					temp = A;
					A = B;
					B = temp;
					break;
				case PathIterator.SEG_LINETO:
					//make sure (x,y) is inside the bounding box of the line segment
					if (x1 > (Math.min(A[0],B[0])-_margin) && x1 < (Math.max(A[0],B[0])+_margin) &&
					y1 > (Math.min(A[1],B[1])-_margin) && y1 < (Math.max(A[1],B[1])+_margin)) {
						//test if (x,y) is < _margin away from the line (approximately)
						c = B[0] - A[0];
						d = B[1] - A[1];
						if (Math.abs((y1-A[1])*c - (x1-A[0])*d) / (Math.abs(c) + Math.abs(d) - 0.5f*
						Math.min(Math.abs(c),Math.abs(d))) < _margin) {
							contains = true;
						}
					}
					
					//switch the arrays, to avoid having to copy B into A
					temp = A;
					A = B;
					B = temp;
					break;
				default:
					System.out.println("unsupported PathIterator type in EdgeShape.contains()");
			}
			pathIterator.next();
		}
		
		return contains;
	}
	
	/** Tests if the specified rectangle intersects this <CODE>LineShape</CODE>.  Calling this
	 * method is equivalent to calling the <CODE>intersects(double, double, double, double)</CODE>
	 * method.
	 *
	 * @param x the x-coordinate of the rectangle.
	 * @param y the y-coordinate of the rectangle
	 * @param w the width of the rectangle.
	 * @param h the height of the rectangle.
	 * @return <CODE>true</CODE> if the specified rectangle intersects this <CODE>LineShape</CODE>; <CODE>false</CODE> otherwise.
	 */	
	public boolean contains(double x, double y, double w, double h) {
		boolean contains = false;
		
		return contains;
	}
	
	/** Tests if the specified point is within the margin of this line.
	 *
	 * @param point the point to test.
	 * @return <CODE>true</CODE> if the specified point is within the margin of this line; <CODE>false otherwise</CODE>.
	 */	
	public boolean contains(Point2D point) {
		boolean contains = contains(point.getX(), point.getY());
		
		return contains;
	}
	
	/** Tests if the specified rectangle intersects this <CODE>LineShape</CODE>.  Calling this
	 * method is equivalent to calling the <CODE>intersects(Rectangle2D)</CODE> method.
	 *
	 * @param rectangle the rectangle to test for intersection.
	 * @return <CODE>true</CODE> if the specified rectangle intersects this <CODE>LineShape</CODE>; <CODE>false</CODE> otherwise.
	 */	
	public boolean contains(Rectangle2D rectangle) {
		boolean contains = false;
		
		return contains;
	}
	
	/** Returns an integer <CODE>Rectangle</CODE> that completely encloses the <CODE>Shape</CODE>.
	 *
	 * @return an integer <CODE>Rectangle</CODE> that completely encloses the <CODE>Shape</CODE>.
	 */	
	public Rectangle getBounds() {
		Rectangle bounds = _shape.getBounds();
		
		return bounds;
	}
	
	/** Returns a high precision and more accurate bounding box of the <CODE>Shape</CODE>
	 * than the <CODE>getBounds</CODE> method.
	 *
	 * @return an instance of <CODE>Rectangle2D</CODE> that is a high-precision bounding box of the <CODE>Shape</CODE>.
	 */	
	public Rectangle2D getBounds2D() {
		Rectangle2D bounds = _shape.getBounds2D();
		
		return bounds;
	}
	
	/** Returns an iterator object that iterates along the <CODE>Shape</CODE> boundary and
	 * provides access to the geometry of the <CODE>Shape</CODE> outline.
	 *
	 * @param transform an optional <CODE>AffineTransform</CODE> to be applied to the coordinates as they
	 * are returned in the iteration, or <CODE>null</CODE> if untransformed coordinates
	 * are desired.
	 * @return a new <CODE>PathIterator</CODE> object, which independently traverses the
	 * geometry of the <CODE>Shape</CODE>.
	 */	
	public PathIterator getPathIterator(AffineTransform transform) {
		PathIterator iterator = _shape.getPathIterator(transform);
		
		return iterator;
	}
	
	/** Returns an iterator object that iterates along the <CODE>Shape</CODE> boundary and
	 * provides access to a flattened view of the <CODE>Shape</CODE> outline geometry.
	 *
	 * @param transform an optional <CODE>AffineTransform</CODE> to be applied to the coordinates as
	 * they are returned in the iteration, or <CODE>null</CODE> if untransformed
	 * coordinates are desired.
	 * @param flatness the maximum distance that the line segments used to approximate the curved
	 * segments are allowed to deviate from any point on the original curve.
	 * @return a new <CODE>PathIterator</CODE> that independently traverses the <CODE>Shape</CODE> geometry.
	 */	
	public PathIterator getPathIterator(AffineTransform transform, double flatness) {
		PathIterator iterator = _shape.getPathIterator(transform, flatness);
		
		return iterator;
	}
	
	/** Tests if this <CODE>LineShape</CODE> intersects the specified rectangle.  If the width
	 * and height of the rectangle are both less than 2.0, the rectangle is assumed to represent
	 * a single point, and the <CODE>contains(double, double)</CODE> method is called with the
	 * center point of the rectangle.
	 *
	 * @param x the x-coordinate of the rectangle.
	 * @param y the y-coordinate of the rectangle.
	 * @param w the width of the rectangle.
	 * @param h the height of the rectangle.
	 * @return <CODE>true</CODE> if the specified rectangle intersects this <CODE>LineShape</CODE>; <CODE>false</CODE> otherwise.
	 */	
	public boolean intersects(double x, double y, double w, double h) {
		boolean intersects = false;
		
		_rectangle.setFrame(x, y, w, h);
		intersects = intersects(_rectangle);
		
		return intersects;
	}
	
	/** Tests if this <CODE>LineShape</CODE> intersects the specified rectangle.  If the width
	 * and height of the rectangle are both less than 2.0, the rectangle is assumed to represent
	 * a single point, and the <CODE>contains(double, double)</CODE> method is called with the
	 * center point of the rectangle.
	 *
	 * @param rectangle the rectangle to test for intersection.
	 * @return <CODE>true</CODE> if the specified rectangle intersects this <CODE>LineShape</CODE>; <CODE>false</CODE> otherwise.
	 */	
	public boolean intersects(Rectangle2D rectangle) {
		boolean intersects = false;
		
		PathIterator pathIterator = null;
		
		//if it's a small rectangle, it's a mouse click
		if (rectangle.getWidth() < 2.0d && rectangle.getHeight() < 2.0d) {
			intersects = contains(rectangle.getCenterX(), rectangle.getCenterY());
		}
		//otherwise it's a drag
		else {
			pathIterator = _shape.getPathIterator(_identity, _flatness);
			while (!pathIterator.isDone() && !intersects) {
				switch (pathIterator.currentSegment(B)) {
					case PathIterator.SEG_MOVETO:
						//switch the arrays, to avoid having to copy B into A
						temp = A;
						A = B;
						B = temp;
						break;
					case PathIterator.SEG_LINETO:
						//make sure (x,y) is inside the bounding box of the line segment
						if (rectangle.intersectsLine(A[0], A[1], B[0], B[1])) {
							intersects = true;
						}
						
						//switch the arrays, to avoid having to copy B into A
						temp = A;
						A = B;
						B = temp;
						break;
					default:
						System.out.println("unsupported PathIterator type in EdgeShape.intersects()");
				}
				pathIterator.next();
			}
		}
		
		return intersects;
	}
	
}

/* 
 * $Log: LineShape.java,v $
 * Revision 1.2  2002/01/22 23:01:21  zcox
 * added Revision tag
 *
 * Revision 1.1  2002/01/22 23:00:47  zcox
 * addition of LineShape class
 *
 *
 *
 */