/* $Id: NodeShape.java,v 1.4 2002/01/19 00:31:45 zcox Exp $
 *
 *
 * Created on October 20, 2000, 12:16 AM
 */

package fcmodeler.view.figures;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;

/** <p><CODE>NodeShape</CODE> provides a <CODE>Shape</CODE> object to be used as the outline for a node figure,
 * as well as functionality for positioning, translating, and transforming the <CODE>Shape</CODE>.
 * This interface and the <CODE>NodeFigure</CODE> class define the two main classes in the State
 * design pattern.  <CODE>NodeShape</CODE> represents the State interface, and concrete <CODE>NodeShape</CODE>
 * implementation classes represent different shapes to be used as node figure outlines.</p>
 *
 * <p>Because of the internals of <CODE>NodeFigure</CODE>, <CODE>NodeShape</CODE>
 * must be cloneable.  All <CODE>NodeShape</CODE> implementations should define a
 * <CODE>clone</CODE> method, and implement it as follows (example taken from
 * <CODE>RectangleNodeShape</CODE>):</p>
 *
 * <p><PRE>
 * public Object clone() throws CloneNotSupportedException {
 *  //create the clone
 *  RectangleNodeShape clone = (RectangleNodeShape) super.clone();
 *
 *  //create new copies of any mutable fields
 *  clone._shape = new Rectangle2D.Double(_shape.getX(), _shape.getY(), _shape.getWidth(), _shape.getHeight());
 *
 *  return clone;
 * }
 * </PRE></p>
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.4 $
 * @since JDK1.3
 */

public interface NodeShape extends Cloneable {
	/** Creates and returns a copy of this <CODE>NodeShape</CODE>.  The returned
	 * <CODE>Object</CODE> should be a concrete implementation type of <CODE>NodeShape</CODE>
	 * and also have copies of any mutable fields of this object.
	 *
	 * @throws CloneNotSupportedException should never be thrown.
	 * @return a clone of this <CODE>NodeShape</CODE>.
	 */	
	public Object clone() throws CloneNotSupportedException;
	
	/** Ensures that this <CODE>NodeShape</CODE> completely encloses the specified
	 * rectangle.
	 *
	 * @param rectangle the rectangle to enclose.
	 */	
	public void enclose(Rectangle2D rectangle);
	
	/** Returns the <CODE>Shape</CODE> object represented by this <CODE>NodeShape</CODE>.
	 *
	 * @return the <CODE>Shape</CODE> object represented by this <CODE>NodeShape</CODE>.
	 */	
	public Shape getShape();
	
	/** Positions this <CODE>NodeShape</CODE> at the specified coordinates.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param x the x-coordinate of the new position.
	 * @param y the y-coordinate of the new position.
	 */	
	public void position(double x, double y);
	
	/** Translates this <CODE>NodeShape</CODE> by the specified x and y values.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param dx the amount to translate the x direction.
	 * @param dy the amount to translate in the y direction.
	 */	
	public void translate(double dx, double dy);
	
	/** Transforms this <CODE>NodeShape</CODE> using the specified <CODE>AffineTransform</CODE>.
	 * Note that calling this method may create a new <CODE>Shape</CODE> instance.  Therefore,
	 * a new call to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param transform the <CODE>AffineTransform</CODE> used to transform this <CODE>NodeShape</CODE> object.
	 */	
	public void transform(AffineTransform transform);
}

/* 
 * $Log: NodeShape.java,v $
 * Revision 1.4  2002/01/19 00:31:45  zcox
 * added doc for new clone method
 *
 * Revision 1.3  2002/01/18 19:52:52  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.2  2002/01/15 20:40:06  zcox
 * some changes
 *
 * Revision 1.1  2001/05/22 16:45:02  zcox
 * code clean-up
 *
 * Revision 1.2  2001/04/05 03:01:28  zcox
 * made FCMNodeShape a real flyweight
 *
 * Revision 1.1  2001/04/04 01:33:45  zcox
 * initial commit of some fcmodeler.view.figures classes
 *
 *
 *
 */
