/* $Id: RectangleNodeShape.java,v 1.5 2002/03/22 16:54:30 xein Exp $
 *
 *
 * Created on October 20, 2000, 12:19 AM
 */

package fcmodeler.view.figures;

import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.awt.geom.AffineTransform;

/** A <CODE>NodeShape</CODE> implementation providing a rectangular node shape.
 * <CODE>RectangleNodeShape</CODE> uses a <CODE>Rectangle2D.Double</CODE> as its
 * <CODE>Shape</CODE> object.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.5 $
 * @since JDK1.3
 */

public class RectangleNodeShape extends Object implements NodeShape {
	private Rectangle2D.Double _shape = null;
	private double _space = 10.0d;
	private static final String _string = "rectangle";

	/** Creates a new RectangleNodeShape with coordinates (0,0), zero width, and zero height.
	 *
	 */
	public RectangleNodeShape() {
		_shape = new Rectangle2D.Double();
	}

	/** Creates a new RectangleNodeShape with the specified coordinates, width, and height.
	 *
	 * @param x the x-coordinate of this <CODE>RectangleNodeShape</CODE>.
	 * @param y the y-coordinate of this <CODE>RectangleNodeShape</CODE>.
	 * @param w the width of this <CODE>RectangleNodeShape</CODE>.
	 * @param h the height of this <CODE>RectangleNodeShape</CODE>.
	 */
	public RectangleNodeShape(double x, double y, double w, double h) {
		_shape = new Rectangle2D.Double(x, y, w, h);
	}

	/** Creates and returns a copy of this <CODE>RectangleNodeShape</CODE>.  The returned
	 * <CODE>Object</CODE> is guaranteed to be of type <CODE>RectangleNodeShape</CODE>
	 * and is also guaranteed to have copies of any mutable fields of this object.
	 *
	 * @throws CloneNotSupportedException should never be thrown, as this class extends <CODE>Object</CODE> and implements
	 * <CODE>Cloneable</CODE> by implementing <CODE>NodeShape</CODE>.
	 * @return a clone of this <CODE>RectangleNodeShape</CODE>.
	 */
	public Object clone() throws CloneNotSupportedException {
		//create the clone
		RectangleNodeShape clone = (RectangleNodeShape) super.clone();

		//create new copies of any mutable fields
		clone._shape = new Rectangle2D.Double(_shape.getX(), _shape.getY(), _shape.getWidth(), _shape.getHeight());

		return clone;
	}

	/** Ensures that this <CODE>RectangleNodeShape</CODE> completely encloses the specified
	 * rectangle.
	 *
	 * @param rectangle the rectangle to enclose.
	 */
	public void enclose(Rectangle2D rectangle) {
		_shape.setRect(rectangle.getX()-_space, rectangle.getY()-_space, rectangle.getWidth()+2*_space, rectangle.getHeight()+2*_space);
	}

	/** Returns the <CODE>Shape</CODE> object represented by this <CODE>RectangleNodeShape</CODE>.
	 *
	 * @return the <CODE>Shape</CODE> object represented by this <CODE>RectangleNodeShape</CODE>.
	 */
	public Shape getShape() {
		return _shape;
	}

	/** Positions this <CODE>RectangleNodeShape</CODE> at the specified coordinates.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param x the x-coordinate of the new position.
	 * @param y the y-coordinate of the new position.
	 */
	public void position(double x, double y) {
		_shape.setRect(x, y, _shape.getWidth(), _shape.getHeight());
	}

	public String toString() {
		return _string;
	}

	/** <p>Transforms this <CODE>RectangleNodeShape</CODE> using the specified <CODE>AffineTransform</CODE>.
	 * Note that calling this method may create a new <CODE>Shape</CODE> instance.  Therefore,
	 * a new call to <CODE>getShape</CODE> may be necessary after calling this method.</p>
	 *
	 * <p>This method will throw an <CODE>UnsupportedOperationException</CODE>
	 * until a way to modify a Rectangle2D.Double based on the specified
	 * <CODE>AffineTransform</CODE> is found.</p>
	 *
	 * @param transform the <CODE>AffineTransform</CODE> used to transform this <CODE>RectangleNodeShape</CODE> object.
	 */
	public void transform(AffineTransform transform) {
		//can't do this because the call won't return a Rectangle2D.Double
		//if transforms are going to be used, need to find a way to adjust the
		//Rectangle2D.Double appropriately
		//_shape = transform.createTransformedShape(_shape);

		throw new UnsupportedOperationException("transform not supported yet");
	}

	/** Translates this <CODE>RectangleNodeShape</CODE> by the specified x and y values.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param dx the amount to translate the x direction.
	 * @param dy the amount to translate in the y direction.
	 */
	public void translate(double dx, double dy) {
		_shape.setRect(_shape.getX()+dx, _shape.getY()+dy, _shape.getWidth(), _shape.getHeight());
	}

}

/*
 * $Log: RectangleNodeShape.java,v $
 * Revision 1.5  2002/03/22 16:54:30  xein
 * - Changed NodeShapeEditor and NodeShapeValue to include   DiamondNodeShape
 * - BezierEdgeFigureRenderer was used to test FilledRectangleHead connector   end
 * - No change to RectangleNodeShape
 *
 * Revision 1.4  2002/02/15 20:21:18  zcox
 * big time update
 *
 * Revision 1.3  2002/01/18 19:52:52  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.2  2002/01/15 20:40:06  zcox
 * some changes
 *
 * Revision 1.1  2001/05/22 16:45:01  zcox
 * code clean-up
 *
 * Revision 1.2  2001/04/05 03:01:29  zcox
 * made FCMNodeShape a real flyweight
 *
 * Revision 1.1  2001/04/04 01:33:45  zcox
 * initial commit of some fcmodeler.view.figures classes
 *
 *
 *
 */
