/* $Id: RoundRectangleNodeShape.java,v 1.4 2002/02/15 20:21:18 zcox Exp $
 *
 *
 * Created on October 20, 2000, 12:44 AM
 */

package fcmodeler.view.figures;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;


/** A <CODE>NodeShape</CODE> implementation providing a round rectangular node shape.
 * <CODE>RoundRectangleNodeShape</CODE> uses a <CODE>RoundRectangle2D.Double</CODE> as its
 * <CODE>Shape</CODE> object.
 *
 * @author Zach Cox <zcox@iastate.edu>
 * @version $Revision: 1.4 $
 * @since JDK1.3
 */

public class RoundRectangleNodeShape extends Object implements NodeShape {
	private RoundRectangle2D.Double _shape = null;
	private double _space = 10.0d;
	private double _arcWFraction = 0.75d;
	private double _arcHFraction = 0.75d;
	private static final String _string = "round rectangle";
	
	/** Creates a new RoundRectangleNodeShape with coordinates (0,0), zero width, and zero height.
	 *
	 */
	public RoundRectangleNodeShape() {
		_shape = new RoundRectangle2D.Double();
	}
	
	/** Creates a new RoundRectangleNodeShape with the specified coordinates, width, and height.
	 *
	 * @param x the x-coordinate of this <CODE>RoundRectangleNodeShape</CODE>.
	 * @param y the y-coordinate of this <CODE>RoundRectangleNodeShape</CODE>.
	 * @param w the width of this <CODE>RoundRectangleNodeShape</CODE>.
	 * @param h the height of this <CODE>RoundRectangleNodeShape</CODE>.
	 */
	public RoundRectangleNodeShape(double x, double y, double w, double h) {
		_shape = new RoundRectangle2D.Double(x, y, w, h, w*_arcWFraction, h*_arcHFraction);
	}
	
	/** Creates and returns a copy of this <CODE>RoundRectangleNodeShape</CODE>.  The returned
	 * <CODE>Object</CODE> is guaranteed to be of type <CODE>RoundRectangleNodeShape</CODE>
	 * and is also guaranteed to have copies of any mutable fields of this object.
	 *
	 * @throws CloneNotSupportedException should never be thrown, as this class extends <CODE>Object</CODE> and implements
	 * <CODE>Cloneable</CODE> by implementing <CODE>NodeShape</CODE>.
	 * @return a clone of this <CODE>RoundRectangleNodeShape</CODE>.
	 */	
	public Object clone() throws CloneNotSupportedException {
		//create the clone
		RoundRectangleNodeShape clone = (RoundRectangleNodeShape) super.clone();
		
		//create new copies of any mutable fields
		clone._shape = new RoundRectangle2D.Double(_shape.getX(), _shape.getY(), _shape.getWidth(), _shape.getHeight(), 
			_shape.getArcWidth(), _shape.getArcHeight());
		
		return clone;
	}
	
	/** Ensures that this <CODE>RoundRectangleNodeShape</CODE> completely encloses the specified
	 * rectangle.
	 *
	 * @param rectangle the rectangle to enclose.
	 */
	public void enclose(Rectangle2D rectangle) {
		_shape.setRoundRect(rectangle.getX()-_space, rectangle.getY()-_space,
			rectangle.getWidth()+2*_space, rectangle.getHeight()+2*_space,
			rectangle.getWidth()*_arcWFraction, rectangle.getHeight()*_arcHFraction);
	}
	
	/** Returns the <CODE>Shape</CODE> object represented by this <CODE>RoundRectangleNodeshape</CODE>.
	 *
	 * @return the <CODE>Shape</CODE> object represented by this <CODE>RoundRectangleNodeshape</CODE>.
	 */	
	public Shape getShape() {
		return _shape;
	}
	
	/** Positions this <CODE>RoundRectangleNodeshape</CODE> at the specified coordinates.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param x the x-coordinate of the new position.
	 * @param y the y-coordinate of the new position.
	 */	
	public void position(double x, double y) {
		_shape.setRoundRect(x, y, _shape.getWidth(), _shape.getHeight(), 
			_shape.getWidth()*_arcWFraction, _shape.getHeight()*_arcHFraction);
	}
	
	public String toString() {
		return _string;
	}
	
	/** <p>Transforms this <CODE>RoundRectangleNodeshape</CODE> using the specified <CODE>AffineTransform</CODE>.
	 * Note that calling this method may create a new <CODE>Shape</CODE> instance.  Therefore,
	 * a new call to <CODE>getShape</CODE> may be necessary after calling this method.</p>
	 *
	 * <p>This method will throw an <CODE>UnsupportedOperationException</CODE>
	 * until a way to modify a Rectangle2D.Double based on the specified
	 * <CODE>AffineTransform</CODE> is found.</p>
	 *
	 * @param transform the <CODE>AffineTransform</CODE> used to transform this <CODE>RoundRectangleNodeshape</CODE> object.
	 */	
	public void transform(AffineTransform transform) {
		//can't do this because the call won't return a Rectangle2D.Double
		//if transforms are going to be used, need to find a way to adjust the
		//Rectangle2D.Double appropriately
		//_shape = transform.createTransformedShape(_shape);
		
		throw new UnsupportedOperationException("transform not supported yet");
	}
	
	/** Translates this <CODE>RoundRectangleNodeshape</CODE> by the specified x and y values.  Note that calling
	 * this method may create a new <CODE>Shape</CODE> instance.  Therefore, a new call
	 * to <CODE>getShape</CODE> may be necessary after calling this method.
	 *
	 * @param dx the amount to translate the x direction.
	 * @param dy the amount to translate in the y direction.
	 */	
	public void translate(double dx, double dy) {
		_shape.setRoundRect(_shape.getX()+dx, _shape.getY()+dy, _shape.getWidth(), _shape.getHeight(), 
			_shape.getWidth()*_arcWFraction, _shape.getHeight()*_arcHFraction);
	}
	
}

/* 
 * $Log: RoundRectangleNodeShape.java,v $
 * Revision 1.4  2002/02/15 20:21:18  zcox
 * big time update
 *
 * Revision 1.3  2002/01/18 19:52:52  zcox
 * changes to NodeFigure and NodeShape architecture
 *
 * Revision 1.2  2002/01/15 20:40:06  zcox
 * some changes
 *
 * Revision 1.1  2001/05/22 16:45:01  zcox
 * code clean-up
 *
 * Revision 1.2  2001/04/05 03:01:29  zcox
 * made FCMNodeShape a real flyweight
 *
 * Revision 1.1  2001/04/04 01:33:45  zcox
 * initial commit of some fcmodeler.view.figures classes
 *
 *
 *
 */
